/**
 * Approve Button View
 *
 * @since 1.0.0
 */
Huddle.ApproveButtonView = Huddle.View.extend({
    template: wp.template('approve-button'),
    tagName: 'button',

    /**
     * Set Button attributes
     * @returns {object} attributes to add to the button
     */
    attributes: function () {
        return {
            class: this.classes(),
            'data-mode': 'bottom',
            'data-tip': this.dataTip()
        };
    },

    /**
     * Create class attribute
     *
     * @returns {string}
     */
    classes: function () {
        var classes = 'ph-approve-button ph-approve-image ';
        classes += this.model.get('approved') ? 'ph-approved' : 'ph-pending';
        return classes;
    },

    /**
     * Create tooltip attribute
     *
     * @returns {string}
     */
    dataTip: function () {
        if (this.model.get('approved')) {
            return Huddle.translations.unapprove_image ? Huddle.translations.unapprove_image : 'UnApprove Image';
        } else {
            return Huddle.translations.unapprove_image ? Huddle.translations.approve_image : 'Approve Image';
        }
    },

    events: {
        'click': 'approve'
    },

    identityCheck: false,

    initialize: function () {
        this.listenTo(this.model, 'change:approved', this.changeApproval);
        this.listenTo(this.model, 'approveImage', this.addLoading);
        this.listenTo(this.model, 'identifySuccess', this.doApproval);

        // broadcast init
        wp.hooks.doAction('Huddle.ApproveButtonView.initialize', this);
    },

    /**
     * Adds loading class to button
     */
    addLoading: function () {
        this.$el.addClass('ph-spinner ph-spinner-dark');
    },

    /**
     * Approval change handler
     */
    changeApproval: function () {
        this.renderTransition(_.bind(this.swapApproval, this));
        if ( this.model.get('approved') ) {
            toastr.success(Huddle.translations.approved);
        } else {
            toastr.info(Huddle.translations.unapproved);
        }
    },

    doApproval: function () {
        this.identityChecking && this.approve();
    },

    /**
     * Make transition smooth
     *
     * @param callback What to change before smooth transition
     */
    renderTransition: function (callback) {
        // store original width
        this.$el.data('oWidth', this.$el.width());

        // callback function
        callback && callback();

        // find new width and store in data
        this.$el.css('width', 'auto').data('nWidth', this.$el.outerWidth());

        // set old width
        this.$el.width(this.$el.data('oWidth'));

        // animate new width and re-render
        this.$el.animate({width: this.$el.data('nWidth')}, 500, 'easeOutCubic', _.bind(function () {
            this.$el.css({'width': 'auto'}).removeClass('width-transition spinner');
            this.render();
        }, this));
    },

    /**
     * Changes button approval html
     */
    swapApproval: function () {
        if (this.model.get('approved')) {
            this.$el.addClass('ph-approved');
            // this.$el.removeClass('ph-pending');
            this.$('.ph-button-text').html(Huddle.translations.unapprove);
        } else {
            // this.$el.addClass('ph-pending');
            this.$el.removeClass('ph-approved');
            this.$('.ph-button-text').html(Huddle.translations.approve);
        }

        // update attributes
        this.$el.attr(_.extend({}, _.result(this, 'attributes')));
    },

    /**
     * Change approval in model
     */
    approve: function () {
        if (!Huddle.permissions.get('allowComments')) {
            toastr.info(this.model.get('approved') ? Huddle.translations.cant_approve : Huddle.translations.cant_approve);
            return false;
        }

        // if it's approved and unapprovals aren't allowed, do nothing
        if (this.model.get('approved') && !Huddle.permissions.get('allowUnApprovals')) {
            toastr.info(Huddle.translations.cant_unapprove);
            return false;
        }

        // if it's approved and unapprovals aren't allowed, do nothing
        if (!this.model.get('approved') && !Huddle.permissions.get('allowApprovals')) {
            toastr.info(Huddle.translations.cant_approve);
            return false;
        }

        // validate user data
        if (!Huddle.user.attributes.user_email) {
            this.identityChecking = true;
            Huddle.Event.trigger('requireIdentity', this.model);
            return false;
        }

        // confirm approval
        this.confirmApproval(false, _.bind(this.approveWhich, this));
    },

    approveWhich: function (options, value) {
        var approval = !this.model.get('approved');

        if (value.approve_project) {
            // save in db
            Huddle.container.model.save({
                'approved': approval
            }, {
                url: Huddle.container.model.urlRoot + '/' + this.model.get('id') + '/approve'
            });
            // change all images in collection
            this.model.collection.forEach(_.bind(function (model, index) {
                model.set('approved', approval);
            }, this));
        } else {
            this.model.save({
                'approved': approval
            }, {
                url: this.model.urlRoot + '/' + this.model.get('id') + '/approve'
            });
        }
    },

    /**
     * Confirm approval dialog
     *
     * @param options Callback options
     * @param callback Callback function
     */
    confirmApproval: function (options, callback) {
        var t             = Huddle.translations,
            approveText   = this.model.get('approved') ? t.unapprove : t.approve,
            approveString = this.model.get('approved') && t ? t.confirmUnApprove : t.confirmApprove;

        // add image name
        var tags = {
            approval_status: approveText.toLowerCase(),
            image_number: this.model.collection.length - 1,
            image_name: this.model.get('title'),
            project_name: Huddle.container.model.get('name'),
            terms: '<u class="toggle-terms">' + t.approveCheckboxLink + '</u>',
            user_name: '<strong>' + Huddle.user.get('display_name') + '</strong>'
        };

        // replace any tags with variables
        var approveReplaced = approveString.replace(/\{\{(.*?)\}\}/g, function (match, token) {
            return tags[token];
        });

        // replace any tags with variables
        var termsReplaced = t.approveCheckbox.replace(/\{\{(.*?)\}\}/g, function (match, token) {
            return tags[token];
        });

        // replace any tags with variables
        var projectReplaced = t.approveProject.replace(/\{\{(.*?)\}\}/g, function (match, token) {
            return tags[token];
        });

        var vexOptions = {
            className: 'vex-theme-dialog vex-left-buttons',
            showCloseButton: true,
            message: approveReplaced,
            afterOpen: function ($vexContent) {
                $vexContent.find('.toggle-terms').click(function () {
                    jQuery('.ph-terms').slideToggle('fast');
                });
                $vexContent.find('.approve-option').appendTo('.vex-dialog-buttons');
            },
            callback: function (value) {
                if (!value) return false;
                callback && callback(options, value);
            }
        };

        // options
        var checked = Huddle.options.global.approval_type === 'project' ? 'checked' : '';
        var hidden  = Huddle.options.global.approval_type === 'project' ? 'ph-hidden' : '';

        vexOptions.input = '';
        if ( this.model.collection.length > 1 ) {
            vexOptions.input += "<div class='approve-option " + hidden +"'><input type='checkbox' name='approve_project' " + checked + "/><label>" + projectReplaced + "</label></div>";
        }
        if (parseInt(Huddle.options.global.require_terms)) {
            vexOptions.input += "<div class='terms'><input type='checkbox' required name='terms' /><label>" + termsReplaced + "</label></div>";
            vexOptions.input += "<div class='ph-terms'>" + t.approveTerms + "</div>";
        }

        // confirm approval
        if ( this.model.collection.length > 1 && Huddle.options.global.approval_type !== 'project' ) {
            vex.dialog.alert(vexOptions);
        } else {
            vex.dialog.open(vexOptions);
        }
    }
});