<?php
/**
 * AJAX Functions
 *
 * Process the front-end AJAX actions.
 *
 * @package     PH Image Versions
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Admin_Project_Images Class
 *
 * This class handles the all the ajax requests made by the plugin
 *
 * @since 1.0.0
 */
class PHA_Ajax_Events {

	private $user;

	/**
	 * Setup project admin
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// register ajax actions
		$this->add_ajax_events();
	}

	public function add_ajax_events() {

		// event => nopriv
		$ajax_events = array(
			'ph_approve_image' => true
		);

		// register each event
		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_' . $ajax_event, array( $this, $ajax_event ) );

			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_' . $ajax_event, array( $this, $ajax_event ) );
				// phv AJAX can be used for frontend ajax requests
				add_action( 'phv_ajax_' . $ajax_event, array( $this, $ajax_event ) );
			}
		}

	}

	/**
	 * Verify nonce for all functions
	 *
	 * @since 1.0.0
	 *
	 * @param $post_data $_POST data
	 */
	public function verify_nonce( $post_data ) {

		// make sure post is set
		if ( empty( $post_data ) ) {
			error_log( 'ProjectHuddle: verify_nonce returned an error. $post_data is empty.' );
			wp_send_json_error( __( 'Hmmmm... something went wrong. Please try again.', 'project-huddle' ) );
		}

		// verify nonce
	    if ( ! wp_verify_nonce( $post_data['nonce'], 'ph-approval-nonce' ) ) {
		    wp_send_json_error( __( 'Your session expired. Please reload the page again.', 'project-huddle' ) );
	    }

	}

	/**
	 * Checks the identity of the ajax poster
	 *
	 * @param $post_data {array} $_POST data from ajax request
	 *
	 * @since 1.0.0
	 */
	public function checkIdentity( $post_data ) {

		// get user email and image id
		$user_email = (array) $post_data['user'];

		// validate post ID with server
		$project_id = (int) url_to_postid( wp_get_referer() );

		// sanitize
		$user_email['email'] = esc_attr( $user_email['email'] );

		// check if non-logged in comments allowed
		$allow_comments = get_post_meta( $project_id, 'ph_project_comments', true );

		// check approvals
		$allow_approvals = get_post_meta( $project_id, 'ph_project_approval', true );
		$allow_unapprovals = get_post_meta( $project_id, 'ph_project_unapproval', true );

		if ( ! $project_id ) {
			error_log( 'ProjectHuddle: Tried to check identity but the post ID was not defined. /n $project_id=' . $project_id );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// check user has an identity
		if ( ! $user_email['email'] || $user_email['email'] == 'false' ) {
			wp_send_json_error( __( 'You must identify yourself! Please login or add a comment first.', 'project-huddle' ) );
		}

		// validate email address
		if ( ! filter_var( $user_email['email'], FILTER_VALIDATE_EMAIL ) ) {
			wp_send_json_error( __( 'Your email is not valid. Please reload the page and try again.', 'project-huddle' ) );
		}

		// if public comments aren't allowed
		if ( $allow_comments != 'on' ) {
			// user should be logged in
			if ( ! is_user_logged_in() ) {
				wp_send_json_error( __( 'You need a user account to do that. Please login or signup.', 'project-huddle' ) );
			}
			// user should be admin
			if ( ! is_admin() ) {
				wp_send_json_error( __( 'Sorry! Only administrators are allowed to do that.', 'project-huddle' ) );
			}
		}

		// allow users with comment moderation permissions to always change approvals
		if ( ! current_user_can( 'edit_ph-project', $project_id ) ) {

			// if approvals are not allowed and model is trying to be approved
			if ( $allow_approvals == 'off' && $post_data['model']['approved'] == 'false' ) {
				wp_send_json_error( __( 'You\'re not allowed to approve this image .', 'project-huddle' ) );
			}

			// if approvals are not allowed and model is trying to be approved
			if ( $allow_unapprovals == 'off' && $post_data['model']['approved'] == 'true' ) {
				wp_send_json_error( __( 'You\'re not allowed to unapprove this image .', 'project-huddle' ) );
			}
		}

	}

	/**
	 * Sets approval of image
	 *
	 * @since 1.0.0
	 *
	 * @uses do_action() on ph_set_image_approval_before before approval is set
	 * @uses do_action() on ph_set_image_approval_after after approval is set
	 *
	 * @return object Success/Fail json object with updated user model
	 */
	public function ph_approve_image() {
		// current user
		global $current_user;

		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		// Get model
		$model      = (array) $_POST['model'];
		$user       = (array) $_POST['user'];
		$image_id   = (int) $_POST['image_id'];

		// validate post ID with server
		$project_id = (int) url_to_postid( wp_get_referer() );

		// destringify :(
		if ( $model['approved'] == 'false' ) {
			$model['approved'] = false;
		}

		if ( $model['approved'] == 'true' ) {
			$model['approved'] = true;
		}

		// switch
		$model['approved'] = ! (boolean) $model['approved'];

		do_action( 'ph_set_image_approval_before', $model );

		// get current user
		$current_user = wp_get_current_user();

		// find last approval
		$last_approval = ph_get_latest_approval( (int) $model['ID'] );

		// automatically update model, don't store approval if they're the same
		if ( $last_approval == (boolean) $model['approved'] ) {
			wp_send_json( $model );
		}

		// add latest approval to chain
		$approval[] = array(
			'approval' => (boolean) $model['approved'],
			'by'       => array(
				'email'        => esc_attr( $user['email'] ),
				'display_name' => esc_attr( $user['display_name'] ),
				'wp_user'      => $current_user ? $current_user : false
			),
			'on'       => date( "Y-m-d H:i:s" )
		);

		// store approval in post meta
		update_post_meta( (int) $model['ID'], 'ph_approved', $approval );

		// send email via ph email function
		ph_email_image_approved( $user, $model, $image_id, $project_id );

		// add email to enabled
		PH()->events->add_email( $user, $project_id );

		// image approval action
		do_action( 'ph_set_image_approval_after', $model, $user, $project_id );

		// send update, no need to update the model
		wp_send_json( $model );
	}
}