<?php
/**
 * Extend Project Data to include approvals
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.2.2
 */
class PH_Project_Approval extends PH_Project {
	/**
	 * Register actions
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// clear transients when post meta is updated
		add_action( 'added_post_meta', array( $this, 'clear_approval_transients' ), 10, 4 );
		add_action( 'updated_post_meta', array( $this, 'clear_approval_transients' ), 10, 4 );

		// call parents constructor
		parent::__construct();
	}

	/**
	 * Checks all images in a project to see if project is approved
	 *
	 * @param int $id ID of the project
	 *
	 * @return bool True/false if project is approved
	 */
	public function is_approved( $id = 0 ) {
		$total           = PH()->project->total_images( $id );
		$approved        = PH_Approvals()->project->total_approved_images( $id );

		// if approved if approved images are more or equal to total
		return $total && $approved >= $total ? true : false;
	}

	/**
	 * Checks all images in a project to see if project is approved
	 *
	 * @param int $id ID of the project
	 *
	 * @uses transient to store query to speed up page load
	 * @return array|bool Latest approved image data
	 */
	public function approval( $id = 0 ) {
		$last = get_transient( 'ph_approved_project_' . $id );

		// this code runs when there is no valid transient set
		if ( false === $last ) {
			$time = 0;
			$last = array();

			foreach ( (array) PH()->project->image_ids( (int) $id ) as $key => $image ) {
				$approval = PH_Approvals()->image->last_approval( $image );

				if ( isset($approval['on']) && $approval['on'] ) {
					$on = new DateTime($approval['on']);
					$on = $on->format('d/m/Y');
					if ( $on > $time ) {
						$last = $approval;
						$time = $on;
					}
				}
			}

			// set last approval
			$last['approval'] = $this->is_approved( $id ) ? true : false;

			set_transient( 'ph_approved_project_' . $id, $last, 30 * DAY_IN_SECONDS ); // expires in 1 month
		}

		// return last approval data
		return $last;
	}

	/**
	 * Clears transient when approval meta is updated
	 *
	 * @param $meta_id    int       ID of meta value
	 * @param $post_id    int       Post ID
	 * @param $meta_key   string    Meta key
	 * @param $meta_value mixed     Value of meta
	 */
	public function clear_approval_transients( $meta_id, $post_id, $meta_key, $meta_value ) {
		if ( get_post_type( $post_id ) != 'project_image' ) {
			return;
		}
		$project = PH()->image->get_project( $post_id );

		delete_transient( 'ph_project_approved_' . $project );
		delete_transient( 'ph_approved_image_ids_' . $project );
		delete_transient( 'ph_approved_project_' . $project );
	}

	/**
	 * Returns approved images
	 *
	 * @param $id int Project ID
	 *
	 * @uses transient to store query to speed up page load
	 *
	 * @return array
	 */
	public function approved_image_ids( $id ) {
		$image_ids = get_transient( 'ph_approved_image_ids_' . $id );

		// this code runs when there is no valid transient set
		if ( false === $image_ids ) {
			$image_ids = PH()->project->image_ids( $id );

			if ( ! empty( $image_ids ) ) {
				foreach ( $image_ids as $key => $image ) {
					if ( ! PH_Approvals()->image->is_approved( $image ) ) {
						unset( $image_ids[ $key ] );
					}
				}
			}
			set_transient( 'ph_approved_image_ids_' . $id, $image_ids, 30 * DAY_IN_SECONDS ); // expires in 1 month
		}

		return $image_ids;
	}

	/**
	 * Get total approved images count
	 *
	 * @param $id integer Project ID
	 *
	 * @return int Number of approved images
	 */
	public function total_approved_images( $id ) {
		$approved = $this->approved_image_ids( $id );
		return is_array( $approved ) ? count( $approved ) : 0;
	}
}