<?php
/**
 * Plugin Name: ProjectHuddle - Image Approvals
 * Plugin URL: http://projecthuddle.io
 * Description: Adds an approval system for projecthuddle images
 * Version: 1.0.0
 * Author: Andre Gagnon
 * Author URI: http://projecthuddle.io
 * Text Domain: project-huddle
 * Domain Path: languages
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Setup Constants before init because we're running plugin on plugins_loaded
 *
 * @since 1.1.1
 */

// Plugin Folder Path
if ( ! defined( 'PH_APPROVALS_PLUGIN_DIR' ) ) {
	define( 'PH_APPROVALS_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
}

// Plugin Folder URL
if ( ! defined( 'PH_APPROVALS_PLUGIN_URL' ) ) {
	define( 'PH_APPROVALS_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
}

// Plugin Root File
if ( ! defined( 'PH_APPROVALS_PLUGIN_FILE' ) ) {
	define( 'PH_APPROVALS_PLUGIN_FILE', __FILE__ );
}

if ( ! class_exists( 'PH_Image_Approvals' ) ) :
	/**
	 * Main PH_Image_Approvals Class
	 * Uses singleton design pattern
	 *
	 * @since 1.0.0
	 */
	final class PH_Image_Approvals {

		/**
		 * Holds only one PH_Image_Approvals instance
		 * @var $instance
		 * @since 1.0
		 */
		private static $instance;

		/**
		 * Main PH_Image_Approvals Instance
		 *
		 * Insures that only one instance of PH_Image_Approvals exists in memory at any one
		 * time. Also prevents needing to define globals all over the place.
		 *
		 * @since  1.0.0
		 * @static var array $instance
		 * @uses   PH_Image_Approvals::setup_constants() Setup the constants needed
		 * @uses   PH_Image_Approvals::includes() Include the required files
		 * @uses   PH_Image_Approvals::load_textdomain() load the language files
		 * @see    PH()
		 * @return PH_Image_Approvals $instance The one true PH_Image_Approvals
		 */
		public static function instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof PH_Image_Approvals ) ) {

				// do nothing if ProjectHuddle is not activated
				if( ! class_exists( 'Project_Huddle', false ) ) {
					return false;
				}

				// make sure we have the correct minimum version
				if ( ! defined( 'PH_VERSION' ) || version_compare( PH_VERSION, '1.1.0' ) < 0 ) {
					add_action( 'admin_notices', array( self::$instance, 'update_required' ) );
					return false;
				}

				self::$instance = new PH_Image_Approvals;

				// load includes
				self::$instance->includes();

				// ajax events
				self::$instance->project = new PH_Project_Approval();
				self::$instance->image   = new PH_Image_Approval();
				self::$instance->events  = new PHA_Ajax_Events();
				self::$instance->api     = new PHA_Api();
				self::$instance->actions = new PHA_Approval_Actions();

				// Loaded action
				do_action( 'projecthuddle_approvals_loaded' );
			}

			return self::$instance;

		}

		/**
		 * Throw error on object clone
		 *
		 * The whole idea of the singleton design pattern is that there is a single
		 * object therefore, we don't want the object to be cloned.
		 *
		 * @since 1.0.0
		 *
		 * @uses _doing_it_wrong() Mark something as being incorrectly called.
		 *
		 * @access public
		 * @return void
		 */
		public function __clone() {
			// Cloning instances of the class is forbidden
			_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'project-huddle' ), '1.0.0' );
		}

		/**
		 * Disable un-serializing of the class
		 *
		 * @since 1.0.0
		 *
		 * @uses _doing_it_wrong() Mark something as being incorrectly called.
		 *
		 * @access public
		 * @return void
		 */
		public function __wakeup() {
			// Un-serializing instances of the class is forbidden
			_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'project-huddle' ), '1.0.0' );
		}

		/**
		 * Show notice if ProjectHuddle needs updating
		 *
		 * @since 1.0.0
		 *
		 * @access protected
		 * @return string
		 */
		public function update_required() {
			?>
			<div class="update-nag">
				<p><?php _e( 'The ProjectHuddle approvals plugin requires the ProjectHuddle plugin to be at least version 1.0.2. Please update your ProjectHuddle plugin.', 'project-huddle' ); ?></p>
			</div>
			<?php
		}

		/**
		 * Include required files
		 *
		 * @access private
		 * @since 1.0.0
		 * @return void
		 */
		private function includes() {

			// classes
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/class-pha-project.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/class-pha-image.php';

			// api
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/api/class-ph-approvals-api.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/api/class-ph-approvals-actions.php';

			// functions
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/misc-functions.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/shortcodes.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/translations.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/template-functions.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/style-functions.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/project-functions.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/email/email-functions.php';

			// admin
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/admin/settings.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/admin/meta-boxes.php';
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/admin/post-type-ui.php';

			// scripts
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/scripts.php';

			// ajax
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/class-pha-ajax.php';

			// templates
			require_once PH_APPROVALS_PLUGIN_DIR . 'includes/templates.php';
		}
	}
endif;

/**
 * The main function responsible for returning the one true PH_Approvals
 * Instance to functions everywhere.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * Example: <?php $approvals = PH_Approvals(); ?>
 *
 * @since 1.0.0
 * @return object The one true PH_Approvals Instance
 */
function PH_Approvals() {
	return PH_Image_Approvals::instance();
}
add_action( 'plugins_loaded', 'PH_Approvals' );