/**
 * Mixins used in the app. Used to sync admin-ajax.php with backbone
 *
 * @package     ProjectHuddle
 * @subpackage  js
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

/**
 * A mixin for collections/models that syncs with admin-ajax.php
 *
 * @since 1.0.0
 */
var phVersionsAdminAjaxMixin = {

	// admin-ajax.php url
	url:  phVersion.ajaxurl,

	// sync function
	sync: function (method, object, options) {

		// check if data is provided
		if (typeof options.data === 'undefined') {
			options.data = {};
		}

		// send nonce and method
		options.data.nonce = phVersion.phNonce;
		options.data.action_type = method;

		// send current project ID
		options.data.project_id = Huddle.projectID ? Huddle.projectID : false;

		// send current user info
		options.data.user = Huddle.user ? {
			'email': Huddle.user.attributes.user_email,
			'display_name': Huddle.user.attributes.display_name
		} : false;

		// set default action if nothing is set
		if (typeof options.data.action === 'undefined') {
			options.data.action = 'no_action_defined';
		}

		/**
		 * Convert model or collection to json
		 */
		var json = this.toJSON();
		var formattedJSON = {};

		/**
		 * If the json is an array, it's a collection
		 * Otherwise, its a model
		 */
		if (json instanceof Array) {
			formattedJSON.models = json;
		} else {
			formattedJSON.model = json;
		}

		/**
		 * Extend the options.data object with the formatted json
		 */
		_.extend(options.data, formattedJSON);

		/**
		 * Prevent needing to json decode in php and use $_POST instead
		 */
		options.emulateJSON = true;

		/**
		 * Call Backbone sync method
		 */
		return Backbone.sync.call(this, 'create', object, options);
	}
};

/**
 * A model for all the syncable models to extend
 *
 * Adds toast errors if they exist
 *
 * @since 1.0.0
 */
var VersionsModel = Backbone.Model.extend( _.defaults( {
	/**
	 * Extend constructor
	 */
	constructor: function() {
		Backbone.Model.apply(this, arguments);
	},

	/**
	 * Show errors if there are any
	 *
	 * @param response {object} Ajax response
	 */
	parse: function ( response ) {
		// Show our errors
		if ( response.success == false ) {

			// log error
			console.error( response );

			// trigger WordPress error if it exists
			if ( typeof response.data == 'string' ) {
				alert( response.data );
			}
			return false;

		} else {
			return response;
		}
	},

	/**
	 * Makes for easier ajax functions and triggers
	 *
	 * @param model             {object}    Model to run function
	 * @param callback          {function}  Function to run
	 * @param trigger           {string}    Event to trigger on Success
	 * @param failObjectTrigger {string}    Event to trigger if failure is object
	 */
	doAction: function( model, callback, trigger, failObjectTrigger ) {
		model[callback]( undefined, {
			success: function ( model, response, options ) {
				// handle success
				if ( response.success != false ) {
					model.trigger( trigger, model );
					// global event too
					Huddle.Event && Huddle.Event.trigger(  trigger, model  );
				} else {
					// if response is new object
					if ( typeof response.data == 'object' ) {
						failObjectTrigger && model.trigger( failObjectTrigger, response.data );
					}
				}
			},
			error: function ( model, xhr, options ) {
				console.error( xhr ); // don't output error for user
				alert( 'Something went wrong. Please try again.' );
			}
		} );
	}
}, phVersionsAdminAjaxMixin ) );

/**
 * A collection for all the syncable collections to extend
 *
 * @since 1.0.0
 */
var VersionsCollection = Backbone.Collection.extend( _.defaults( {
	/**
	 * Show errors if there are any
	 *
	 * @param response {object} Ajax response
	 */
	parse: function ( response ) {
		// Show our errors
		if ( response.success == false ) {

			// log error
			console.error( response );

			// trigger WordPress error if it exists
			if ( typeof response.data == 'string' ) {
				alert( response.data );
			}

			return false;

		} else {
			// trigger WordPress error if it exists
			return response;
		}
	},

	/**
	 * Makes for easier ajax functions and triggers
	 *
	 * @param model             {object}    Model to run function
	 * @param callback          {function}  Function to run
	 * @param trigger           {string}    Event to trigger on Success
	 * @param failObjectTrigger {string}    Event to trigger if failure is object
	 */
	doAction: function( model, callback, trigger, failObjectTrigger ) {
		model[callback]( undefined, {
			success: function ( model, response, options ) {
				// handle success
				if ( response.success != false ) {
					model.trigger( trigger, model );

					// global event too
					Huddle.Event && Huddle.Event.trigger(  trigger, model  );
				} else {
					// if response is new object
					if ( typeof response.data == 'object' ) {
						failObjectTrigger && model.trigger( failObjectTrigger, response.data );
					}
				}
			},
			error: function ( model, xhr, options ) {
				console.error( xhr ); // don't output error for user
				alert( 'Something went wrong. Please try again.' );
			}
		} );
	}
}, phVersionsAdminAjaxMixin ) );

/**
 * A view for all wp views to extend
 *
 * Extends wp.Backbone.View with .prepare() and .inject()
 *
 * @since 1.0.0
 */
Huddle.VersionsView = wp.Backbone.View.extend({
	inject: function( selector ) {
		this.render();
		jQuery(selector).html( this.el );
		this.views.ready();
	},
	injectTo: function( selector ) {
		this.render();
		jQuery(selector).append( this.el );
		this.views.ready();
	},
	prepare: function() {
		if ( ! _.isUndefined( this.model ) && _.isFunction( this.model.toJSON ) ) {
			this._parseTriggers();
			return this.model.toJSON();
		} else {
			return {};
		}
	},

	_parseTriggers: function () {
		if ( this.triggers ) {
			_.each( this.triggers || {}, function ( fn, event ) {
				var handler = this[fn];
				if ( _.isFunction( handler ) ) {
					this.stopListening( this.model, event, handler );
					this.listenTo( this.model, event, handler );
				}
			}, this );
		}
	}
});