<?php
/**
 * AJAX Functions
 *
 * Process the front-end AJAX actions.
 *
 * @package     PH Image Versions
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Admin_Project_Images Class
 *
 * This class handles the all the ajax requests made by the plugin
 *
 * @since 1.0.0
 */
class PHV_Ajax_Events {

	/**
	 * Setup project admin
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// register ajax actions
		$this->add_ajax_events();
	}

	public function add_ajax_events() {

		// event => nopriv
		$ajax_events = array(
			'phv_fetch_versions' => true
		);

		// register each event
		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_' . $ajax_event, array( $this, $ajax_event ) );

			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_' . $ajax_event, array( $this, $ajax_event ) );
				// phv AJAX can be used for frontend ajax requests
				add_action( 'phv_ajax_' . $ajax_event, array( $this, $ajax_event ) );
			}
		}

	}

	/**
	 * Verify nonce for all functions
	 *
	 * @since 1.0.0
	 *
	 * @param $post_data $_POST data
	 */
	public function verify_nonce( $post_data ) {

		// make sure post is set
		if ( empty( $post_data ) ) {
			error_log( 'ProjectHuddle: verify_nonce returned an error. $post_data is empty.' );
			wp_send_json_error( __( 'Hmmmm... something went wrong. Please try again.', 'project-huddle' ) );
		}
		// verify nonce
	    if ( ! wp_verify_nonce( $post_data['nonce'], 'ph-version-nonce' ) ) {
		    wp_send_json_error( __( 'Your session expired. Please reload the page again.', 'project-huddle' ) );
	    }

	}

	/**
	 * Fetches parents, current items and children
	 *
	 * @since 1.0.0
	 *
	 * @return object Success/Fail json object with updated model
	 */
	public function phv_fetch_versions() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// get model
		$image_id = $_POST['image_id'];

		// validate data
		if ( ! isset( $image_id ) ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" image ID is not set.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// sanitize
		$image_id = (int) $image_id;

		// add pre action
		do_action( 'phv_fetch_versions_before', $image_id );

		/**
		 * Get parents (previous versions)
		 */
		$versions = (array) PH_Versions()->versions->get_parents( $image_id );

		/**
		 * Current version
		 */
		$versions[] = PH()->image->get_image( $image_id );

		/**
		 * Get children (future versions)
		 */
		if ( $children = (array) PH_Versions()->versions->get_children( $image_id ) ) {
			foreach( $children as $child ) {
				$versions[] = $child;
			}
		}

		// add version numbers
		foreach( $versions as $key => $version ) {
			$versions[$key]['version'] = $key + 1;
		}

		// add post action
		do_action( 'phv_fetch_versions_after', $image_id, $versions );

		wp_send_json( $versions );
	}
}