<?php
/**
 * Plugin Name: ProjectHuddle - Image Versions
 * Plugin URL: http://projecthuddle.io
 * Description: Adds a version system for project images
 * Version: 1.0.0
 * Author: Andre Gagnon
 * Author URI: http://projecthuddle.io
 * Text Domain: project-huddle
 * Domain Path: languages
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Setup Constants before init because we're running plugin on plugins_loaded
 *
 * @since 1.1.1
 */

// Plugin Folder Path
if ( ! defined( 'PH_VERSIONS_PLUGIN_DIR' ) ) {
	define( 'PH_VERSIONS_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
}

// Plugin Folder URL
if ( ! defined( 'PH_VERSIONS_PLUGIN_URL' ) ) {
	define( 'PH_VERSIONS_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
}

// Plugin Root File
if ( ! defined( 'PH_VERSIONS_PLUGIN_FILE' ) ) {
	define( 'PH_VERSIONS_PLUGIN_FILE', __FILE__ );
}

if ( ! class_exists( 'PH_Image_Versions' ) ) :
/**
 * Main PH_Image_Versions Class
 * Uses singleton design pattern
 *
 * @since 1.0.0
 */
final class PH_Image_Versions {

	/**
	 * Holds only one PH_Image_Versions instance
	 * @var $instance
	 * @since 1.0
	 */
	private static $instance;

	/**
	 * Main PH_Image_Versions Instance
	 *
	 * Insures that only one instance of PH_Image_Versions exists in memory at any one
	 * time. Also prevents needing to define globals all over the place.
	 *
	 * @since  1.0.0
	 * @static var array $instance
	 * @uses   PH_Image_Versions::setup_constants() Setup the constants needed
	 * @uses   PH_Image_Versions::includes() Include the required files
	 * @uses   PH_Image_Versions::load_textdomain() load the language files
	 * @see    PH()
	 * @return PH_Image_Versions $instance The one true PH_Image_Versions
	 */
	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof PH_Image_Versions ) ) {

			// do nothing if ProjectHuddle is not activated
			if( ! class_exists( 'Project_Huddle', false ) ) {
				return false;
			}

			self::$instance = new PH_Image_Versions;

			// make sure we have the correct minimum version
			if ( version_compare( PH_VERSION, '1.1.0' ) < 0 ) {
				add_action( 'admin_notices', array( self::$instance, 'update_required' ) );
				return false;
			}

			// load includes
			self::$instance->includes();

			self::$instance->versions = new PH_VERSIONS_IMAGE();
			self::$instance->events   = new PHV_Ajax_Events();

			// Loaded action
			do_action( 'projecthuddle_versions_loaded' );
		}

		return self::$instance;
	}

	/**
	 * Throw error on object clone
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @since 1.0.0
	 *
	 * @uses _doing_it_wrong() Mark something as being incorrectly called.
	 *
	 * @access public
	 * @return void
	 */
	public function __clone() {
		// Cloning instances of the class is forbidden
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'project-huddle' ), '1.0.0' );
	}

	/**
	 * Disable un-serializing of the class
	 *
	 * @since 1.0.0
	 *
	 * @uses _doing_it_wrong() Mark something as being incorrectly called.
	 *
	 * @access public
	 * @return void
	 */
	public function __wakeup() {
		// Un-serializing instances of the class is forbidden
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'project-huddle' ), '1.0.0' );
	}

	/**
	 * Show notice if ProjectHuddle needs updating
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 * @return string
	 */
	public function update_required() {
		?>
		<div class="update-nag">
			<p><?php _e( 'The ProjectHuddle versions plugin requires the ProjectHuddle plugin to be at least version 1.0.2. Please update your ProjectHuddle plugin.', 'project-huddle' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Include required files
	 *
	 * @access private
	 * @since 1.0.0
	 * @return void
	 */
	private function includes() {
		// templates
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/templates.php';
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/scripts.php';

		// meta boxes
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/meta-boxes/meta-box-images.php';

		// classes
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/class-ph-versions-image.php';
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/class-phv-ajax.php';

		// functions
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/phv-project-functions.php';
		require_once PH_VERSIONS_PLUGIN_DIR . 'includes/misc-functions.php';
	}
}

endif; // end if class_exists

/**
 * The main function responsible for returning the one true PH_Versions
 * Instance to functions everywhere.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * Example: <?php $versions = PH_Versions(); ?>
 *
 * @since 1.0.0
 * @return object The one true PH_Versions Instance
 */
function PH_Versions() {
	return PH_Image_Versions::instance();
}
add_action( 'plugins_loaded', 'PH_Versions' );