<?php
/**
 * Rest Object Class
 * Abstract class for our models to extend to create
 * internal API requests.
 *
 * For example, to get pages, you can use
 * PHW()->page->get_items(array(), array('per_page', 20) );
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

abstract class PH_Rest_Object {
	/**
	 * Meta type
	 * @var string
	 */
	protected $meta_type = 'post';

	/**
	 * How to store collection name in model
	 * @var string
	 */
	public $collection_name = 'collection';

	/**
	 * Post type
	 * @var string
	 */
	public $post_type = 'post';

	/**
	 * Parent collection post type
	 * @var string
	 */
	public $parent_post_type = 'post';

	/**
	 * Post type route base
	 * @var string
	 */
	public $route_base = '/projecthuddle/v2';

	/**
	 * Post type route
	 * @var string
	 */
	public $route = '';

	/**
	 * Holds post type schema
	 * @var array
	 */
	public $schema = array();

	/**
	 * Get post type route
	 * @return string
	 */
	public function route() {
		$post_type_object = get_post_type_object( $this->post_type );
		return $this->route_base . '/' . $post_type_object->rest_base;
	}

	/**
	 * Get object
	 *
	 * @param int    $id           ID of item
	 * @param array  $params       Request Parameters
	 * @param array  $query_params Request Query Parameters
	 * @param string $route        Overwrite route
	 *
	 * @return mixed
	 */
	public function get_item( $id = 0, $params = array(), $query_params = array(), $route = '' ) {
		if ( ! $id ) {
			global $post;
			$id = $post->ID;
		}

		$route   = ! $route ? $this->route() . '/' . $id : $route;
		$request = new WP_REST_Request( 'GET', $route, $params );

		return $this->do_request( $request, $params, $query_params );
	}

	/**
	 * Get object items
	 *
	 * @param array $params       Request Parameters
	 * @param array $query_params Request Query Parameters
	 *
	 * @return mixed
	 */
	public function get_items( $params = array(), $query_params = array() ) {
		$request = new WP_REST_Request( 'GET', $this->route(), $params );

		return $this->do_request( $request, $params );
	}

	/**
	 * Create item
	 *
	 * @param array  $params       Request Parameters
	 * @param array  $query_params Request Query Parameters
	 * @param string $route        Overwrite route
	 *
	 * @return mixed
	 */
	public function create_item( $params = array(), $query_params = array(), $route = '' ) {
		$route   = ! $route ? $this->route() : $route;
		$request = new WP_REST_Request( 'POST', $route, $params );

		return $this->do_request( $request, $params, $query_params );
	}

	/**
	 * Method mapping for internal filters
	 * @param $method
	 *
	 * @return string
	 */
	public function map_method($method) {
		switch ($method) {
			case 'POST':
				$method = 'new';
				break;
			case 'PUT':
			case 'PATCH':
				$method = 'edit';
				break;
			case 'DELETE':
				$method = 'delete';
				break;
			default:
				$method = 'get';
				break;
		}
		return $method;
	}

	/**
	 * Do internal API request
	 *
	 * @param       $request
	 * @param array $params
	 * @param array $query_params
	 *
	 * @return mixed
	 */
	public function do_request( $request, $params = array(), $query_params = array() ) {
		// Add specified request parameters into the request.
		if ( ! empty( $params ) ) {
			foreach ( $params as $param_name => $param_value ) {
				$request->set_param( $param_name, $param_value );
			}
		}

		// set query params (i.e. per_page, context, etc.)
		if ( ! empty($query_params) ) {
			$request->set_query_params($query_params);
		}

		$response = rest_do_request( $request );
		$data     = $response->get_data();

		return $data;
	}

	/**
	 * Add filters to type
	 */
	public function add_filters() {
		if ( $this->parent_post_type ) {
			// add collection data on get
			add_filter( "rest_prepare_{$this->parent_post_type}", array( $this, 'collection_data' ), 10, 3 );
		}

		// add additional collection parameters
		if ( $this->collection_params() ) {
			add_filter( "rest_{$this->post_type}_collection_params", array( $this, 'additional_collection_params' ), 10, 2 );
			add_filter( "rest_{$this->post_type}_query", array( $this, 'additional_collection_query' ), 10, 2 );
		}

		// add parent collection params
		add_filter( "rest_{$this->post_type}_collection_params", array( $this, 'parent_collection_param' ), 10, 2 );
		add_filter( "rest_{$this->post_type}_query", array( $this, 'parent_collection_query' ), 10, 2 );

		// convert everything to shortlinks to prevent 404
		add_filter( "rest_api_init", array( $this, 'use_shortlinks' ), 10, 2 );
	}

	/**
	 * Get public collection query params
	 *
	 * @param $query_params
	 * @param $post_type
	 *
	 * @return mixed
	 */
	public function additional_collection_params($query_params, $post_type) {
		foreach( (array) $this->collection_params() as $name => $data ) {
			$query_params[$name] = $data;
		}
		return $query_params;
	}

	/**
	 * Add collection queries
	 * @param $args
	 * @param $request
	 *
	 * @return mixed
	 */
	public function additional_collection_query($args, $request) {
		foreach( (array) $this->collection_params() as $name => $data ){
			if ( $data['meta'] ) {
				$args['meta_query'] = apply_filters( "ph_{$name}_collection_query_args", array(
					array(
						'key'     => $name,
						'value'   => $request[$name],
					)
				), $name, $request, $args );
			}
		}

		return $args;
	}

	/**
	 * Add parent model to collection parameters
	 *
	 * @param $query_params array Query parameters
	 * @param $post_type    string Post type
	 *
	 * @return array
	 */
	public function parent_collection_param( $query_params, $post_type ) {
		$query_params['parent_id'] = array(
			'description' => __( 'Limit result set to items that have a parent model.' ),
			'type'        => 'integer',
		);

		return $query_params;
	}

	/**
	 * Query by parent model
	 *
	 * @param $args    array Query arguments
	 * @param $request object Request object
	 *
	 * @return array Filters query argument
	 */
	public function parent_collection_query($args, $request) {
		if ( isset( $request['parent_id'] ) ) {
			$args['meta_query']     = array(
				array(
					'key'   => 'parent_id',
					'value' => $request['parent_id'],
				)
			);
			$args['order']          = 'asc';
			$args['posts_per_page'] = 100;
		}

		return $args;
	}

	/**
	 * Add child collection data to response
	 *
	 * @param $response
	 *
	 * @return false|object
	 */
	public function collection_data( $response, $post ) {
		$response->data[$this->collection_name] = $this->get_items( array(
			'parent_id' => $post->ID
		) );
		return $response;
	}

	/**
	 * Register our custom fields with API
	 */
	public function register_fields_from_schema() {
		if ( empty( $this->schema ) ) {
			return;
		}
		foreach ( $this->schema as $key => $property ) {
			$meta = array(
				'type'         => $property['type'], // this will sanitize our type
				'description'  => $property['description'],
				'single'       => true,
				'show_in_rest' => true,
			);

			// optional/additional sanitize callback
			if ( array_key_exists( 'arg_options', $property ) && array_key_exists( 'sanitize_callback', $property['arg_options'] ) ) {
				$meta['sanitize_callback'] = $property['arg_options']['sanitize_callback'];
			}

			// register
			register_meta( $this->post_type, $key, $meta );
		}
	}

	/**
	 * Add additional fields to rest object
	 */
	public function rest_fields_from_schema() {
		if ( empty( $this->schema ) ) {
			return;
		}

		foreach ( $this->schema as $key => $property ) {
			$this->property = $property;

			// if we need to run a custom rest function
			if ( ! isset( $property['custom'] ) ) {
				$this->register_meta_rest_field($key, $property);
			}

			// register each rest field
			register_rest_field( $this->post_type, $key, array(
				'get_callback'    => function ( $post, $attr, $request, $object_type ) use ($property) {
					$data = get_metadata($this->meta_type, $post['id'], $attr, true);

					return rest_sanitize_value_from_schema( $data, $property );
				},
				'update_callback' => function ( $value, $post, $attr, $request, $object_type ) use ($property) {
					// add permissions filter for granular control
					if ( ! apply_filters("ph_website_update_{$this->action_slug}_{$attr}_allowed", true, $post, $value ) ) {
						return new WP_Error( 'rest_forbidden_meta', __( 'Sorry, you are not allowed to do this.', 'project-huddle' ), array( 'status' => rest_authorization_required_code() ) );
					}

					// filter attribute name
					$attr = apply_filters( 'ph_update_'. $attr, $attr );

					// update
					update_metadata( $this->meta_type, $post->ID, $attr, $value );

					// run through get to make sure it's a sanitized value to prevent xss
					$meta = get_metadata( $this->meta_type, $post->ID, $attr, true );

					// run action on update
					do_action( "ph_website_rest_update_{$this->action_slug}_attribute", $attr, $meta, $post );
					do_action( "ph_website_rest_update_{$this->action_slug}_{$attr}", $meta, $post );

					// sanitize
					return rest_sanitize_value_from_schema( $meta, $property );
				},
				'schema'          => $property
			) );
		}
	}

	/**
	 * Register meta rest fields
	 *
	 * @param $key
	 * @param $property
	 */
	public function register_meta_rest_field($key, $property) {
		// register each rest field
		register_rest_field( $this->post_type, $key, array(
			'get_callback'    => function ( $post, $attr, $request, $object_type ) use ($property) {
				$data = get_metadata($this->meta_type, $post['id'], $attr, true);

				return rest_sanitize_value_from_schema( $data, $property );
			},
			'update_callback' => function ( $value, $post, $attr, $request, $object_type ) use ($property) {
				// add permissions filter for granular control
				if ( ! apply_filters("ph_website_update_{$this->action_slug}_{$attr}_allowed", true, $post, $value ) ) {
					return new WP_Error( 'rest_forbidden_context', __( 'Sorry, you are not allowed to do this.', 'project-huddle' ), array( 'status' => rest_authorization_required_code() ) );
				}

				// update
				update_metadata( $this->meta_type, $post->ID, $attr, $value );

				// run through get to make sure it's a sanitized value to prevent xss
				$meta = get_metadata( $this->meta_type, $post->ID, $attr, true );

				// run action on update
				do_action( "ph_website_rest_update_{$this->action_slug}_attribute", $attr, $meta, $post );
				do_action( "ph_website_rest_update_{$this->action_slug}_{$attr}", $meta, $post );

				// sanitize
				return rest_sanitize_value_from_schema( $meta, $property );
			},
			'schema'          => $property
		) );
	}

	public function use_shortlinks() {
		register_rest_field(
			$this->post_type,
			'ph_short_link',
			array(
				'get_callback'    => array( $this, 'get_shortlink' ),
				'update_callback' => null,
				'schema'          => array(
					'description' => 'Short Link',
					'type'        => 'string',
					'readonly'    => true,
					'context'     => array( 'view' )
				)
			)
		);
	}

	/**
	 * Return a shortlink
	 * @param $post
	 *
	 * @return string
	 */
	public function get_shortlink($post) {
		return wp_get_shortlink( (int) $post['id']);
	}

	// collection params
	public function collection_params() {
		return array();
	}

	/**
	 * Standardize actions for each model
	 * I.E. Threads:
	 *
	 * PHP objects Fires every time
	 * ph_website_publish_thread
	 * ph_website_edit_thread
	 * ph_website_delete_thread
	 *
	 * REST actions fire only for rest actions.
	 * This is useful for instance if you want to do an action
	 * for only rest requests but not for internal changes to post types
	 *
	 * ph_website_rest_publish_thread
	 * ph_website_rest_edit_thread
	 * ph_website_rest_delete_thread
	 */
	public function add_actions() {
		// regular actions
		add_action( "publish_{$this->post_type}", array( $this, 'new_action' ), 10, 2 );
		add_action( "edit_{$this->post_type}", array( $this, 'edit_action' ), 10, 2 );
		add_action( "trashed_{$this->post_type}", array( $this, 'delete_action' ), 10, 2 );

		// rest actions
		add_filter( "rest_prepare_{$this->post_type}", array( $this, 'handle_rest_prepare' ), 10, 3 );
		add_action( "rest_insert_{$this->post_type}", array( $this, 'edit_rest_action'), 20, 3 );
		add_action( "rest_delete_{$this->post_type}", array( $this, 'delete_rest_action'), 20, 3 );
	}

	/**
	 * New thread action
	 *
	 * @param int        $id      The Post ID.
	 * @param WP_Comment $item Post object
	 */
	public function new_action( $id, $item ) {
		do_action( "ph_website_publish_{$this->action_slug}", $id, $item );
	}

	/**
	 * Edit Filter
	 *
	 * @param int   $id The Post ID
	 * @param array $item    Post Object
	 */
	public function edit_action( $id, $item ) {
		do_action( "ph_website_edit_{$this->action_slug}", $id, $item );
	}

	/**
	 * Edit Filter
	 *
	 * @param int   $id The Post ID
	 * @param array $item    Post Object
	 */
	public function delete_action( $id, $item ) {
		do_action( "ph_website_delete_{$this->action_slug}", $id, $item );
	}

	/**
	 * Need to do a special case for POST since meta are added after
	 * rest_insert_post
	 *
	 * @param $response
	 * @param $object
	 * @param $request
	 *
	 * @return mixed
	 */
	public function handle_rest_prepare( $response, $object, $request ) {
		if ( $request->get_method() === 'POST' ) {
			do_action( "ph_website_rest_publish_{$this->action_slug}", $object, $request );
		}

		return $response;
	}

	/**
	 * Fires after a single post object is updated via the REST API.
	 *
	 * @param WP_Post         $post     Inserted or updated post object.
	 * @param WP_REST_Request $request  Request object.
	 * @param bool            $creating True when creating a post, false when updating.
	 */
	public function edit_rest_action( $post, $request, $creating ) {
		// must not be new
		if ( $creating ) {
			return;
		}

		do_action( "ph_website_rest_edit_{$this->action_slug}", $post, $request );
	}

	/**
	 * Fires after a single post object is updated via the REST API.
	 *
	 * @param object           $post     The deleted or trashed post.
	 * @param WP_REST_Response $response The response data.
	 * @param WP_REST_Request  $request  The request sent to the API.
	 */
	public function delete_rest_action( $post, $response, $request ) {
		do_action( "ph_website_rest_delete_{$this->action_slug}", $post, $response, $request );
	}
}