<?php
/**
 * ProjectHuddle API Endpoints
 *
 * Creates rest routes, sanitizes and validates, adds filters to hook functions.
 *
 * @package     ProjectHuddle
 * @subpackage  PH Website Versions
 * @copyright   Copyright (c) 2016, Andre Gagnon
 * @since       1.2.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Admin_Project_Images Class
 *
 * This class handles the all the ajax requests made by the plugin
 *
 * @since 1.0.0
 */
class PH_Website_API {

	/**
	 * Current API Version
	 *
	 * @var string
	 */
	protected $version = 'v1';

	/**
	 * Namespace for our API
	 *
	 * @var string
	 */
	protected $api_name = 'projecthuddle';

	/**
	 * Store route URL
	 * @var
	 */
	protected $route_url;

	/**
	 * Setup project admin
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// route URL for website api
		$this->route_url = $this->api_name . '/' . $this->version . '/website';

		// add rest API Endpoints
		add_action( 'rest_api_init', array( $this, 'project_routes' ) );
		add_action( 'rest_api_init', array( $this, 'page_routes' ) );
		add_action( 'rest_api_init', array( $this, 'thread_routes' ) );
		add_action( 'rest_api_init', array( $this, 'comment_routes' ) );
	}

	public function project_routes() {
		$base = 'project';

		// POST new
		register_rest_route( $this->route_url, '/' . $base . '/',
			array(
				'methods'  => array( 'POST' ),
				'callback' => array( $this, 'new_website_project' ),
				'permission_callback' => function () {
					$current_user = wp_get_current_user();
					return isset( $current_user->allcaps['publish_ph-websites'] )
						? $current_user->allcaps['publish_ph-websites']
						: false;
				}
			)
		);

		// Update
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => array( $this, 'edit_website_project' ),
				'permission_callback' => function () {
					return current_user_can( 'edit_ph-websites' );
				},
				'args'                => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
			)
		);

		// Update project subscribers /{project_id}/actions/subscribe
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/subscribe/',
			array(
				'methods'             => array( 'POST' ),
				'callback'            => array( $this, 'add_project_subscriber' ),
				'permission_callback' => function () {
					return current_user_can( 'edit_ph-websites' );
				},
				'args'                => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
			)
		);

		// Remove project subscribers /{project_id}/actions/subscribe
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/subscribe/',
			array(
				'methods'  => array( 'DELETE' ),
				'callback' => array( $this, 'remove_project_subscriber' ),
				'permission_callback' => function () {
					return current_user_can( 'edit_ph-websites' );
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
			)
		);
	}

	/**
	 * Page Thread Routes
	 */
	public function page_routes() {
		$base = 'page';

		// POST new
		register_rest_route( $this->route_url, '/' . $base . '/',
			array(
				'methods'  => array( 'POST' ),
				'callback' => array( $this, 'new_website_page' ),
			)
		);

		// Update
		register_rest_route( $this->route_url, '/' . $base . '/',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => array( $this, 'edit_website_page' ),
				'permission_callback' => function () {
					return current_user_can( 'edit_ph-websites' );
				}
			)
		);

		$base = 'page/reorder';
		// Update
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'POST', 'PUT', 'PATCH', 'GET' ),
				'callback' => array( $this, 'reorder_website_page' ),
			)
		);
	}

	/**
	 * Comment Thread Routes
	 */
	public function thread_routes() {
		$base = 'comment-thread';

		// GET thread
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'GET' ),
				'callback' => array( $this, 'get_website_thread' ),
				'permission_callback' => function () {
					return current_user_can( 'read' );
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
			)
		);

		// POST new
		register_rest_route( $this->route_url, '/' . $base . '/',
			array(
				'methods'  => array( 'POST' ),
				'callback' => array( $this, 'new_website_thread' ),
				'permission_callback' => function () {
					return current_user_can( 'read' );
				}
			)
		);

		// EDIT existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => array( $this, 'edit_website_thread' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function () {
					return current_user_can( 'read' );
				}
			)
		);

		// resolve
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/resolve',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => array( $this, 'resolve_website_thread' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function () {
					$current_user = wp_get_current_user();
					return isset( $current_user->allcaps['resolve_ph-website_threads'] )
						? $current_user->allcaps['resolve_ph-website_threads']
						: false;
				}
			)
		);

		// EDIT existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/assign',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => array( $this, 'assign_website_thread' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function () {
					$current_user = wp_get_current_user();
					return isset( $current_user->allcaps['assign_ph-website_threads'] )
						? $current_user->allcaps['assign_ph-website_threads']
						: false;
				}
			)
		);



		// DELETE existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'DELETE' ),
				'callback' => array( $this, 'delete_website_thread' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function () {
					$current_user = wp_get_current_user();
					return isset( $current_user->allcaps['delete_ph-website_threads'] )
						? $current_user->allcaps['delete_ph-website_threads']
						: false;
				}
			)
		);

		$base = 'thread/reorder';
		// Update
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'POST', 'PUT', 'PATCH', 'GET' ),
				'callback' => array( $this, 'reorder_website_thread' ),
			)
		);
	}

	/**
	 * Comment Thread Routes
	 */
	public function comment_routes() {
		$base = 'comment';

		// POST new
		register_rest_route( $this->route_url, '/' . $base,
			array(
				'methods'  => array( 'POST' ),
				'callback' => array( $this, 'new_website_comment' ),
			)
		);

		// PUT to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => array( $this, 'edit_website_comment' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function ( $request ) {
					$data = $request->get_params();

					$user = wp_get_current_user();
					$comment = get_comment( $data['id'] );

					// let the person edit their own
					if ( $comment->comment_author_email == $user->user_email ) {
						return true;
					}

					return isset( $user->allcaps['moderate_ph-website_comments'] )
						? $user->allcaps['moderate_ph-website_comments']
						: false;
				}
			)
		);

		// DELETE to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'DELETE' ),
				'callback' => array( $this, 'delete_website_comment' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function ( $request ) {
					$data = $request->get_params();

					$user = wp_get_current_user();
					$comment = get_comment( $data['id'] );

					// let the person edit their own
					if ( $comment->comment_author_email == $user->user_email ) {
						return true;
					}

					return isset( $user->allcaps['moderate_ph-website_comments'] )
						? $user->allcaps['moderate_ph-website_comments']
						: false;
				}
			)
		);
	}

	/**
	 * Callback functions
	 *
	 * @param object $data Model object
	 *
	 * @return object Filtered object
	 */
	public function new_website_project( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function edit_website_project( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function add_project_subscriber( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function remove_project_subscriber( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function new_website_page( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function edit_website_page( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function reorder_website_page( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function get_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function new_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function edit_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function resolve_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function assign_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function trash_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function reorder_website_thread( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function new_website_comment( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function edit_website_comment( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}
	public function trash_website_comment( $data ) {
		return apply_filters( 'ph_' . __FUNCTION__, $data );
	}

}