<?php
/**
 * Webpage Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Project Class
 *
 * This class handles the project data
 *
 * @since 1.0
 */
class PH_Website_Page extends PH_Rest_Object {
	/**
	 * Post type
	 * @var string
	 */
	public $post_type = 'ph-webpage';

	/**
	 * Parent collection post type
	 * @var string
	 */
	public $parent_post_type = 'ph-website';

	/**
	 * Collection name
	 * @var string
	 */
	public $collection_name = 'pages';

	/*
	 * Slug for actions
	 * @var string
	 */
	public $action_slug = 'page';

	/**
	 * Actions and filters
	 */
	public function __construct() {
		// add schema
		$this->schema = $this->schema();

		// add fields
		$this->register_fields_from_schema();

		// standardize actions
		$this->add_actions();

		// register fields
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );

		// make sure page doesn't exist with same url before creating
		add_filter( "rest_pre_insert_{$this->post_type}", array( $this, 'check_for_existing_page' ), 10, 2 );
		add_filter( "rest_pre_insert_{$this->post_type}", array( $this, 'always_publish' ), 10, 2 );

		// check for trailing slash in website_url query
		add_filter( "ph_page_url_collection_query_args", array( $this, 'smart_match_url' ), 10, 3 );

		// pages user order
		add_filter( "rest_prepare_{$this->parent_post_type}", array( $this, 'user_order_data' ), 10, 3 );

		// add filters
		$this->add_filters();
	}

	/**
	 * Schema for post meta
	 * @return array
	 */
	public function schema() {
		return apply_filters( 'ph_website_page_meta', array(
			'parent_id' => array(
				'description' => esc_html__( 'ID of parent website.', 'project-huddle' ),
				'type'        => 'integer',
				'default'     => 0,
				'required'    => true
			),
			'user_order'     => array(
				'description' => esc_html__( 'The user\'s custom thread order', 'project-huddle' ),
				'type'        => 'array',
				'items'       => array(
					'description' => esc_html__( 'Custom order ids.', 'project-huddle' ),
					'type'        => 'integer',
					'readonly'     => true,
				),
				'readonly'    => true
			),
			'page_url'  => array(
				'description' => esc_html__( 'Url for the page.', 'project-huddle' ),
				'type'        => 'url',
				'required'    => true
			)
		) );
	}

	/**
	 * Collection query params
	 *
	 * Allows querying collections via the following meta data.
	 *
	 * @return array
	 */
	public function collection_params() {
		return apply_filters( 'ph_website_page_collection_parameters', array(
			'page_url' => array(
				'description' => esc_html__( 'Url for the page.', 'project-huddle' ),
				'type'        => 'string',
				'meta'        => true
			)
		) );
	}

	/**
	 * Also check for trailing slash in website_url attribute query
	 * instead of exact query math
	 *
	 * @param string          $name Parameter name
	 * @param WP_REST_Request $request
	 *
	 * @return array
	 */
	public function smart_match_url( $args, $name, $request ) {
		if ( ! $request[ $name ] ) {
			return $args;
		}
		// get https and http variants
		$url = $request[ $name ];
		$parse = parse_url($url);

		if ( $parse['scheme'] === 'http' ) {
			$url_https = preg_replace("/^http:/i", "https:", $url);
		} else {
			$url_https = $url;
			$url = preg_replace("/^https:/i", "http:", $url);
		}

		return array(
			array(
				'key'     => $name,
				'value'   => array(
					$url,
					trailingslashit( $url ),
					$url_https,
					trailingslashit( $url_https )
				),
				'compare' => 'IN'
			)
		);
	}

	public function user_order_data( $response, $post ) {
		$user_id = get_current_user_id();

		// on admin only
		if ( ! is_admin() || ! $user_id ) {
			return $response;
		}

		// get threads order stored in user meta by parent id
		$response->data['user_order'] = get_user_meta( $user_id, 'pages_order_' . (int) $response->data['id'], true );

		return $response;
	}

	/**
	 * Check for existing webpage
	 *
	 * @param $prepared_post object Prepared post object
	 * @param $request       object Request object
	 *
	 * @return object Request object
	 */
	public function check_for_existing_page( $prepared_post, $request ) {
		$params = $request->get_params();
		$method = $request->get_method();

		// if we're creating a new page and have page_url
		if ( 'POST' === $method && isset( $params['page_url'] ) ) {
			$items = $this->get_items( array(
				'page_url' => $params['page_url'],
				'status'   => 'publish'
			) );

			// if page already exists
			if ( isset( $items[0] ) ) {
				// make sure parent ids match (same website project)
				if ( $items['parent_id'] === $params['parent_id'] ) {
					// short-circuit save and return already saved model instead
					wp_send_json( $items[0], '208' );
				}
			}
		}

		// return prepared post as usual
		return $prepared_post;
	}

	/**
	 * Check for existing webpage
	 *
	 * @param $prepared_post object Prepared post object
	 * @param $request       object Request object
	 *
	 * @return object Request object
	 */
	public function always_publish( $prepared_post, $request ) {
		if ( $prepared_post->post_type == $this->post_type ) {
			$prepared_post->post_status = 'publish';
		}
		return $prepared_post;
	}

	public function get_page_title($url){
		if( !class_exists( 'WP_Http' ) )
			include_once( ABSPATH . WPINC . '/class-http.php' );
		$request = new WP_Http;
		$result = $request->request( $url );
		if( is_wp_error( $result ) )
			return false;

		if( preg_match("#<title>(.+)<\/title>#iU", $result['body'], $t))  {
			return trim($t[1]);
		} else {
			return false;
		}
	}

	/**
	 * Fallback filters
	 * @param $prepared_post
	 * @param $request
	 *
	 * @return object
	 */
	public function filters( $prepared_post, $request ) {
		$method = $this->map_method( $request->get_method() );

		return apply_filters( 'ph_' . $method . '_website_page', $prepared_post, $request );
	}

	/**
	 * Get page data
	 * @deprecated Use get_items method instead
	 *
	 * @param bool $site_id
	 * @param      $page_id
	 * @param      $page_url
	 *
	 * @return array|bool
	 */
	public function data( $site_id = false, $page_id, $page_url) {
		// bail if deleted
		if ( false == get_post_status( $page_id ) ) {
			return false;
		}

		$url           = $this->url( $site_id, $page_url );
		$threads       = PHW()->thread->get_website_threads( $page_id );
		$comment_count = sizeof( $threads );

		// bail if no comments
		if ( $comment_count == 0 ) {
			return false;
		}

		$user = wp_get_current_user();

		return array(
			'title'         => esc_html( get_the_title( $page_id ) ),
			'id'            => (int) $page_id,
			'page_url'      => $page_url,
			'url'           => $url,
			'comment_count' => $comment_count,
			'user_order'    => get_user_meta( $user->ID, 'threads_order_' . $page_id, true ),
			'threads'       => $threads
		);
	}

	/**
	 * Get all comment threads on specific page
	 *
	 * @param bool   $id     Page ID
	 * @param string $status Publish status
	 *
	 * @return mixed
	 */
	public function get_comment_threads($id = false, $status = 'publish') {
		global $post;
		$id = $id ? $id : $post->ID;

		$comment_threads = get_post_meta( $id, 'comment_locations', true);

		// check status
		foreach ( $comment_threads as $key => $comment_thread ) {
			if ( $status != get_post_status( $comment_thread ) || !$comment_thread ) {
				unset( $comment_threads[$key] );
				continue;
			}
		}

		return $comment_threads;
	}

	/**
	 * Gets the website URL
	 *
	 * @param bool   $id   ID of website project
	 * @param string $page Page url
	 *
	 * @return mixed|string
	 */
	public function url($id = false, $page = '') {
		global $post;
		$id = $id ? $id : $post->ID;

		// normalize url
		$website_url = parse_url( get_post_meta( (int) $id, 'ph_website_url', true ) );
		$port        = isset( $website_url['port'] ) ? ':' . $website_url['port'] : '';
		$url         = $website_url['scheme'] . '://' . $website_url['host'] . $port . $page;

		return $url;
	}

	/**
	 * New webpage post
	 *
	 * @return bool|int ID or false
	 */
	public function new_page( $title = 'Webpage') {
		// arguments to create a new post
		$post_args = apply_filters( 'ph_new_comment_location_website_args', array(
			'post_title'  => esc_html( $title ),
			'post_type'   => 'ph-webpage',
			'post_status' => 'publish',
		) );

		// insert new post and store id
		$id = wp_insert_post( $post_args, true );

		// return id or false
		return is_wp_error( $id ) ? false : $id;
	}

	/**
	 * Gets page ID from URL
	 * 
	 * @param $site_id
	 * @param $url
	 *
	 * @return bool|int
	 */
	public function get_webpage_id( $site_id, $url ) {
		// get existing subdirectories
		$existing = (array) get_post_meta( $site_id, 'ph_webpages', true );

		// return id of page in array
		$id = array_search( $url, $existing );

		// make sure it's published
		if ( get_post_status( $id ) != 'publish' ) {
			return false;
		}

		// return webpage id
		return (int) $id;
	}

	/**
	 * Adds a comment location to the website
	 *
	 * @param array $args Webpage id and thread id
	 *
	 * @return void
	 */
	public function add_thread( $args = array() ) {
		// Parse incoming $args into an array and merge it with $defaults
		$args = wp_parse_args( $args, array(
			'webpage_id' => 0,
			'thread_id' => 0,
			'assigned' => 0
		) );
		
		// get existing comments associated with image
		$comment_locations = get_post_meta( (int) $args['webpage_id'], 'comment_locations', true );

		// add to or start a comment locations array
		if ( isset( $comment_locations ) && is_array( $comment_locations ) ) {
			$comment_locations[] = (int) $args['thread_id'];
		} else {
			$comment_locations = array( (int) $args['thread_id'] );
		}

		if ( $args['assigned'] ) {
			update_post_meta( (int) $args['thread_id'], 'assigned', (int) $args['assigned'] );
		}

		// update locations
		update_post_meta( (int) $args['webpage_id'], 'comment_locations', array_unique( $comment_locations ) );
	}

	/**
	 * Updates page order in database
	 *
	 * @param int   $id
	 * @param array $thread_ids
	 *
	 * @return bool Success/Failure
	 */
	public function set_thread_order( $id = 0, $thread_ids = array() ) {
		// get current pages
		$threads = (array) get_post_meta( $id, 'comment_locations', true );

		// return if empty
		if ( empty( $threads ) ) {
			return false;
		}

		// order (flip to values, replace, then flip back)
		$threads_ordered = array_flip( array_replace( array_flip( array_map( 'intval', $thread_ids ) ), array_flip($threads) ) );

		// update order
		update_post_meta( $id, 'comment_locations', $threads_ordered );

		return true;
	}

}