<?php
/**
 * Comment Thread Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2017, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) { exit; }

/**
 * Website Thread Class
 * @since 2.6.0
 */
class PH_Website_Thread extends PH_Rest_Object {
	/**
	 * Post type
	 * @var string
	 */
	public $post_type = 'phw_comment_loc';

	/**
	 * Parent collection post type
	 * @var string
	 */
	public $parent_post_type = 'ph-webpage';

	/**
	 * Collection name
	 * @var string
	 */
	public $collection_name = 'threads';

	/*
	 * Slug for actions
	 * @var string
	 */
	public $action_slug = 'thread';

	/**
	 * Add object meta and filters
	 */
	public function __construct() {
		// add schema
		$this->schema = $this->schema();

		// add fields
		$this->register_fields_from_schema();

		// register fields
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );

		// auto add comment data
		add_action( 'rest_api_init', array( $this, 'add_comment_data' ) );

		// add filters
		$this->add_filters();

		// standardize actions
		$this->add_actions();

		// permissions
		add_filter( 'user_has_cap', array( $this, 'caps' ), 10, 4 );

		// thread user order
		add_filter( "rest_prepare_{$this->parent_post_type}", array( $this, 'user_order_data' ), 10, 3 );

		// maybe create page if it doesn't exist
		add_action( "rest_prepare_{$this->post_type}", array( $this, 'maybe_create_page' ), 10, 3 );

		// always publish threads
		add_filter( "rest_pre_insert_{$this->post_type}", array( $this, 'always_publish' ), 10, 2 );

		// force comments open for this post type
		add_filter( "comments_open", array( $this, 'force_comments_open' ), 10, 2 );
	}

	/**
	 * Schema for post meta
	 * @return array
	 */
	public function schema() {
		return apply_filters( 'ph_website_thread_meta', array(
			'path'           => array(
				'description' => esc_html__( 'Path to the clicked element.', 'project-huddle' ),
				'type'        => 'string',
				'default'     => ''
			),
			'relativeX'      => array(
				'description' => esc_html__( 'Relative horizontal click position of the element.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => 0
			),
			'relativeY'      => array(
				'description' => esc_html__( 'Relative vertical click position of the element.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => 0
			),
			'html'           => array(
				'description' => esc_html__( 'Inner html of the clicked element.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'wp_kses_post',
				),
				'default'     => ''
			),
			'resX'           => array(
				'description' => esc_html__( 'Horizontal resolution of user who reported issue.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => ''
			),
			'resY'           => array(
				'description' => esc_html__( 'Vertical resolution of user who reported issue.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => ''
			),
			'resolved'       => array(
				'description' => esc_html__( 'Issue resolve status.', 'project-huddle' ),
				'type'        => 'boolean',
				'default'     => false
			),
			'browser'        => array(
				'description' => esc_html__( 'Browser information.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'sanitize_text_field',
				),
				'default'     => ''
			),
			'browserVersion' => array(
				'description' => esc_html__( 'Browser version information.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => '',
			),
			'browserOS'      => array(
				'description' => esc_html__( 'Operating system information.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'sanitize_text_field',
				),
				'default'     => ''
			),
			'assigned'       => array(
				'description' => esc_html__( 'ID of user who is assigned.', 'project-huddle' ),
				'type'        => 'integer',
				'default'     => 0
			),
			'parent_id'       => array(
				'description' => esc_html__( 'ID of parent webpage.', 'project-huddle' ),
				'type'        => 'integer',
				'default'     => 0,
			),
			'page_url'  => array(
				'description' => esc_html__( 'Url for the page.', 'project-huddle' ),
				'type'        => 'url',
			),
			'page_title' => array(
				'description' => esc_html__( 'Title for the new page.', 'project-huddle' ),
				'type'        => 'string'
			),
			'website_id' => array(
				'description' => esc_html__( 'Website project id for the thread.', 'project-huddle' ),
				'type'        => 'integer'
			),
			'comments'       => array(
				'description' => esc_html__( 'Array of comment objects in thread.', 'project-huddle' ),
				'type'        => 'array',
				'items'       => array(
					'description' => esc_html__( 'Comment object.', 'project-huddle' ),
					'type'        => 'object',
					'readonly'     => true,
				),
				'readonly'    => true,
				'custom' => true
			)
		) );
	}

	/**
	 * Collection query params
	 *
	 * Allows querying collections via the following meta data.
	 *
	 * @return array
	 */
	public function collection_params() {
		return apply_filters( 'ph_website_thread_collection_parameters', array(
			'resolved' => array(
				'description' => esc_html__( 'Limit results by resolved status.', 'project-huddle' ),
				'type'        => 'boolean',
				'meta'        => true
			),
			'assigned' => array(
				'description' => esc_html__( 'Limit by ID of user who is assigned.', 'project-huddle' ),
				'type'        => 'integer',
				'meta'        => true
			)
		) );
	}

	/**
	 * Add comment data as rest field
	 */
	public function add_comment_data() {
		register_rest_field( $this->post_type, 'comments',
			array(
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					$data = PHW()->comment->get_items(array(
						'post'          => array( $post['id'] ),
						'order'         => 'asc',
						'per_page'      => 100
					));
					return $data;
				},
				'schema'          => $this->schema()['comments']
			)
		);
	}

	/**
	 * Filter on the current_user_can() function.
	 * This function is used to explicitly allow users to edit their own comments
	 * Regardless of their capabilities or roles.
	 *
	 * @param array  $allcaps All the capabilities of the user
	 * @param array  $cap     [0] Required capability
	 * @param array  $args    [0] Requested capability
	 *                        [1] User ID
	 *                        [2] Associated object ID
	 * @param object $user    User object
	 *
	 * @return array
	 */
	public function caps($allcaps, $cap, $args, $user ) {
		// Bail out if we're not asking about a post:
		if ( 'edit_post' != $args[0] ) {
			return $allcaps;
		}

		// Load the post data:
		$post = get_post( $args[2] );

		if ( ! $post ) {
			return $allcaps;
		}

		// bail if not our comment type
		if ( $this->post_type !== $post->post_type ) {
			return $allcaps;
		}

		// Bail out if the user is the post author:
		if ( (int) $user->ID !== (int) $post->post_author ) {
			return $allcaps;
		}

		// add required caps
		$allcaps['edit_comment']         = true;
		$allcaps['edit_published_posts'] = true;
		$allcaps[ $cap[0] ]              = true;

		return $allcaps;
	}

	/**
	 * Adds user order data to model
	 *
	 * @param $response
	 * @param $post
	 *
	 * @return mixed
	 */
	public function user_order_data( $response, $post ) {
		$user_id = get_current_user_id();

		// on admin only
		if ( ! is_admin() || ! $user_id ) {
			return $response;
		}

		// get threads order stored in user meta by parent id
		$response->data['user_order'] = get_user_meta( $user_id, 'threads_order_' . (int) $response->data['id'], true );

		return $response;
	}

	/**
	 * Filters the post data for a response.
	 *
	 * The dynamic portion of the hook name, `$this->post_type`, refers to the post type slug.
	 *
	 * @since 4.7.0
	 *
	 * @param WP_REST_Response $response The response object.
	 * @param WP_Post          $post     Post object.
	 * @param WP_REST_Request  $request  Request object.
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function maybe_create_page( $response, $post, $request ) {
		// if we're posting a new thread
		if ( $request->get_method() !== 'POST' ) {
			return $response;
		}

		// Can't create comment if didn't supply parent_id or page_url
		if ( ! isset( $request['parent_id'] ) || ! isset( $request['page_url'] ) ) {
			return new WP_Error( 'rest_cannot_create', __( 'Sorry, you need to provide a page id or page url for the thread.' ), array( 'status' => rest_authorization_required_code() ) );
		}

		// if we already have the parent id, don't do anything
		if ( $request['parent_id'] ) {
			return $response;
		}

		// can't create comment if blank parent_id or page_url
		if ( ! $request['page_url'] ) {
			return new WP_Error( 'rest_cannot_create', __( 'Sorry, you need to provide a page id or page url for the thread.' ), array( 'status' => rest_authorization_required_code() ) );
		} else {
			// get website id provided by api
			$website_id = isset( $request['website_id'] ) ? (int) $request['website_id'] : 0;

			// find existing page
			$items = PHW()->page->get_items( array(
				'page_url' => $request['page_url'],
			) );

			// if page already exists and website project is published
			if ( isset( $items[0] ) && get_post_status( $items[0]['parent_id'] ) === 'publish' ) {
				update_post_meta( $post->ID, 'parent_id', $items[0]['id'] );
				$response->data['parent_id'] = $items[0]['id'];
			} else {
				// get website id provided by api
				$website_id = isset( $request['website_id'] ) ? (int) $request['website_id'] : 0;

				// if website id is not provided, query by url as backup
				if ( ! $website_id ) {
					// find website with same hostname
					$parsed  = parse_url( $request['page_url'] );
					$origin  = $parsed['scheme'] . '://' . $parsed['host'];
					$origin  .= isset( $parsed['port'] ) ? $parsed['port'] : '';
					$website = PHW()->website->get_items( array(
						'website_url' => $origin,
					) );

					$website_id = ! empty( $website[0] ) || isset( $website[0]['id'] ) ? $website[0]['id'] : 0;
				}

				// otherwise create parent page in website project
				// if no website, do nothing
				if ( ! $website_id ) {
					return new WP_Error( 'rest_cannot_create', __( 'Sorry, you need to provide a website id or valid page url for the thread.' ), array( 'status' => rest_authorization_required_code() ) );
				}

				// double-check permissions
				if ( ! current_user_can( 'read_ph-website', $website_id ) ) {
					return new WP_Error( 'rest_cannot_create', __( 'Sorry, you don\'t have access to this website project.' ), array( 'status' => rest_authorization_required_code() ) );
				}

				// create page
				$page = PHW()->page->create_item( array(
					'title'     => isset( $request['page_title'] ) ? $request['page_title'] : $request['page_url'],
					'page_url'  => $request['page_url'],
					'parent_id' => $website_id
				) );

				// update thread parent
				update_post_meta( $post->ID, 'parent_id', $page['id'] );
				$response->data['parent_id'] = $page['id'];
			}
		}

		return $response;
	}

	/**
	 * Check for existing webpage
	 *
	 * @param $prepared_post object Prepared post object
	 * @param $request       object Request object
	 *
	 * @return object Request object
	 */
	public function always_publish( $prepared_post, $request ) {
		if ( $prepared_post->post_type == $this->post_type ) {
			$prepared_post->post_status = 'publish';
		}
		return $prepared_post;
	}

	/**
	 * Finds existing webpage post based on referring url
	 * @return mixed
	 */
	public function find_post( $post_id = false) {
		global $post;

		// get referring URL to find correct webpage post
		$referring_url = parse_url( $_SERVER['HTTP_REFERER'] );

		// get existing pages on website
		$existing = (array) get_post_meta( $post->ID, 'ph_webpages', false );

		if ( empty( $existing ) || ! isset( $existing[0] ) || ! is_array( $existing[0] ) ) {
			return false;
		}

		// return webpage id if it exists
		return array_search( $referring_url['path'], $existing[0] );
	}

	/**
	 * Make sure comments are always open on our parent post
	 * type before comment is saved
	 *	 *
	 * @return array
	 */
	public function force_comments_open( $open, $post_id ) {
		if ( get_post_type( $post_id ) === $this->post_type ) {
			if ( ! $open ) {
				$open = true;
			}
		}

		return $open;
	}

	/**
	 * Adds locations meta and comments to comment locations
	 */
	public function comment_location_meta( $id = false ) {
		$id = $id ? $id : $this->webpage_id;

		$this->locations = get_post_meta( (int) $id, 'comment_locations', true );

		$this->get_location_info($this->locations);
	}

	public function get_location_info( $locations ) {
		if ( ! is_array( $locations ) || empty( $locations ) ) {
			if ( is_int( $locations ) ) {
				$locations = array($locations);
			} else {
				return false;
			}
		}
		foreach ( $locations as $key => $location_post_id ) {
			// make sure it's published
			if ( !$location_post_id || get_post_status( $location_post_id ) != 'publish' ) {
				continue;
			}

			$status = get_post_status( $location_post_id );

			$this->comments[ $key ]                 = new stdClass();
			$this->comments[ $key ]->path           = get_post_meta( (int) $location_post_id, 'path', true );
			$this->comments[ $key ]->order          = get_post_meta( (int) $location_post_id, 'order', true );
			$this->comments[ $key ]->relativeX      = get_post_meta( (int) $location_post_id, 'relativeX', true );
			$this->comments[ $key ]->relativeY      = get_post_meta( (int) $location_post_id, 'relativeY', true );
			$this->comments[ $key ]->html           = get_post_meta( (int) $location_post_id, 'html', true );
			$this->comments[ $key ]->resolved       = (bool) get_post_meta( (int) $location_post_id, 'resolved', true );
			$this->comments[ $key ]->resX           = (int) get_post_meta( (int) $location_post_id, 'resX', true );
			$this->comments[ $key ]->resY           = (int) get_post_meta( (int) $location_post_id, 'resY', true );
			$this->comments[ $key ]->browser        = esc_html( get_post_meta( (int) $location_post_id, 'browser', true ) );
			$this->comments[ $key ]->browserVersion = (int) get_post_meta( (int) $location_post_id, 'browserVersion', true );
			$this->comments[ $key ]->browserOS      = get_post_meta( (int) $location_post_id, 'browserOS', true );
			$this->comments[ $key ]->id             = (int) $location_post_id;
			$this->comments[ $key ]->assigned       = get_post_meta( (int) $location_post_id, 'assigned', true );

			$user = get_userdata( $this->comments[ $key ]->assigned );

			$this->comments[ $key ]->assignedName   = is_object( $user ) && isset( $user->display_name ) ? $user->display_name : false;
			$this->comments[ $key ]->assignedAvatar = is_object( $user ) && isset( $user->ID ) ? get_avatar( $user->ID, 24 ) : false;
			$this->comments[ $key ]->items          = $this->comments( $location_post_id );
			if ( isset( $this->comments[ $key ]->items[0] ) ) {
				$this->comments[ $key ]->avatar         = $this->comments[ $key ]->items[0]->avatar;
				$this->comments[ $key ]->comment_author = $this->comments[ $key ]->items[0]->comment_author;
				$this->comments[ $key ]->date           = $this->comments[ $key ]->items[0]->date;
			}
			$this->comments[ $key ]->url = get_the_permalink( $location_post_id );
			$terms                       = wp_get_post_terms( $location_post_id, 'ph_status' );
			if ( ! is_wp_error( $terms ) ) {
				$this->comments[ $key ]->status_name = isset( $terms[0] ) ? $terms[0]->name : false;
				$this->comments[ $key ]->status_id   = isset( $terms[0] ) ? $terms[0]->term_id : false;
			}

		}

		return $this->comments;
	}

	/**
	 * Gets comments
	 *
	 * @param $id
	 *
	 * @return array
	 */
	public function comments( $id ) {
		// make sure post is published
		if ( $id == null || get_post_status( $id ) != 'publish' ) {
			return false;
		}

		// get comments
		$comments = get_comments( array(
			'post_id' => (int) $id,
			'orderby' => 'date',
			'order'   => 'ASC',
			'type'    => 'ph_website_comment',
			'ph_force_show' => true
		) );

		// check empty
		if ( empty( $comments) ) {
			return false;
		}

		// return with meta
		return $this->comment_meta( $comments );
	}

	/**
	 * Adds comment meta to comments
	 *
	 * @param array $comments Existing comments
	 *
	 * @return object Comments with meta added
	 */
	public function comment_meta( $comments ) {
		// process data
		foreach ( $comments as $comment ) {
			if ( isset($comment->user_id) && $comment->user_id ) {
				$user_info = get_userdata( $comment->user_id );
				$comment->avatar          = get_avatar( $comment->user_id, 64 );
				$comment->comment_author  = $user_info->display_name;
			} else {
				$comment->avatar          = get_avatar( $comment->comment_author_email, 64 );
			}

			$comment->date            = ph_human_time_diff( mysql2date( 'U', $comment->comment_date, true ), current_time( 'timestamp' ) );

			$comment->comment_content = wp_kses_post( $comment->comment_content  );
		}

		return $comments;
	}

	/**
	 * Gets comments based on referring url
	 *
	 * @return array|bool Comment locations with meta and comments
	 */
	public function get_website_threads( $page_id = 0 ) {
		$this->comments = array();
		if ( $page_id ) {
			$this->webpage_id = $page_id;
		} else {
			if ( ! $this->webpage_id = $this->find_post() ) {
				return false;
			}
		}
		// add meta to the comment locations
		$this->comment_location_meta();

		return array_values( $this->comments );
	}

	/**
	 * Add new thread
	 *
	 * @param string $title
	 * @param array  $meta
	 *
	 * @return bool|int|WP_Error
	 */
	public function add_thread( $title = 'New Thread', $meta = array() ) {
		// get our default term of backlog
		$term = get_term_by( 'slug', 'backlog', 'ph_status' );
		$term_id = $term ? $term->term_id : false;

		// arguments to create a new post
		$post_args = apply_filters( 'ph_new_comment_location_website_args', array(
			'post_type'   => 'phw_comment_loc',
			'post_status' => 'publish',
			'tax_input' => $term_id ? array( 'ph_status' => $term_id ) : false
		) );

		// insert new post and store id
		$post_id =  wp_insert_post( $post_args, true );

		// make sure success happens
		if ( is_wp_error( $post_id ) ) {
			return false;
		}

		// add meta
		$this->add_meta($post_id, $meta);

		// return success
		return $post_id;
	}

	/**
	 * Add thread meta values
	 * @param int   $id
	 * @param array $meta
	 */
	public function add_meta( $id = 0, $meta = array() ) {
		global $post;

		// marge defaults
		$meta = wp_parse_args( $meta, array(
			'path'           => false,
			'order'          => 0,
			'relativeX'      => 0,
			'relativeY'      => 0,
			'html'           => '',
			'resX'           => 0,
			'resY'           => 0,
			'resolved'       => 0,
			'browser'        => __( 'Unknown', 'project-huddle' ),
			'browserVersion' => __( 'Unknown', 'project-huddle' ),
			'browserOS'      => __( 'Unknown', 'project-huddle' ),
			'webpage_id'     => 0,
			'website_id'     => $post ? $post->ID : 0
		) );

		// add meta
		add_post_meta( $id, 'path', esc_html( $meta['path'] ) );
		add_post_meta( $id, 'order', (int) $meta['order'] );
		add_post_meta( $id, 'relativeX', floatval( $meta['relativeX'] ) );
		add_post_meta( $id, 'relativeY', floatval( $meta['relativeY'] ) );
		add_post_meta( $id, 'html', esc_html( $meta['html'] ) );
		add_post_meta( $id, 'resX', esc_html( $meta['resX'] ) );
		add_post_meta( $id, 'resY', esc_html( $meta['resY'] ) );
		add_post_meta( $id, 'browser', esc_html( $meta['browser'] ) );
		add_post_meta( $id, 'browserVersion', (int) $meta['browserVersion'] );
		add_post_meta( $id, 'browserOS', $meta['browserOS'] );
		add_post_meta( $id, 'webpage_id', (int) $meta['webpage_id'] );
		add_post_meta( $id, 'resolved', (bool) $meta['resolved'] );
		add_post_meta( $id, 'website_id', (int) $meta['website_id'] );
	}
}