<?php
/**
 * Email functions
 */

/**
 * Email subscribed people about new comments
 *
 * @param integer    $id      Inserted or updated comment id.
 * @param WP_Comment $comment Inserted or updated comment object.
 *
 * @return void
 */
function ph_website_new_comment_email( $id, $comment ) {
	// get ids of parent posts
	$parents = ph_get_parents_ids( $id, 'comment' );

	// get website
	$page    = get_post( $parents['page'] );

	// get all emails
	if ( ! $to_emails = ph_get_project_emails( $parents['website'], get_userdata( $comment->comment_author_email ) ) ) {
		return;
	}

	// filter
	$to_emails = (array) apply_filters( 'ph_website_create_comment_email_to', $to_emails, $comment );

	// send each email individually
	foreach( $to_emails as $email ) {
		$email = ( new PH_Mail )
			->template( ph_locate_template( 'email/new-website-comment-email.php', '', PH_WEBSITE_PLUGIN_DIR . 'templates/' ), [
				'commenter'    => esc_html( $comment->comment_author ),
				'avatar'       => get_avatar( get_current_user_id(), 42, '', '', array(
					'force_display' => true,
					'extra_attr'    => 'style="border-radius: 42px"'
				) ),
				'project_name' => sanitize_text_field( html_entity_decode( get_the_title( $parents['website'] ) ) ),
				'page_name'    => sanitize_text_field( html_entity_decode( $page->post_title ) ),
				'content'      => wpautop( $comment->comment_content ),
				'link'         => ph_email_link( wp_get_shortlink( (int) $comment->comment_post_ID ), __( 'View Comment', 'project-huddle' ) )
			] )
			->to( $email )
			->subject( get_option( 'ph_website_new_comment_email_subject', '{{commenter}} made a new comment on {{project_name}}.' ) )
			->send();
	}
}

add_action( 'ph_website_publish_comment', 'ph_website_new_comment_email', 10, 2 );

/**
 * Email subscribed people about new comments
 *
 * @param boolean $value  Meta value data
 * @param WP_Post $thread Post object
 *
 * @return void
 */
function ph_website_assigned_thread_email( $value, $thread ) {
	// only run for assigned changes
	if ( ! isset( $value ) ) {
		return;
	}

	$user         = get_userdata( (int) $value );
	$current_user = wp_get_current_user();

	// need a user
	if ( ! $user ) {
		return;
	}

	// get assigned user email
	if ( ! $to_emails = array( sanitize_email( $user->user_email ) ) ) {
		return;
	}

	// filter
	$to_emails = (array) apply_filters( 'ph_website_assigned_thread_email_to', $to_emails, $thread, $value );

	// get parent ids
	$parents = ph_get_parents_ids( $thread->ID );

	// get original comment
	$first_comment = get_comments( array(
		'post_id'       => $thread->ID,
		'ph_force_show' => true,
		'number'        => 1,
		'order'         => 'ASC'
	) );

	// send each email individually
	foreach( $to_emails as $email ) {
		$email = ( new PH_Mail )
			->template( ph_locate_template( 'email/assigned-website-thread-email.php', '', PH_WEBSITE_PLUGIN_DIR . 'templates/' ), [
				'commenter'    => esc_html( $current_user->display_name ),
				'content'      => wpautop( wp_kses_post( $first_comment[0]->comment_content ) ),
				'avatar'       => get_avatar( $current_user->ID, 42, '', '', array(
					'force_display' => true,
					'extra_attr'    => 'style="border-radius: 42px"'
				) ),
				'page_name'    => sanitize_text_field( html_entity_decode( get_the_title( $parents['page'] ) ) ),
				'project_name' => sanitize_text_field( html_entity_decode( get_the_title( $parents['website'] ) ) ),
				'link'         => ph_email_link( wp_get_shortlink( (int) $thread->ID ), __( 'View Comment', 'project-huddle' ) )
			] )
			->subject( get_option( 'ph_website_assigned_thread_email_subject', 'You have been assigned to an issue on {{project_name}}.' ) )
			->to( $email )
			->send();
	}
}

add_action( 'ph_website_rest_update_thread_assigned', 'ph_website_assigned_thread_email', 10, 2 );

/**
 * Email subscribed people about new comments
 *
 * @param boolean $value  Meta value data
 * @param WP_Post $thread Post object
 *
 * @return void
 *
 */
function ph_website_resolved_thread_email( $value, $thread ) {
	// make sure there's a value
	if ( ! isset( $value ) ) {
		return;
	}

	// get current user
	$current_user = wp_get_current_user();

	// get ids of parent posts
	$parents = ph_get_parents_ids( $thread->ID );

	// get original comment
	$first_comment = get_comments( array(
		'post_id'       => $thread->ID,
		'ph_force_show' => true,
		'number'        => 1,
		'order'         => 'ASC'
	) );

	// get emails
	if ( ! $to_emails = ph_get_project_emails( $parents['website'], get_userdata( $current_user->user_email ) ) ) {
		return;
	}

	// filter
	$to_emails = (array) apply_filters( 'ph_website_resolved_thread_email_to', $to_emails, $thread, $value );

	if ( $value ) {
		// send each email individually
		foreach( $to_emails as $email ) {
			$email = ( new PH_Mail )
				->template( ph_locate_template( 'email/resolved-website-thread-email.php', '', PH_WEBSITE_PLUGIN_DIR . 'templates/' ), [
					'commenter'    => sanitize_text_field( $current_user->display_name ),
					'avatar'       => get_avatar( $current_user->ID, 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px"'
					) ),
					'content'      => wpautop( wp_kses_post( $first_comment[0]->comment_content ) ),
					'page_name'    => sanitize_text_field( html_entity_decode( get_the_title( $parents['page'] ) ) ),
					'project_name' => sanitize_text_field( html_entity_decode( get_the_title( $parents['website'] ) ) ),
					'link'         => ph_email_link( wp_get_shortlink( (int) $thread->ID ), __( 'View Comment', 'project-huddle' ) )
				] )
				->subject( get_option( 'ph_website_resolved_thread_email_subject', '{{commenter}} resolved an issue on {{project_name}}.' ) )
				->to( $email )
				->send();
		}
	} else {
		// send each email individually
		foreach( $to_emails as $email ) {
			$email = ( new PH_Mail )
				->template( ph_locate_template( 'email/unresolved-website-thread-email.php', '', PH_WEBSITE_PLUGIN_DIR . 'templates/' ), [
					'commenter'    => sanitize_text_field( $current_user->display_name ),
					'avatar'       => get_avatar( $current_user->ID, 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px"'
					) ),
					'content'      => wpautop( wp_kses_post( $first_comment[0]->comment_content ) ),
					'page_name'    => sanitize_text_field( html_entity_decode( get_the_title( $parents['page'] ) ) ),
					'project_name' => sanitize_text_field( html_entity_decode( get_the_title( $parents['website'] ) ) ),
					'link'         => ph_email_link( wp_get_shortlink( (int) $thread->ID ), __( 'View Comment', 'project-huddle' ) )
				] )
				->subject( get_option( 'ph_website_unresolved_thread_email_subject', '{{commenter}} marked an issue on {{project_name}} unresolved.' ) )
				->to( $email )
				->send();
		}
	}
}

add_action( 'ph_website_rest_update_thread_resolved', 'ph_website_resolved_thread_email', 10, 2 );

function ph_website_email_new_member( $data ) {
	global $access_token_override;

	$access_token_override = true;

	if ( ! isset( $data['id'] ) || ! isset( $data['email'] ) ) {
		return;
	}

	$current_user = wp_get_current_user();

	// filter
	$to_emails = (array) apply_filters( 'ph_website_member_subscribed_email_to', $data['email'], $data['id'] );

	// send each email individually
	foreach( $to_emails as $email ) {
		$email = ( new PH_Mail )
			->template( ph_locate_template( 'email/website-subscribe-project-email.php', '', PH_WEBSITE_PLUGIN_DIR . 'templates/' ), [
				'commenter'    => sanitize_text_field( $current_user->display_name ),
				'avatar'       => get_avatar( $current_user->ID, 42, '', '', array(
					'force_display' => true,
					'extra_attr'    => 'style="border-radius: 42px"'
				) ),
				'project_name' => sanitize_text_field( html_entity_decode( get_the_title( $data['id'] ) ) ),
				'link'         => ph_email_link( wp_get_shortlink( $data['id'] ), __( 'View Project', 'project-huddle' ) )
			] )
			->subject( get_option( 'ph_website_member_subscribed_email_subject', 'You have been invited to collaborate on {{project_name}}.', $data['id'] ) )
			->to( $email )
			->send();
	}
}

add_action( 'ph_website_add_project_subscriber', 'ph_website_email_new_member', 10, 2 );

/**
 * Add user to notifications
 *
 * @param integer $id
 * @param WP_Comment $comment
 */
function ph_website_add_user_to_notifications( $id, $comment ) {
	// get author of comment
	if ( ! $author = get_user_by( 'ID', $comment->user_id ) ) {
		return;
	}

	// get ids of parent posts
	$parents = ph_get_parents_ids( $id, 'comment');

	// add as project member
	ph_add_project_member( $parents['website'], $author->user_email );
}

add_action( 'ph_website_publish_comment', 'ph_website_add_user_to_notifications', 10, 2 );