<?php
/**
 * Add our query vars
 *
 * @param $query_vars array Array of vars
 *
 * @return array New array with ours added
 */
function ph_query_vars( $query_vars ) {
	$query_vars[] = 'ph_apikey'; // our API key
	$query_vars[] = 'ph_website'; // our website
	$query_vars[] = 'ph_handler'; // our ajax handler
	return $query_vars;
}
add_filter( 'query_vars', 'ph_query_vars' );

/**
 * Loads specific files based on query var
 *
 * ph_handler for ajax requests
 * ph_website for loading javascript file
 *
 * @param $wp
 */
function ph_parse_request( &$wp ) {
	if ( array_key_exists( 'ph_handler', $wp->query_vars ) ) {
		include PH_WEBSITE_PLUGIN_DIR . 'includes/ph-website-ajax-handler.php';
		exit();
	}
	if ( array_key_exists( 'ph_website', $wp->query_vars ) ) {
		include PH_WEBSITE_PLUGIN_DIR . 'includes/ph-website-template.php';
		exit();
	}

	return;
}
add_action( 'parse_request', 'ph_parse_request' );

/**
 * When website is saved, generate api key
 *
 * @param $post_id
 * @param $post
 *
 * @return string|false
 */
function ph_generate_api_key($post_id, $post = false, $url = false) {
	$requested  = isset( $_REQUEST['ph_website_url'] ) ? $_REQUEST['ph_website_url'] : false;
	$url        = $url == false ? $requested : $url;
	$stored_api = get_post_meta( $post_id, 'ph_website_api', true );

	// we need a website url
	if ( ! isset( $url ) || ! $url ) {
		return false;
	}

	// if there's no api set yet
	if ( ! $stored_api ) {
		// generate api key based on post id, url and time
		$api_key = md5($post_id . $url . time());

		// update post meta
		update_post_meta( $post_id, 'ph_website_api', sanitize_text_field( $api_key ) );
	} else {
	    $api_key = $stored_api;
    }

	return isset( $api_key ) ? sanitize_text_field( $api_key ) : false;
}
add_action( 'save_post_ph-website','ph_generate_api_key', 1, 2 );

/**
 * Create or showunique access token
 *
 * @param $post_id
 *
 * @return string Created or stored access token
 */
function ph_access_token( $post_id ) {
	if ( ! $token = get_post_meta( $post_id, 'access_token', true ) ) {
		// generate api key based on post id, url and time
		$token = md5( uniqid( $post_id, true ) );
		update_post_meta( $post_id, 'access_token', esc_sql( $token ) );

		return $token;
	}

	return esc_html( $token );
}

/**
 * Create and echo API url
 *
 * @param mixed $post_id Post ID or Post Object
 */
function ph_the_api_url( $post_id ) {
	// convert post object to id
	if ( is_object($post_id) ) {
		$post_id = $post_id->ID;
	}

	// get nonce and API key
	$api_key = get_post_meta( $post_id, 'ph_website_api', true );

	$base_url = home_url( '?p=' . $post_id );
	$base_url = str_replace( 'http:', '', $base_url );
	$base_url = str_replace( 'https:', '', $base_url );

	// echo URL
	echo add_query_arg( array(
		'ph_apikey' => esc_html( $api_key ),
	), $base_url );
}

/**
 * Checks if the public API key matches the post we're trying to get
 *
 * @param string $key API key to check
 *
 * @return bool True if it matches
 */
function ph_public_api_matches( $key ) {
	global $post;

	$saved = get_post_meta( (int) $post->ID, 'ph_website_api', true );

	// they match
	if ( $saved == $key ) {
		return true;
	}

	// don't match
	return false;
}

/**
 * Check if two domains match
 *
 * @param string $url        Domain to check against
 * @param array  $wl_domains White-listed domains
 *
 * @return bool True if matches anything in white-list
 */
function ph_domain_matches( $url, $wl_domains = array() ) {
	$domain = parse_url( $url, PHP_URL_HOST );

	// Check if we match the domain exactly
	if ( in_array( $domain, $wl_domains ) ) {
		return true;
	}

	// check for partial matches
	foreach( $wl_domains as $wl_domain ) {
		$wl_domain = '.' . $wl_domain; // Prevent things like 'evilsitetime.com'

		if( strpos( $domain, $wl_domain ) === ( strlen( $domain ) - strlen( $wl_domain ) ) ) {
			return true;
			break;
		}
		if ( parse_url( $wl_domain, PHP_URL_HOST ) ==  $domain ) {
			return true;
		}
	}
	return false;
}

/**
 * Hides admin bar on website toolbar pages
 *
 * @return bool
 */
function ph_hide_website_admin_bar( $bool ) {
	if ( is_singular( 'ph-website' ) ) {
		return false;
	}

	return $bool;
}
add_filter( 'show_admin_bar', 'ph_hide_website_admin_bar', 1000 );

/**
 * Update permalink on website page without hitting save
 *
 * @param string  $return    Sample permalink HTML markup.
 * @param int     $post_id   Post ID.
 * @param string  $new_title New sample permalink title.
 * @param string  $new_slug  New sample permalink slug.
 * @param WP_Post $post      Post object.
 *
 * @return string
 */
function ph_website_auto_save_permalinks( $return, $post_id, $new_title, $new_slug, $post ) {
	if ( get_post_type( $post_id ) != 'ph-website' ) {
		return $return;
	}

	// save slug
	wp_update_post( array(
		'ID'        => (int) $post_id,
		'post_name' => $new_slug
	) );

	return $return;
}

add_filter( 'get_sample_permalink_html', 'ph_website_auto_save_permalinks', 10, 5 );

function add_ph_code() {
	global $post;

	// make sure it's enabled
	if ( get_option( 'ph_disable_self' ) == true ) {
		return false;
	}

	// get site post
	$post_id = (int) get_option( 'ph_site_post' );

	// only if published
	if ( get_post_status( $post_id ) != 'publish' ) {
		return;
	}

	if ( ! $post_id || get_post_type( $post ) == 'ph-website' || get_post_type( $post ) == 'ph-project' ) {
		return;
	} ?>

	<script>
		(function (d, t) {
			var ph = d.createElement(t), s = d.getElementsByTagName(t)[0];
			ph.type = 'text/javascript';
			ph.src = '<?php ph_the_api_url($post_id); ?>';
			s.parentNode.insertBefore(ph, s);
		})(document, 'script');
	</script>
<?php
}
add_action( 'wp_footer', 'add_ph_code' );

/**
 * Adds status indicator for this website.
 *
 * @param $states array States array.
 *
 * @return array States array with ours added
 */
function ph_this_website_state( $states ) {
	global $post;

	// bail if not our post type
	if ( get_post_type( $post ) !== 'ph-website' ) {
		return $states;
	}

	// get site post
	$post_id = (int) get_option( 'ph_site_post' );

	if ( $post->ID === $post_id ) {
		$states[] = __( 'This Website', 'project-huddle' );
	}

	return $states;
}

add_filter( 'display_post_states', 'ph_this_website_state' );

if ( ! function_exists( 'ph_url_origin' ) ) :
	/**
	 * ph_url_origin
	 */
	function ph_url_origin( $url = '' ) {
		if ( ! $url ) {
			return '';
		}
		$parsed_url = parse_url( $url );

		$host = $parsed_url['scheme'] . '://' . $parsed_url['host'];
		$host .= isset( $parsed_url['port'] ) && $parsed_url['port'] ? ':' . $parsed_url['port'] : '';

		return esc_url( $host );
	}
endif;

/**
 * Rename permalink to Magic Share Link
 * Also adds tootip to explain.
 *
 * @param $html
 *
 * @return string
 */
function ph_website_rename_permalink( $html, $post_id, $new_title, $new_slug, $post ) {
	if ( get_current_screen()->post_type !== 'ph-website' ) {
		return $html;
	}

	$html = '<div class="ph-magic-link-wrap">
        <strong class="ph-permalink-label">'
	        . __( 'Project Access Link', 'project-huddle' )
	        . '<span class="ph-tooltip-wrap">
            <i class="ion-help-circled"></i>
                <span class="ph-tooltip">
                    ' . __( 'Share this link with someone to give them access to the project.', 'project-huddle' ) . '
                </span>
            </span>'
	        . '</strong>'
	        . $html . '<button type="button" class="ph-copy-link button button-primary button-small hide-if-no-js" aria-label="Copy Access Link" data-clipboard-text="' . esc_attr(get_permalink($post->ID)) . '">Copy</button>
    </div>';

	return $html;
}

add_filter( 'get_sample_permalink_html', 'ph_website_rename_permalink', 10, 5 );

/**
 * Add access token to website project links
 *
 * @param $permalink
 * @param $post
 *
 * @return string
 */
function ph_add_access_token( $permalink, $post ) {
    global $access_token_override;

	if ( get_post_type( $post->ID ) === 'ph-website' && is_admin() || $access_token_override ) {
		$permalink = $permalink . '?access_token=' . ph_access_token( $post->ID );
	}

	return $permalink;
}

add_filter( 'post_type_link', 'ph_add_access_token', 10, 2 );
add_filter( 'preview_post_link', 'ph_add_access_token', 10, 2 );