<?php
/**
 * Projects in Admin
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Admin_Project_Images Class
 *
 * This class handles the metabox for the project images in the admin
 *
 * @since 1.0
 */
class PH_Project_Admin {

	/**
	 * Custom post type slug
	 *
	 * @since 1.0
	 */
	public $post_type_slug = 'ph-project';

	/**
	 * Setup project admin
	 *
	 * @since 10
	 */
	public function __construct() {

		// run only on admin pages
		if ( ! is_admin() ) {
			return;
		}

		// remove project shortlink
		remove_action( 'wp_head', 'wp_shortlink_wp_head', 10 );

		// remove shortlink
		add_filter( 'pre_get_shortlink', array( $this, 'remove_shortlink' ), 10, 2 );

		// Add admin page scripts
		add_action( 'admin_enqueue_scripts', array( $this, 'add_admin_scripts' ), 99 );

		// add collaborators ui
		add_action( 'post_submitbox_start', array( $this, 'collaboratorsBox' ) );
	}

	/**
	 * Adds styles and scripts needed on the page
	 *
	 * @since 1.0
	 */
	public function add_admin_scripts( $hook ) {
		// get post type
		global $post_type;

		// store css and javascript directories
		$css_dir = PH_PLUGIN_URL . 'assets/css/';
		$js_dir  = PH_PLUGIN_URL . 'assets/js/';

		// Use minified libraries if SCRIPT_DEBUG is turned off
		$suffix = ''; //defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG  ? '' : '.min';

		if ( $post_type == 'ph-project' || $post_type == 'ph-website' ) {
			wp_deregister_script( 'yoast-seo-select2' );
			wp_dequeue_script( 'yoast-seo-select2' );
			wp_deregister_style( 'select2-css' );
			wp_deregister_script( 'select2-js' );
			wp_dequeue_style('select2');
			wp_deregister_style('select2');
			wp_dequeue_script('select2');
			wp_deregister_script('select2');
		}

		// bail out early if we are not on a project add/edit screen.
		if ( $this->post_type_slug != $post_type || ( $hook !== 'post.php' && $hook !== 'post-new.php' ) ) {
			return false;
		}

		// register reusable scripts
		wp_register_script( 'ph-select2-admin', $js_dir . 'select2-admin.js', array( 'jquery', 'select2' ), PH_VERSION );
		wp_register_script( 'select2', $js_dir . 'includes/select2.full.min.js', array( 'jquery' ), '4.0.2' );
		wp_register_script( 'autosize', $js_dir . 'includes/autosize.min.js', array(), '3.0.2' );

		wp_register_style( 'select2', $css_dir . 'includes/select2.min.css', array(), '4.0.2' );
		wp_register_style( 'ionicons', $css_dir . 'includes/ionicons.css', array(), '2.0.0' );
		wp_register_style( 'ph-select2-admin', $css_dir . 'project-huddle-select2-admin' . $suffix . '.css', array( 'select2' ), '4.0.2' );

		// enqueue media controls
		wp_enqueue_media();

		// enqueue jquery ui style
		wp_enqueue_style( 'jquery-ui-css', $css_dir . 'includes/jquery-ui-fresh.min.css', array(), PH_VERSION );

		// enqueue main style
		wp_enqueue_style( 'project-huddle-admin-css', $css_dir . 'project-huddle-admin' . $suffix . '.css', array( 'ph-select2-admin', 'ionicons' ), PH_VERSION );

		// color picker
		wp_enqueue_style( 'wp-color-picker' );

		// place cursor at end
		wp_register_script( 'place-cursor-at-end', $js_dir . 'includes/jquery.placecursoratend.js', array( 'jquery' ), '1.0' );

		// enqueue needed admin script
		wp_enqueue_script( 'project-huddle-admin-js', $js_dir . 'project-huddle.admin' . $suffix . '.js', array(
			'underscore',
			'backbone',
			'wp-backbone',
			'jquery-ui-sortable',
			'jquery-ui-dialog',
			'place-cursor-at-end',
			'wp-color-picker',
			'ph-select2-admin'
		), PH_VERSION );

		// Add nonce
		wp_localize_script( 'project-huddle-admin-js', 'PH', array(
			'nonce'        => wp_create_nonce( 'project-image-nonce' ),
			'site_url'     => get_site_url(),
			'translations' => array(
				'wordpress_users'       => __( 'WordPress Users', 'project-huddle' ),
				'subscribe_people'      => __( 'Subscribe People To Notifications', 'project-huddle' ),
				'no_results'            => __( 'Email is invalid or no users found.', 'project-huddle' ),
				'input_too_short'       => __( 'Enter an email or search for a user', 'project-huddle' ),
				'subscribe_placeholder' => __( 'Start typing to subscribe someone...', 'project-huddle' )
			)
		) );

		wp_localize_script( 'ph-select2-admin', 'PH_Select2', array(
			'options' => array(
				'anon_users' => true
			),
		) );

		wp_localize_script( 'ph-select2-admin', 'wpApiSettings', array(
			'root'          => esc_url_raw( get_rest_url() ),
			'nonce'         => wp_create_nonce( 'wp_rest' ),
			'versionString' => 'wp/v2/',
        ));
	}

	/**
	 * Display the metabox that holds our project images
	 * Holds spinner animation for loading
	 *
	 * @since 1.0
	 */
	public function get_project_meta_box() { ?>

		<!-- projects wrapper -->
		<div class="ph-projects-wrapper">
			<span class="spinner"></span>
		</div>
		<!-- ph-projects-wrapper -->

	<?php
	}

	/**
	 * Shows collaborators and controls
	 *
	 * @since 1.0
	 */
	// TODO: Maybe separate functionality of collaborators
	public function collaboratorsBox() { ?>
	<?php
	}


	/**
	 * Removes wp.me shortlink functionality
	 *
	 * @param $false
	 * @param $post_id
	 *
	 * @return string
	 */
	public function remove_shortlink( $false, $post_id ) {
		return 'ph-project' === get_post_type( $post_id ) ? '' : $false;
	}

}