<?php
/**
 * Project Options Meta Box
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Meta_Box_Project_Options Class
 *
 * @since 1.0
 */
class PH_Meta_Box_Project_Images {

	/**
	 * Display the metabox that holds our project images
	 * Holds spinner animation for loading
	 *
	 * @since 1.0
	 */
	public static function output() {
		global $post;

		// create nonce
		wp_nonce_field( 'project_huddle_save_data', 'project_huddle_meta_nonce' );

		// add thickbox popup
		add_thickbox();

		?>

		<!-- projects wrapper -->
		<div class="ph-projects-wrapper">
			<span class="spinner"></span>
		</div>
		<!-- ph-projects-wrapper -->

		<?php

		// allow adding divs
		do_action( 'ph_image_meta_box_output', $post );
	}

	/**
	 * Save meta box data
	 * Validates data on input
	 *
	 * @since 1.0
	 */
	public static function save( $post_id, $post ) {

		// get images array from input $_POST
		$collection['ID']      = isset( $_POST['ph-project-id'] ) ? (array) $_POST['ph-project-id'] : array();
		$collection['title']   = isset( $_POST['ph-project-title'] ) ? (array) $_POST['ph-project-title'] : array();
		$collection['imageID'] = isset( $_POST['ph-project-imageID'] ) ? (array) $_POST['ph-project-imageID'] : array();

		// get image options
		foreach ( PH()->image->project_options as $key => $option ) {
			$collection[ $key ] = isset( $_POST[ $key ] ) ? (array) $_POST[ $key ] : array();
		}

		$collection = apply_filters( 'ph_meta_box_image_save_hidden_fields', $collection );

		// combine into single array to store
		$images = self::merge_common_keys( $collection );

		// Create a post if any ids are attachments, store new ids
		$images_ids = self::create_attachment_post( $images, $post );

		// get old
		$old = get_post_meta( $post_id, 'ph_project_images', true );

		// If there's some images set, set new var
		if ( isset( $images_ids ) ) {
			$new = $images_ids;
		} else {
			$new = null;
		}

		// Update or delete check to clean up unused meta
		if ( ( $new && $new != $old ) || ( $new && $new == 0 ) ) {
			// filter out non-digit values for security
			update_post_meta( $post_id, 'ph_project_images', $new );
		} elseif ( '' == $new && $old || ! isset( $images ) ) {
			// delete post meta if no new or images aren't set
			delete_post_meta( $post_id, 'ph_project_images', $old );
		}

		// return nothing
		return false;
	}

	/**
	 * Checks array of ids and converts any image attachments to project posts if needed
	 *
	 * @param $images array Ids to save
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function create_attachment_post( $images, $post ) {
		// global var to prevent being executed twice
		static $flag;

		// makes sure it doesn't get executed twice
		if ( $flag == 0 ) {
			$flag = 1;
		} else {
			return $images;
		}

		// make sure not empty
		if ( empty( $images ) ) {
			return false;
		}

		// temporarily disable save_post filter
		remove_action( 'save_post', 'PH_Meta_Box_Project_Images::create_attachment_post' );

		$image_ids = array();
		foreach ( $images as $key => $image ) {

			// if id is already a post, add id to new array
			if ( 'attachment' != get_post_type( $image['ID'] ) ) {

				$image_ids[] = $image['ID'];

				// update the post title
				wp_update_post( apply_filters( 'ph_meta_image_save_args', array(
					'ID'           => (int) $image['ID'],
					'post_title'   => esc_html( $image['title'] )
				) ) );

				// update post meta ( image options )
				self::update_image_meta( $image['ID'], $image );

				continue;
			}

			$image_post = self::create_image_post( $image, $post, $images );

			// update post meta
			self::update_image_meta( $image_post, $image );

			// add post ID to custom field array
			$image_ids[] = $image_post;
		}

		// return changed ids
		return $image_ids;
	}

	/**
	 * Creates an image post with save data
	 *
	 * @param $image array image information
	 *
	 * @return int Post ID
	 * @since 1.0.0
	 */
	public static function create_image_post( $image, $post, $images ) {
		// arguments to create a new post
		$post_args = apply_filters( 'ph_create_image_post', array(
			'post_title'  => esc_html( $image['title'] ),
			'post_type'   => 'project_image',
			'post_status' => 'publish',
		), $image, $post, $images );

		// insert new post and store id
		$post_id = wp_insert_post( $post_args, true );

		// check for success
		if ( is_wp_error( $post_id ) ) {
			error_log( 'ProjectHuddle: Tried to create an image post but couldn\'t insert post.' );
			wp_send_json_error('Something went wrong. Check the error log');
		}

		// Insert the attachment.
		set_post_thumbnail( $post_id, $image['ID'] );

		return $post_id;
	}

	/**
	 * Updates meta options and url slug for an image post
	 *
	 * @param $post_id int ID for the post
	 * @param $save_data array Options to save
	 *
	 * @since 1.0.0
	 */
	public static function update_image_meta( $post_id, $save_data ) {
		global $post;

		// update each post meta
		foreach ( PH()->image->project_options as $key => $option ) {
			if ( isset( $save_data[$key] ) ) {
				// correct escape
				if ( $key != 'ph_image_background_image' ) {
					update_post_meta( $post_id, $key, esc_attr( $save_data[$key] ) );
				} else {
					update_post_meta( $post_id, $key, esc_url( $save_data[$key] ) );
				}
			}
		}

		// store parent id
		update_post_meta( $post_id, 'ph_project_id', $post->ID );
	}

	/**
	 * Combines posts and their attributes into a single array
	 *
	 * @param array $args
	 *
	 * @return array
	 *
	 * @since 1.0
	 */
	private static function merge_common_keys( array $args = array() ) {
		// get keys needed for each post
		$model_attributes = array_unique( array_keys( $args ) );

		// get number of posts
		reset( $args );
		$first_key = key( $args );
		$first     = count( $args[ $first_key ] );

		// set default output
		$merged = array();

		// loop through (number of posts) times to set each item
		for ( $i = 0; $i < $first; ++ $i ) {

			// create array for the item
			$merged[ $i ] = array();

			// loop through (number of posts) times to set each property for each item
			for ( $i2 = 0; $i2 < $first; ++ $i2 ) {
				// set attributes for each of the items
				foreach ( $model_attributes as $attribute ) {
					$merged[ $i ][ $attribute ] = isset( $args[ $attribute ][ $i ] ) ? $args[ $attribute ][ $i ] : null;
				}
			}
		}

		// return merged array
		return $merged;
	}
}