<?php
/**
 * Register Settings
 *
 * @package     Project Huddle
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class PH_Extensions extends PH_Settings {
	/**
	 * Stores settings within class
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $extensions;

	/**
	 * Setup the options page
	 *
	 * Creates settings, menu items and settings link on plugin page
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// Initialise settings
		add_action( 'admin_init', array( $this, 'init' ) );

		// register available extensions
		add_action( 'admin_init' , array( $this, 'register_extensions' ) );

		// Add settings page to menu
		add_action( 'admin_menu' , array( $this, 'add_menu_item' ), 20 );
	}

	/**
	 * Initialise settings
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function init() {
		$this->extensions = $this->extension_fields();
	}

	/**
	 * Add settings page to admin menu
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function add_menu_item() {
		$page = add_submenu_page(
			'edit.php?post_type=ph-project',
			__( 'ProjectHuddle Extensions', 'project-huddle' ),
			__( 'Extensions', 'project-huddle' ),
			'manage_ph_settings' ,
			'project_huddle_extensions' ,
			array( $this, 'extensions_page' )
        );
		add_action( 'admin_print_styles-' . $page, array( $this, 'extensions_assets' ) );
	}

	/**
	 * Load settings JS & CSS
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function extensions_assets() {

		// store javascript directory
		$js_dir = PH_PLUGIN_URL . 'assets/js/';

		// store css directory
		$css_dir = PH_PLUGIN_URL . 'assets/css/';

		// Use minified libraries if SCRIPT_DEBUG is turned off
		$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		wp_enqueue_style( 'ph-extensions-admin', $css_dir . 'ph-extensions-admin' . $suffix . '.css', false, PH_VERSION );
	}

	/**
     * Extension Fields
	 * @return array
	 */
	public function extension_fields() {
	    $extensions = array();

		$extensions['pro'] = apply_filters( 'ph_extensions_pro', array(
			'title'  => __( 'Pro', 'project-huddle' ),
			'fields' => array(
				'email_editor' => array(
					'id'          => 'email_editor',
					'image'       => '',
					'title'       => __( 'Email Editor', 'project-huddle' ),
					'description' => 'Description',
					'link'        => 'http://projecthuddle.io',
					'type'        => 'extension',
                    'plugin_file' => 'ph-emails-editor/ph-emails-editor.php'
				)
			)
		) );

		$extensions['free'] = apply_filters( 'ph_extensions_pro', array(
			'title'  => __( 'Free', 'project-huddle' ),
			'fields' => array(
				'email_editor' => array(
					'id'          => 'email_editor',
					'title'       => __( 'Free Email Editor', 'project-huddle' ),
					'description' => 'Description',
					'image'       => '',
					'link'        => 'http://projecthuddle.io',
					'type'        => 'extension',
				)
			)
		) );

		return $extensions;
    }

	/**
	 * Register plugin settings
	 * @return void
	 */
	public function register_extensions() {
		if( is_array( $this->extensions ) ) {
			foreach( $this->extensions as $section => $data ) {

				// Add section to page
				add_settings_section(
					$section,
					$data['title'],
					array( $this, 'settings_section' ),
					'project_huddle_extensions_' . $section
				);

				foreach( $data['fields'] as $field ) {
					$defaults = array(
						'id'       => false,
						'type'     => false,
						'label'    => false,
					);
					$field = wp_parse_args( $field, $defaults );

					// Add field to page
					add_settings_field(
						$field['id'],
						$field['label'],
						array( $this, 'display_extension' ),
						'project_huddle_extensions_' . $section,
						$section,
						array( 'field' => $field )
					);
				}
			}
		}
	}

	public function settings_section( $section ) {
		if ( ! isset( $this->extensions[ $section['id'] ]['description'] ) ) {
			return false;
		}

		$html = '<p> ' . $this->extensions[ $section['id'] ]['description'] . '</p>' . "\n";
		echo $html;
	}

	/**
	 * Generate HTML for displaying fields
	 * @param  array $args Field data
	 * @return void
	 */
	public function display_extension( $args ) {
		$field = $args['field'];

		$html = '';

		switch ( $field['type'] ) {
			case 'extension':
				$html .= '<h3 class="ph-extension-title">' . $field['title'] . '</h3>';
				$html .= '<div class="ph-extension-image">' . $field['image'] . '</div>';
				$html .= '<div class="ph-extension-description">' . wpautop( $field['description'] ) . '</div>';
				if ( is_plugin_active( $field['plugin_file'] ) ) {
					$html .= '<a href="' . $field['link'] . '" class="button-primary" disabled>' . __( 'Activated!', 'project-huddle' ) . '</a>';
				} else {
					$html .= '<a href="' . $field['link'] . '" class="button-secondary">' . __( 'Get this add-on', 'project-huddle' ) . '</a>';
				}
				break;
		}

		echo $html;
	}


	/**
	 * Load settings page content
	 * @return void
	 */
	public function extensions_page() {
		$active_tab = isset( $_GET['tab'] ) ? $_GET['tab'] : 'pro';

		ob_start();
		?>

		<div class="wrap" id="project_huddle_extensions">
			<h2 class="nav-tab-wrapper">
				<?php
				foreach( $this->extensions as $section => $data ) {
					$tab_url = add_query_arg(
						'tab',
						$section,
						remove_query_arg('settings-updated')
					);

					$active = $active_tab == $section ? ' nav-tab-active' : '';

					echo '<a href="' . esc_url( $tab_url ) . '" title="' . esc_attr( $data['title'] ) . '" class="nav-tab' . $active . '">';
					echo esc_html( $data['title'] );
					echo '</a>';
				}
				?>
			</h2>

			<div id="tab_container">
                <?php $this->do_extensions_sections( 'project_huddle_extensions_' . $active_tab ); ?><br>
                <a href="http://projecthuddle.io" class="button-primary" target="_blank">Browse All Extensions</a>
			</div>
			<!-- #tab_container-->
		</div><!-- .wrap -->

		<?php
		// output it
		echo ob_get_clean();
	}

	public function do_extensions_sections( $page ) {
		global $wp_settings_sections, $wp_settings_fields;

		if ( ! isset( $wp_settings_sections[$page] ) )
			return;

		foreach ( (array) $wp_settings_sections[$page] as $section ) {
			if ( $section['title'] )
				echo "<h2>{$section['title']}</h2>\n";

			if ( $section['callback'] )
				call_user_func( $section['callback'], $section );

			if ( ! isset( $wp_settings_fields ) || !isset( $wp_settings_fields[$page] ) || !isset( $wp_settings_fields[$page][$section['id']] ) )
				continue;

			$this->do_extensions_fields( $page, $section['id'] );
		}
	}

    public function do_extensions_fields($page, $section) {
	    global $wp_settings_fields;

	    if ( ! isset( $wp_settings_fields[$page][$section] ) )
		    return;

	    foreach ( (array) $wp_settings_fields[$page][$section] as $field ) {
		    $class = '';

		    if ( ! empty( $field['args']['class'] ) ) {
			    $class = esc_attr( $field['args']['class'] );
		    }

		    echo "<div class='ph-extension $class'>";
		    call_user_func($field['callback'], $field['args']);
		    echo '</div>';
	    }
    }
}

/**
 * Add-ons Get Feed
 *
 * Gets the add-ons page feed.
 *
 * @return string
 */
function ph_add_ons_get_feed( $tab = 'pro' ) {
	$cache = get_transient( 'projecthuddle_add_ons_feed_' . $tab );

	if ( false === $cache ) {
		$url = 'https://projecthuddle.io/?feed=feed-add-ons';

		if ( 'pro' !== $tab ) {
			$url = add_query_arg( array( 'display' => $tab ), $url );
		}

		$feed = wp_remote_get( esc_url_raw( $url ), array( 'sslverify' => false ) );

		if ( ! is_wp_error( $feed ) ) {
			if ( isset( $feed['body'] ) && strlen( $feed['body'] ) > 0 ) {
				$cache = wp_remote_retrieve_body( $feed );

				// cache hourly
				set_transient( 'projecthuddle_add_ons_feed_' . $tab, $cache, HOUR_IN_SECONDS );
			}
		} else {
			$cache = '<div class="error"><p>' . __( 'There was an error retrieving the add-ons list from the server. Please try again later.', 'affiliate-wp' ) . '</div>';
		}

	}

	return $cache;

}