<?php
/**
 * API Functions
 *
 * Process the API Requests
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       2.1.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class PH_Project_Actions extends PH_Project_Actions_Abstract {
	/**
	 * Variable to store author
	 *
	 * @var object
	 */
	protected $user;

	/**
	 * Variable to store model data
	 */
	protected $data;

	/**
	 * Setup project admin
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// images
		add_filter( 'ph_mockup_share_image', array( $this, 'share_image' ), 9);

		// comment threads
		add_filter( 'ph_mockup_new_thread', array( $this, 'new_thread' ), 9 );
		add_filter( 'ph_mockup_edit_thread', array( $this, 'edit_thread' ), 9 );
		add_filter( 'ph_mockup_delete_thread', array( $this, 'delete_thread' ), 9 );

		// comments
		add_filter( 'ph_mockup_new_comment', array( $this, 'new_comment' ), 9 );
		add_filter( 'ph_mockup_edit_comment', array( $this, 'edit_comment' ), 9 );
		add_filter( 'ph_mockup_delete_comment', array( $this, 'delete_comment' ), 9 );

		// add email address
		add_action( 'ph_after_mockup_new_comment', array( $this, 'add_email' ), 9 );
		add_action( 'ph_after_mockup_new_thread', array( $this, 'add_email' ), 9 );

		// send new comment email
		add_action( 'ph_after_mockup_new_comment', array( $this, 'send_comment_email' ), 9 );
	}

	public function share_image( $data ) {
		$this->data = $data->get_params();

		// make sure post is set
		if ( empty( $this->data['email'] ) || $this->data['email'] == '' ) {
			return new WP_Error( 'no_email', __( 'Hmmmm... something went wrong. Email not set. Please try again.', 'project-huddle' ) );
		}

		$email = $this->data['email'];

		// check honeypot for spam prevention
		if ( ! empty( $email['website'] ) || $email['website'] != '' ) {
			return new WP_Error( 'spam', __( 'Hmmmm... something went wrong. Website not set. Please try again.', 'project-huddle' ) );
		}
		// validate form
		if ( empty( $email['subject'] ) || $email['subject'] == '' ) {
			return new WP_Error( 'no_subject', __( 'You must enter a subject for your email.', 'project-huddle' ) );
		}
		// check message
		if ( empty( $email['message'] ) || $email['message'] == '' ) {
			return new WP_Error( 'no_message', __( 'You must enter a message for your email.', 'project-huddle' ) );
		}

		do_action( 'ph_share_email_before', $this->data, $email );

		// check email sharing permissions
		if ( get_post_meta( $this->data['project_id'], 'ph_project_sharing', true ) == 'off' ) {
			return new WP_Error( 'no_sharing', __( 'Email sharing is not enabled for this project.', 'project-huddle' ) );
		}

		ph_email_share_project( $email['address'], $email['subject'], $email['message'], $email['link'], $email['user'], $this->data );

//		ph_email_share_comment( $email['address'], $email['subject'], $email['message'], $email['link'], $email['user'], $this->data );

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		return $this->data;
	}

	public function new_thread( $data ) {
		$this->data = $data->get_params();
		$this->user = wp_get_current_user();

		// get our default term of backlog
		$term    = get_term_by( 'slug', 'backlog', 'status' );
		$term_id = $term ? $term->term_id : false;

		// create location post
		$this->data['id'] = wp_insert_post(
			apply_filters( 'ph_new_comment_location_args', array(
				'post_title'  => __( 'Thread started by', 'project-huddle' ) . ' ' . esc_html( $this->user->display_name ) . ' ' . __( 'on', 'project-huddle' ) . ' ' . get_the_title( $this->data['image_id'] ),
				'post_type'   => 'ph_comment_location',
				'post_status' => 'publish',
				'tax_input'   => $term_id ? array( 'status' => $term_id ) : false,
				'meta_input'  => array(
					'project_comment_id' => (int) $this->data['image_id'],
					'order'              => (int) $this->data['order'],
					'relativeX'          => floatval( $this->data['relativeX'] ),
					'relativeY'          => floatval( $this->data['relativeY'] )
				)
			) ), true );

		// update existing comment locations associated with image
		$comment_locations   = (array) get_post_meta( $this->data['image_id'], 'comment_locations', true );
		$comment_locations[] = (int) $this->data['id'];
		update_post_meta( $this->data['image_id'], 'comment_locations', $comment_locations );

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		return $this->data;
	}

	/**
	 * Edit thread
	 *
	 * @param $data object Posted Model data
	 *
	 * @return object Updated model
	 */
	public function edit_thread( $data ) {
		$this->data = $data->get_params();

		// Update the post into the database
		$this->data['id'] = wp_update_post( array(
			'ID'         => $this->data['id'],
			'meta_input' => array(
				'order'     => (int) $this->data['order'],
				'relativeX' => floatval( $this->data['relativeX'] ),
				'relativeY' => floatval( $this->data['relativeY'] )
			)
		) );

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		return $this->data;
	}

	/**
	 * Delete thread
	 *
	 * @param $data object Posted Model data
	 *
	 * @return mixed|WP_Error object Updated model or error
	 */
	public function delete_thread( $data ) {
		$this->data = $data->get_params();

		// trashing post also trashed comments
		$trashed = wp_delete_post( $this->data['id'] );

		if ( ! $trashed ) {
			return new WP_Error( 'did_not_trash_comment', __( 'Something went wrong whe trying to trash the comment thread. Please reload the page and try again.', 'project-huddle' ) );
		}

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		return $this->data;
	}

	/**
	 * Post New Comment
	 *
	 * @param $data object Posted Model data
	 *
	 * @return mixed|WP_Error object Updated model or error
	 */
	public function new_comment( $data ) {
		$this->data = $data->get_params();
		$this->user = is_user_logged_in() ? wp_get_current_user() : (object) $data['user'];

		// Validate model data
		if ( ! isset( $this->data['comment_content'] ) || ! $this->data['comment_content'] || $this->data['comment_content'] == '<p><br></p>' ) {
			return new WP_Error( 'no_comment_text', __( 'No comment text is entered. Please enter something to post.', 'project-huddle' ) );
		}
		if ( ! isset( $this->data['comment_post_ID'] ) || ! $this->data['comment_post_ID'] ) {
			return new WP_Error( 'no_comment_post_ID', __( 'No comment post ID is specified', 'project-huddle' ) );
		}

		/**
		 * Create and sanitize new comment
		 */
		$comment_id = wp_insert_comment( wp_filter_comment( array(
			'comment_post_ID'      => (int) $this->data['comment_post_ID'],
			'comment_content'      => wp_kses_post( $this->data['comment_content'] ),
			'user_id'              => $this->user->ID,
			'comment_date'         => current_time( 'mysql' ),
			'comment_date_gmt'     => current_time( 'mysql', 1 ),
			'comment_approved'     => 1,
			'comment_author'       => $this->user->display_name,
			'comment_author_IP'    => '',
			'comment_author_url'   => '',
			'comment_author_email' => $this->user->user_email,
			'comment_type'         => 'ph_image_comment'
		) ) );

		if ( ! $comment_id ) {
			return new WP_Error( 'did_not_insert_comment', __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// get created comment
		$the_comment = get_comment( $comment_id );

		// remove escaping slashes from comment content
		$this->data['comment_content']      = wp_kses_post( $this->data['comment_content'] );
		$this->data['id']                   = $comment_id;
		$this->data['date']                 = ph_human_time_diff( mysql2date( 'U', $the_comment->comment_date, true ), current_time( 'timestamp' ) );
		$this->data['avatar']               = get_avatar( $the_comment->comment_author_email, 64, '', $the_comment->comment_author, array(
			'force_display' => true
		) );
		$this->data['comment_author']       = $the_comment->comment_author;
		$this->data['comment_author_email'] = $the_comment->comment_author_email;

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		// update live model on success
		return $this->data;
	}

	/**
	 * Edit comment
	 *
	 * @param $data object Posted Model data
	 *
	 * @return mixed|WP_Error object Updated model or error
	 */
	public function edit_comment( $data ) {
		$this->data = $data->get_params();

		// validate
		if ( ! $this->data['comment_content'] ) {
			return new WP_Error( 'no_comment_text', __( 'No comment text is entered. Please enter something to post.', 'project-huddle' ) );
		}
		// permissions
		if ( ! current_user_can( 'moderate_ph-project_comments', $this->data['project_id'] ) && get_comment_author_email( $this->data['id'] ) != $data['user']['user_email'] ) {
			return new WP_Error( 'no_comment_text', __( 'You don\'t have permission to edit another person\'s comment!', 'project-huddle' ) );
		}

		// update, assume success in case content is the same
		wp_update_comment( array(
			'comment_post_ID' => (int) $this->data['comment_post_ID'],
			'comment_ID'      => (int) $this->data['id'],
			'comment_content' => wp_kses_post( $this->data['comment_content'] )
		) );

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		// return model
		return $this->data;
	}

	/**
	 * Trash Comment
	 *
	 * @param $data object Posted Model data
	 *
	 * @return mixed|WP_Error object Updated model or error
	 */
	public function delete_comment( $data ) {
		$this->data = $data->get_params();

		$project_id = PH()->comment->get_project_from_comment( $this->data['id'] );

		if ( ! isset( $data['user']['user_email'] ) ) {
			$user                             = wp_get_current_user();
			$this->data['user']['user_email'] = $user->user_email;
		}

		// permissions
		if ( ! current_user_can( 'moderate_ph-project_comments', $project_id ) && get_comment_author_email( $this->data['id'] ) != $this->data['user']['user_email'] ) {
			return new WP_Error( 'no_comment_text', __( 'You don\'t have permission to delete this comment!', 'project-huddle' ) );
		}

		$trashed = wp_trash_comment( $data['id'] );

		if ( ! $trashed ) {
			return new WP_Error( 'did_not_trash_comment', __( 'Something went wrong whe trying to trash the comment. Please reload the page and try again.', 'project-huddle' ) );
		}

		// add filters and actions
		$this->data = $this->filter_action( __FUNCTION__, $this->data );

		return $this->data;
	}
}