<?php
/**
 * ProjectHuddle API Endpoints
 *
 * Creates rest routes, sanitizes and validates, adds filters to hook functions.
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2016, Andre Gagnon
 * @since       1.3.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Admin_Project_Images Class
 *
 * This class handles the all the ajax requests made by the plugin
 *
 * @since 1.0.0
 */
class PH_Project_API {

	/**
	 * Current API Version
	 *
	 * @var string
	 */
	protected $version = 'v1';

	/**
	 * Prefix to our filters
	 *
	 * @var string
	 */
	protected $prefix = 'ph_';

	/**
	 * Namespace for our API
	 *
	 * @var string
	 */
	protected $api_name = 'projecthuddle';

	/**
	 * Store route URL
	 * @var
	 */
	protected $route_url;

	/**
	 * Setup project admin
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// route URL for project api
		$this->route_url = $this->api_name . '/' . $this->version . '/mockup';

		// add rest API Endpoints
		add_action( 'rest_api_init', array( $this, 'register_routes' ) );
	}

	/**
	 * Checks the identity of user
	 *
	 * @param $post_data {array} data from api request
	 *
	 * @since 1.0.0
	 * @return WP_Error | true
	 */
	public function check_identity( $post_data ) {
		// get params
		$data = $post_data->get_params();

		// get user email
		$user       = wp_get_current_user();
		$user_email = $user->user_email ? esc_attr( $user->user_email ) : $data['user']['user_email'];

		if ( ! $data['project_id'] ) {
			return new WP_Error( 'broke', __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// check user has an identity
		if ( ! isset($user_email) || ! $user_email ) {
			return new WP_Error( 'login_first', __( 'You must identify yourself! Please login or add a comment first.', 'project-huddle' ) );
		}

		// validate email address
		if ( ! filter_var( trim( $user_email ), FILTER_VALIDATE_EMAIL ) ) {
			return new WP_Error( 'invalid', __( 'Your email is not valid. Please reload the page and try again.', 'project-huddle' ) );
		}

		// check trash
		if ( isset( $data->project_id ) ) {
			$status = get_post_status( $data->project_id );
			if ( $status == 'trash' || $status === false ) {
				return new WP_Error( 'project_trashed', __( 'It looks like this project is trashed. Please untrash the project or select a new project', 'project-huddle' ) );
			}
		}
		if ( isset( $data->image_id ) ) {
			$status = get_post_status( $data->image_id );
			if ( $status == 'trash' || $status === false ) {
				return new WP_Error( 'image_trashed', __( 'It looks like this image is trashed. Please untrash the image or select a new project', 'project-huddle' ) );
			}
		}
		if ( isset( $data->comment_post_ID ) ) {
			$status = get_post_status( $data->comment_post_ID );
			if ( $status == 'trash' || $status === false ) {
				return new WP_Error( 'thread_trashed', __( 'It looks like this comment thread is trashed. Please untrash this comment thread or select a new project', 'project-huddle' ) );
			}
		}

		// if public comments aren't allowed
		if ( get_post_meta( $data['project_id'], 'ph_project_comments', true ) != 'on' ) {
			// user should be logged in
			if ( ! is_user_logged_in() ) {
				return new WP_Error( 'login_first', __( 'You need a user account to do that. Please login or signup.', 'project-huddle' ) );
			}
			// user should have read permissions
			if ( ! current_user_can( 'read_ph-project', $data['project_id'] ) ) {
				return new WP_Error( 'no_permissions', __( 'Sorry! You don\'t have permissions to add comments to this project.', 'project-huddle' ) );
			}
		}

		return true;
	}

	public function allow_tags() {
		global $allowedtags;

		$allowedtags['br'] = array();
		$allowedtags['p']  = array();
		$allowedtags['ol'] = array();
		$allowedtags['ul'] = array();
		$allowedtags['li'] = array();
	}

	/**
	 * Register our endpoints
	 */
	public function register_routes() {
		$this->test_routes();
		$this->project_routes();
		$this->user_routes();
		$this->image_routes();
		$this->thread_routes();
		$this->comment_routes();
	}

	public function test_routes() {
		register_rest_route( $this->route_url, '/test',
			array(
				'methods'  => array( 'GET', 'POST', 'PATCH', 'PUT', 'DELETE' ),
				'callback' => function ( $data ) {
					return true;
				}
			)
		);
	}

	public function project_routes() {
		register_rest_route( $this->route_url, '/(?P<id>\d+)',
			array(
				'methods'  => array( 'GET' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_get_project', $data ); // make callback filterable
				},
			)
		);
	}

	/**
	 * Page Thread Routes
	 */
	public function user_routes() {
		$base = 'user';

		// Get
		register_rest_route( $this->api_name . '/' . $this->version, '/' . $base ,
			array(
				'methods'  => array( 'GET' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'get_users', $data ); // make callback filterable
				},
			)
		);

		// Get
		register_rest_route( $this->api_name . '/' . $this->version, '/' . $base . '/login',
			array(
				'methods'  => array( 'POST' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'login_user', $data ); // make callback filterable
				}
			)
		);

		// Email notifications suppression add
		register_rest_route( $this->api_name . '/' . $this->version, '/' . $base . '/suppression/email/project/(?P<id>\d+)',
			array(
				'methods'             => array( 'PUT', 'PATCH' ),
				'permission_callback' => 'is_user_logged_in',
				'callback'            => function ( $data ) {
					return apply_filters( $this->prefix . 'add_suppression', $data ); // make callback filterable
				}
			)
		);

		// Email notifications suppression remove
		register_rest_route( $this->api_name . '/' . $this->version, '/' . $base . '/suppression/email/project/(?P<id>\d+)',
			array(
				'methods'             => array( 'DELETE' ),
				'permission_callback' => 'is_user_logged_in',
				'callback'            => function ( $data ) {
					return apply_filters( $this->prefix . 'remove_suppression', $data ); // make callback filterable
				}
			)
		);
	}

	/**
	 * Comment Thread Routes
	 * mockup/comment-thread
	 */
	public function thread_routes() {
		$base = 'comment-thread';

		// GET thread
		register_rest_route( $this->route_url, '/' . $base,
			array(
				'methods'  => array( 'GET' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_get_thread', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function ($data) {
					return $this->check_identity($data);
				}
			)
		);

		// POST new
		register_rest_route( $this->route_url, '/' . $base,
			array(
				'methods'  => array( 'POST' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_new_thread', $data ); // make callback filterable
				},
				'permission_callback' => function ($data) {
					return $this->check_identity($data);
				}
			)
		);

		// EDIT existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_edit_thread', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);

		// resolve
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/resolve',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_resolve_thread', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);

		// EDIT existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/assign',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_assign_thread', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);

		// DELETE existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'DELETE' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_delete_thread', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);

		$base = 'thread/reorder';
		// Update
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'POST', 'PUT', 'PATCH', 'GET' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_reorder_thread', $data ); // make callback filterable
				},
			)
		);
	}

	/**
	 * Comment Thread Routes
	 */
		public function comment_routes() {
		$base = 'comment';

		// POST new
		register_rest_route( $this->route_url, '/' . $base,
			array(
				'methods'             => array( 'POST' ),
				'callback'            => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_new_comment', $data ); // make callback filterable
				},
				'permission_callback' => function ( $data ) {
					return $this->check_identity( $data );
				}
			)
		);

		// PUT to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_edit_comment', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);

		// DELETE to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'DELETE' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_delete_comment', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);
	}

	/**
	 * Comment Thread Routes
	 * RestURL + /mockup/image
	 */
	public function image_routes() {
		$base = 'image';

		// GET
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'GET' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_get_image', $data ); // make callback filterable
				},
				'args'     => array(
					'project_id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);

		// POST new
		register_rest_route( $this->route_url, '/' . $base,
			array(
				'methods'  => array( 'POST' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_new_image', $data ); // make callback filterable
				},
				'args'     => array(
					'project_id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				),
				'permission_callback' => function ($data) {
					return $this->check_identity($data);
				}
			)
		);

		// PUT to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_edit_image', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					)
				)
			)
		);

		// PUT to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)/actions/share',
			array(
				'methods'  => array( 'PUT', 'PATCH' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_share_image', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					)
				)
			)
		);

		// DELETE to existing
		register_rest_route( $this->route_url, '/' . $base . '/(?P<id>\d+)',
			array(
				'methods'  => array( 'DELETE' ),
				'callback' => function ( $data ) {
					return apply_filters( $this->prefix . 'mockup_delete_image', $data ); // make callback filterable
				},
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
					'project_id' => array(
						'sanitize_callback' => 'absint', // only non-negative integers allowed
					),
				)
			)
		);
	}
}
