<?php
/**
 * AJAX Functions
 *
 * Process the front-end AJAX actions.
 *
 * @deprecated  2.1.0
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Admin_Project_Images Class
 *
 * This class handles the all the ajax requests made by the plugin
 *
 * @since 1.0.0
 */
class PH_Ajax_Events {

	/**
	 * Setup project admin
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->add_ajax_events();
	}

	public function add_ajax_events() {

		// event => nopriv
		$ajax_events = array(
			'ph_save_comment_location'   => true,
			'ph_save_project_comment'    => true,
			'ph_move_comment_to_trash'   => true,
			'ph_update_project_comment'  => true,
			'ph_update_comment_location' => true,
			'ph_update_location_order'   => true,
			'ph_move_location_to_trash'  => true,
			'ph_user_login'              => true,
			'ph_user_logout'             => true,
			'ph_share_email'             => true,
			'move_image_to_trash'        => false
		);

		// register each event
		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_' . $ajax_event, array( $this, $ajax_event ) );

			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_' . $ajax_event, array( $this, $ajax_event ) );
				// ph AJAX can be used for frontend ajax requests
				add_action( 'ph_ajax_' . $ajax_event, array( $this, $ajax_event ) );
			}
		}

	}

	/**
	 * Verify nonce for all functions
	 *
	 * @since 1.0.0
	 *
	 * @param $post_data $_POST data
	 */
	public function verify_nonce( $post_data ) {

		// make sure post is set
		if ( empty( $post_data ) ) {
			error_log( 'ProjectHuddle: verify_nonce returned an error. $post_data is empty.' );
			wp_send_json_error( __( 'Hmmmm... something went wrong. Please try again.', 'project-huddle' ) );
		}
		// verify nonce
		if ( ! wp_verify_nonce( $post_data['nonce'], 'project-image-nonce' ) ) {
			wp_send_json_error( __( 'Your session expired. Please reload the page again.', 'project-huddle' ) );
		}

	}

	/**
	 * Checks the identity of the ajax poster
	 *
	 * @param $post_data {array} $_POST data from ajax request
	 *
	 * @since 1.0.0
	 */
	public function checkIdentity( $post_data ) {

		// get user email and image id
		$user_email = (array) $post_data['user'];

		// validate post ID with server
		$project_id = (int) url_to_postid( wp_get_referer() );

		// sanitize
		$user_email['email'] = esc_attr( $user_email['email'] );

		// check if non-logged in comments allowed
		$allow_comments = get_post_meta( $project_id, 'ph_project_comments', true );

		if ( ! $project_id ) {
			error_log( 'ProjectHuddle: Tried to check identity but the post ID was not defined. /n $project_id=' . $project_id );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// check user has an identity
		if ( ! $user_email['email'] || $user_email['email'] == 'false' ) {
			wp_send_json_error( __( 'You must identify yourself! Please login or add a comment first.', 'project-huddle' ) );
		}

		// validate email address
		if ( ! filter_var( trim( $user_email['email'] ), FILTER_VALIDATE_EMAIL ) ) {
			wp_send_json_error( __( 'Your email is not valid. Please reload the page and try again.', 'project-huddle' ) );
		}

		// if public comments aren't allowed
		if ( $allow_comments != 'on' ) {

			// user should be logged in
			if ( ! is_user_logged_in() ) {
				wp_send_json_error( __( 'You need a user account to do that. Please login or signup.', 'project-huddle' ) );
			}
			// user should be admin
			if ( ! is_admin() ) {
				wp_send_json_error( __( 'Sorry! Only administrators are allowed to do that.', 'project-huddle' ) );
			}

		}

	}

	/**
	 * Makes sure nothing is trashed that's not supposed to be
	 *
	 * @param bool|int $thread_id
	 * @param bool|int $image_id
	 * @param bool|int $project_id
	 */
	public function check_trash( $thread_id = false, $image_id = false, $project_id = false ) {
		// make sure project is not trashed or deleted
		if ( $project_id ) {
			$project_status = get_post_status( $project_id );
			if ( $project_status == 'trash' || $project_status === false ) {
				wp_send_json_error( __( 'The image has been removed. Please reload the page.', 'project-huddle' ) );
			}
		}

		// make sure image post is not trashed or deleted
		if ( $image_id ) {
			$image_status = get_post_status( $image_id );
			if ( $image_status == 'trash' || $image_status === false ) {
				wp_send_json_error( __( 'The image has been removed. Please reload the page.', 'project-huddle' ) );
			}
		}

		// make sure comment thread post is not trashed or deleted
		if ( $thread_id ) {
			$thread_status = get_post_status( $thread_id );
			if ( $thread_status == 'trash' || $thread_status === false ) {
				wp_send_json_error( __( 'The comment thread has been removed. Please reload the page.', 'project-huddle' ) );
			}
		}
	}

	/**
	 * Adds user email to project for notifications
	 *
	 * @param array $user       User email and display name
	 * @param int   $project_id Project ID
	 */
	public function add_email( $user, $project_id ) {
		// get all added emails
		$emails = (array) get_post_meta( (int) $project_id, 'ph_project_emails', true );

		// get enabled emails
		$enabled = (array) get_post_meta( (int) $project_id, 'ph_project_emails_enable', true );

		// check for user object
		if ( is_array( $user ) ) {
			$user = $user['email'];
		}

		// if email hasn't already been added
		if ( ! in_array( $user, $emails ) ) {
			// add user email
			$emails[]  = sanitize_email( $user );
			$enabled[] = sanitize_email( $user );

			// add to total email chain
			update_post_meta( (int) $project_id, 'ph_project_emails', $emails );

			// add to enabled emails
			update_post_meta( (int) $project_id, 'ph_project_emails_enable', $enabled );
		}
	}

	/**
	 * Saves comment location
	 *
	 * Creates a new private custom post with x/y coordinates to store comments.
	 * Associated x/y coordinates are stored as meta data
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_save_comment_location_before before save
	 * @uses  do_action() on ph_save_comment_location_after after save
	 *
	 * @return object Success/Fail json object with updated model
	 */
	public function ph_save_comment_location() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		// get model
		$model = $_POST['model'];
		$user  = $_POST['user'];

		// validate data
		if ( ! isset( $model['imageID'] ) ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" image ID is not set.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}
		if ( ! isset( $model['order'] ) ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" location order is not set.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// sanitize
		$model['imageID']   = (int) $model['imageID'];
		$model['order']     = (int) $model['order'];
		$model['relativeX'] = floatval( $model['relativeX'] );
		$model['relativeY'] = floatval( $model['relativeY'] );

		$title = __( 'Thread started by', 'project-huddle' ) . ' ' . esc_html( $user['display_name'] ) . ' ' . __( 'on', 'project-huddle' ) . ' ' . get_the_title( $model['imageID'] );

		// get our default term of backlog
		$term    = get_term_by( 'slug', 'backlog', 'status' );
		$term_id = $term ? $term->term_id : false;

		// arguments to create a new post
		$post_args = apply_filters( 'ph_new_comment_location_args', array(
			'post_title'  => esc_html( $title ),
			'post_type'   => 'ph_comment_location',
			'post_status' => 'publish',
			'tax_input'   => $term_id ? array( 'status' => $term_id ) : false
		) );

		// add pre action
		do_action( 'ph_save_comment_location_before', $model, $post_args );

		// insert new post and store id
		$post_id = wp_insert_post( $post_args, true );

		// add post action
		do_action( 'ph_save_comment_location_after', $model, $post_id );

		// store meta data in post
		add_post_meta( $post_id, 'project_comment_id', (int) $model['imageID'] );
		add_post_meta( $post_id, 'order', (int) $model['order'] );
		add_post_meta( $post_id, 'relativeX', floatval( $model['relativeX'] ) );
		add_post_meta( $post_id, 'relativeY', floatval( $model['relativeY'] ) );

		// get existing comments associated with image
		$comment_locations = get_post_meta( $model['imageID'], 'comment_locations', true );

		// if set, add our comment post to the list
		// otherwise, start a comment locations array
		if ( isset( $comment_locations ) && is_array( $comment_locations ) ) {
			$comment_locations[] = $post_id;
		} else {
			$comment_locations = array( $post_id );
		}

		// update locations
		update_post_meta( $model['imageID'], 'comment_locations', $comment_locations );

		// add custom action
		do_action( 'ph_save_comment_location_meta', $model, $comment_locations );

		// update current location id in model
		$model['id'] = $post_id;

		// update live model on success
		wp_send_json( $model );
	}

	/**
	 * Saves comment to comment location post
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_insert_comment_before before comment is saved
	 * @uses  do_action() on ph_insert_comment_after after comment is saved
	 *
	 * @return object Success/Fail json object with updated model
	 */
	public function ph_save_project_comment() {
		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		/**
		 * Get model and set defaults
		 */
		$model    = (array) $_POST['model'];
		$user     = (array) $_POST['user'];
		$image_id = (int) $_POST['image_id'];

		// validate post ID with server
		$project_id = (int) url_to_postid( wp_get_referer() );

		// before comment hook
		do_action( 'ph_insert_comment_before', $model, $project_id, $image_id, $user );

		// make sure nothing's trashed
		$this->check_trash( $model['comment_post_ID'], $project_id, $image_id );

		if ( ! isset( $image_id ) || $image_id == false ) {
			wp_send_json_error( __( 'Something went wrong. Please try again', 'project-huddle' ) );
		}

		//Validate model data
		if ( ! isset( $model['comment_content'] ) || $model['comment_content'] == '' ) {
			wp_send_json_error( __( 'No comment text is entered. Please enter something to post.', 'project-huddle' ) );
		}

		/**
		 * Create and sanitize new comment
		 */
		$comment_id = wp_insert_comment( wp_filter_comment( array(
			'comment_post_ID'      => (int) $model['comment_post_ID'],
			'comment_content'      => stripslashes( $model['comment_content'] ),
			'user_id'              => $model['user_id'],
			'comment_date'         => current_time( 'mysql' ),
			'comment_date_gmt'     => current_time( 'mysql', 1 ),
			'comment_approved'     => 1,
			'comment_author'       => $model['comment_author'],
			'comment_author_IP'    => '',
			'comment_author_url'   => '',
			'comment_author_email' => $model['comment_author_email'],
			'comment_type'         => 'ph_image_comment'
		) ) );

		if ( ! $comment_id ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" could not insert comment.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		do_action( 'ph_insert_comment_after', $comment_id, $model, $image_id, $user );

		// get created comment
		/** @noinspection PhpParamsInspection */
		$the_comment = get_comment( $comment_id );

		// get comment date
		$comment_date = mysql2date( 'U', $the_comment->comment_date, true );

		// remove escaping slashes from comment content
		$model['comment_content'] = stripslashes( $model['comment_content'] );

		// store comment ID and date model
		$model['id']     = $comment_id;
		$model['date']   = ph_human_time_diff( $comment_date, current_time( 'timestamp' ) );
		$model['avatar'] = get_avatar( $model['comment_author_email'], 64 );

		$this->add_email( $user, $project_id );

		// send email
		ph_email_image_comment( $user, $model, $image_id, $project_id );

		// update live model
		wp_send_json( $model );
	}

	/**
	 * Moves comment to trash
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_trash_comment_before before comment is trashed
	 * @uses  do_action() on ph_trash_comment_after after comment is trashed
	 *
	 * @return object Success/Fail json object with updated model
	 */
	public function ph_move_comment_to_trash() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		// Get model
		$model = (array) $_POST['model'];

		do_action( 'ph_trash_comment_before', $model );

		// validate comment_ID
		if ( ! isset( $model['comment_ID'] ) || ! $model['comment_ID'] || ! is_numeric( $model['comment_ID'] ) ) {
			wp_send_json_error( __( 'This comment is not yet saved!', 'project-huddle' ) );
		}

		// get comment
		$comment = get_comment( $model['comment_ID'] );

		// trash comment
		$trashed = wp_trash_comment( $model['comment_ID'] );

		// trash comment location post if comment thread is empty
		if ( isset( $comment->comment_post_ID ) ) {
			$number = get_comments_number( $comment->comment_post_ID );

			if ( $number == '0' ) {
				wp_trash_post( $comment->comment_post_ID );
			}
		}

		do_action( 'ph_trash_comment_after', $model );

		if ( ! $trashed ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" could not trash comment.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// send update
		wp_send_json_success( $model );
	}

	/**
	 * Updates comment location via ajax
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_update_comment_location_before before comment location is updated
	 * @uses  do_action() on ph_update_comment_location_after after comment location is updated
	 *
	 * @return object Success/Fail json object
	 */
	public function ph_update_comment_location() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// identity check
		$this->checkIdentity( $_POST );

		// Get model
		$model = (array) $_POST['model'];

		// pre action
		do_action( 'ph_update_comment_location_before', $model );

		// needs advanced permissions
		if ( ! current_user_can( 'edit_ph-project', (int) $model['id'] ) ) {
			wp_send_json_error( __( 'You don\'t have permission to do that.', 'project-huddle' ) );
		}

		// check if model has been saved yet
		if ( ! isset( $model['id'] ) || ! $model['id'] ) {
			wp_send_json_success();
		}

		// update location
		update_post_meta( (int) $model['id'], 'relativeX', floatval( $model['relativeX'] ) );
		update_post_meta( (int) $model['id'], 'relativeY', floatval( $model['relativeY'] ) );

		// completed action
		do_action( 'ph_update_comment_location_after', $model );

		// send success
		wp_send_json_success();
	}

	/**
	 * Updates comment text
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_update_comment_before before comment is updated
	 * @uses  do_action() on ph_update_comment_after after comment is updated
	 *
	 * @return object Success/Fail json object
	 */
	public function ph_update_project_comment() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		// Get model
		$model = (array) $_POST['model'];

		// validate
		if ( ! isset( $model['comment_content'] ) || ! $model['comment_content'] ) {
			wp_send_json_error( __( 'You must enter comment text.', 'project-huddle' ) );
		}

		// pre action
		do_action( 'ph_update_comment_before', $model );

		$model['comment_content'] = stripslashes( $model['comment_content'] );

		$comment                    = array();
		$comment['comment_ID']      = (int) $model['comment_ID'];
		$comment['comment_content'] = esc_html( $model['comment_content'] );

		// update comment
		$updated = wp_update_comment( $comment );

		// check success
		if ( ! $updated ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" could not updated comment.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// completed action
		do_action( 'ph_update_comment_after', $model );

		// send success
		wp_send_json_success();
	}

	/**
	 * Updates post meta location order attribute in database
	 * Mostly used for when location posts are deleted and order changes
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_update_location_order_before before comment post order is updated
	 * @uses  do_action() on ph_update_location_order_after after comment post order is updated
	 *
	 * @return object Success json object
	 */
	public function ph_update_location_order() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// Get model
		$model = (array) $_POST['model'];

		// pre action
		do_action( 'ph_update_location_order_before', $model );

		// validate comment_ID
		if ( ! isset( $model['order'] ) ) {
			wp_send_json_error( __( 'There is no order set for the location.', 'project-huddle' ) );
		}

		// update location
		update_post_meta( (int) $model['id'], 'order', (int) $model['order'] );

		// post action
		do_action( 'ph_update_location_order_after', $model );

		// send success
		wp_send_json_success();
	}

	/**
	 * Moves comment location post to trash
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_trash_comment_location_before before comment post is trashed
	 * @uses  do_action() on ph_trash_comment_location_after after comment post is trashed
	 *
	 * @return object Success/Fail json object with updated model
	 */
	public function ph_move_location_to_trash() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		// Get model
		$model = (array) $_POST['model'];

		// pre action
		do_action( 'ph_trash_comment_location_before', $model );

		// validate comment_ID
		if ( ! isset( $model['id'] ) || ! $model['id'] || is_nan( $model['id'] ) ) {
			wp_send_json_error( __( 'There is no ID set for the comment.', 'project-huddle' ) );
		}

		// advanced moderation permissions
		if ( ! current_user_can( 'edit_ph-project', (int) $model['id'] ) ) {
			wp_send_json_error( __( 'You don\'t have permission to do that.', 'project-huddle' ) );
		}

		// get saved parent image id
		$imageID = get_post_meta( (int) $model['id'], 'project_comment_id', true );

		// get parent locations array
		$locations = get_post_meta( (int) $imageID, 'comment_locations', true );

		// unset current image from array
		if ( ( $key = array_search( (int) $model['id'], $locations ) ) !== false ) {
			unset( $locations[ $key ] );
		}

		// update meta
		update_post_meta( (int) $imageID, 'comment_locations', $locations );

		// delete post
		$trashed = wp_trash_post( (int) $model['id'] );

		if ( ! $trashed ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" could not trash comment location.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// post action
		do_action( 'ph_trash_comment_location_after', $model );

		// send update
		wp_send_json_success( $model );
	}

	/**
	 * Logs in user using username and password
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_user_signon_before before user is signed on
	 * @uses  do_action() on ph_user_signon_after after user is signed on
	 *
	 * @return object Success/Fail json object with updated user model
	 */
	public function ph_user_login() {

		// make sure not already logged in
		if ( is_user_logged_in() ) {
			wp_send_json_success( __( 'You are already logged in', 'project-huddle' ) );
		}

		// verify nonce
		$this->verify_nonce( $_POST );

		$info  = array();
		$model = $_POST['model']; // get model

		$info['user_login']    = $model['username'];
		$info['user_password'] = $model['password'];
		$info['remember']      = true;

		// check for empty fields
		if ( empty( $info['user_login'] ) || empty( $info['user_password'] ) ) {
			wp_send_json_error( __( 'The username or password is cannot be empty', 'project-huddle' ) );
		}

		do_action( 'ph_user_signon_before', $model );

		// sign-in
		$user_signon = wp_signon( $info, false );

		// validate
		if ( is_wp_error( $user_signon ) ) {
			if ( $user_signon->get_error_code() == "invalid_username" || $user_signon->get_error_code() == "incorrect_password" ) {
				wp_send_json_error( __( 'Wrong username or password. Please try again.', 'project-huddle' ) );
			} else {
				wp_send_json_error( __( 'There was an error logging you in. Please try again.', 'project-huddle' ) );
			}
		}

		$user_info             = get_userdata( $user_signon->ID );
		$model['display_name'] = $user_info->display_name;
		$model['user_email']   = $user_info->user_email;
		$model['ID']           = $user_signon->ID;

		do_action( 'ph_user_signon_after', $model );

		wp_send_json( $model );
	}

	/**
	 * Logs out user
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_user_logout_before before user is logged out
	 * @uses  do_action() on ph_user_logout_after after user is logged out
	 *
	 * @return object Success json with redirect message
	 */
	public function ph_user_logout() {

		// make sure not already logged in
		if ( ! is_user_logged_in() ) {
			wp_send_json_success( __( 'You are already logged out.', 'project-huddle' ) );
		}

		// verify nonce
		$this->verify_nonce( $_POST );

		// pre action
		do_action( 'ph_user_logout_before' );

		// logout
		wp_logout();

		// post action
		do_action( 'ph_user_logout_after' );

		// send success
		wp_send_json_success( __( 'You\'ve successfully logged out. Redirecting...', 'project-huddle' ) );
	}

	/**
	 * Sends email with share link
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_share_email_before before email is sent
	 * @uses  do_action() on ph_share_email_after after email is sent
	 *
	 * @return object Success/Fail json object with updated user model
	 */
	public function ph_share_email() {

		// verify nonce
		$this->verify_nonce( $_POST );

		// check identity
		$this->checkIdentity( $_POST );

		// Get model
		$model = (array) $_POST['model'];
		$user  = (array) $_POST['user'];
		$email = (array) $model['email'];

		// validate post ID with server
		$project_id = (int) url_to_postid( wp_get_referer() );

		// make sure post is set
		if ( empty( $model['email'] ) || $model['email'] == '' ) {
			wp_send_json_error( __( 'Hmmmm... something went wrong. Email not set. Please try again.', 'project-huddle' ) );
		}

		// check honeypot for spam prevention
		if ( ! empty( $email['website'] ) || $email['website'] != '' ) {
			wp_send_json_error( __( 'Hmmmm... something went wrong. Website not set. Please try again.', 'project-huddle' ) );
		}
		// validate form
		if ( empty( $email['subject'] ) || $email['subject'] == '' ) {
			wp_send_json_error( __( 'You must enter a subject for your email.', 'project-huddle' ) );
		}
		// check message
		if ( empty( $email['message'] ) || $email['message'] == '' ) {
			wp_send_json_error( __( 'You must enter a message for your email.', 'project-huddle' ) );
		}

		do_action( 'ph_share_email_before', $email, $user );

		// check email sharing permissions
		if ( get_post_meta( $project_id, 'ph_project_sharing', true ) == 'off' ) {
			wp_send_json_error( __( 'Email sharing is not enabled for this project.', 'project-huddle' ) );
		}

		ph_email_share_comment( $email['address'], $email['subject'], $email['message'], $email['link'], $user );

		do_action( 'ph_share_email_after', $email, $user );

		wp_send_json_success( __( 'Email sent!', 'project-huddle' ) );
	}

	/**
	 * Saves project images
	 *
	 * @since 1.0.0
	 *
	 * @uses  do_action() on ph_move_image_to_trash_before before image is trashed
	 * @uses  do_action() on ph_move_image_to_trash_after after image is trashed
	 *
	 * @return object Success/Fail json object
	 */
	public function move_image_to_trash() {
		// verify nonce
		$this->verify_nonce( $_POST );

		// get model
		$model = (array) $_POST['model'];

		// check id
		if ( ! $model['ID'] ) {
			wp_send_json_error( __( 'Something went wrong. :(', 'project-huddle' ) );
		}

		// don't trash other post types
		if ( get_post_type( $model['ID'] ) != 'project_image' ) {
			wp_send_json_success();
		}

		// post action
		do_action( 'ph_move_image_to_trash_before', $model );

		// trash image
		$trashed = wp_trash_post( $model['ID'] );

		if ( ! $trashed ) {
			error_log( 'ProjectHuddle: Method "' . __METHOD__ . '" could not trash project image.' );
			wp_send_json_error( __( 'Something went wrong. Please reload the page and try again.', 'project-huddle' ) );
		}

		// completed action
		do_action( 'ph_move_image_to_trash_after', $model );

		// Debug info
		wp_send_json_success();
	}
}