<?php
/**
 * Comment Data Class
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Comment Class
 *
 * Child class of PH_Project
 *
 * This class handles the comment data
 *
 * @since 1.0
 */
class PH_Comment {

	/**
	 * Queries and returns comments by project id.
	 * Combines them by location.
	 *
	 * @param int $id ID of project
	 *
	 * @return array $comments Comment data
	 */
	public function get_comment_items( $id = 1 ) {

		do_action( 'ph_get_comment_items_before', $id );

		// get the array of comment location ids
		$comment_locations = get_post_meta( $id, 'comment_locations', true );

		// combine array
		$location_combined = array();

		// get comments for each location stored in post meta
		if ( isset( $comment_locations ) && is_array( $comment_locations ) ) {
			foreach ( $comment_locations as $key => $location ) {

				// continue if comment post isn't public
				if ( ! $location || get_post_status( $location ) != 'publish' ) {
					continue;
				}

				// comment args
				$args = array(
					'post_id'    => $location,
					'orderby'    => 'date',
					'order'      => 'ASC',
					'ph_comment' => true
				);
				$comments = get_comments( $args );

				// comment avatar and date and decode
				foreach ( $comments as $comment ) {
					$comment->id     = $comment->comment_ID;
					$comment->avatar = get_avatar( $comment->comment_author_email, 64, '', $comment->comment_author, array(
						'force_display' => true
					) );
					$comment->date   = ph_human_time_diff( mysql2date( 'U', $comment->comment_date, true ), current_time( 'timestamp' ) );
				}

				// query comments by post id
				$location_combined[ $location ] = array(
					'id'                 => $location,
					'x'                  => floatval( get_post_meta( $location, 'x', true ) ),
					'y'                  => floatval( get_post_meta( $location, 'y', true ) ),
					'relativeX'          => floatval( get_post_meta( $location, 'relativeX', true ) ),
					'relativeY'          => floatval( get_post_meta( $location, 'relativeY', true ) ),
					'order'              => (int) get_post_meta( $location, 'order', true ),
					'project_comment_id' => (int) get_post_meta( $location, 'project_comment_id', true ),
					'items'              => $comments
				);
			}
		}

		// kill the keys
		$comments = array_values( $location_combined );

		do_action( 'ph_get_comment_items_after', $comments );

		return $comments;
	}

	/**
	 * Get number of comment threads on a project
	 *
	 * @param $id
	 *
	 * @return int
	 */
	public function get_comment_thread_count( $id ) {
		$comment_threads = get_post_meta( $id, 'comment_locations', true );

		if ( empty( $comment_threads ) ) {
			return 0;
		}

		foreach( $comment_threads as $key => $comment ) {
			// continue if comment post isn't public
			if ( get_post_status( $comment ) != 'publish' ) {
				unset($comment_threads[$key]);
			}
		}
		return ! empty($comment_threads) ? count($comment_threads) : 0;
	}

	/**
	 * Gets the number of comments per image
	 *
	 * @param $id
	 *
	 * @return int Total comment count
	 * @since 1.0
	 */
	public function get_comment_count( $id ) {
		// set count and get locations
		$count     = 0;
		$locations = get_post_meta( $id, 'comment_locations', true );

		// get number of comments in each location
		if ( $locations && is_array( $locations ) ) {
			foreach ( $locations as $location ) {
				if ( get_post_status ( $location ) != 'publish') continue;
				$count = $count + (int) get_comments_number( $location );
			}
		}

		return $count;
	}

	/**
	 * Get thread from comment
	 *
	 * @param $comment
	 *
	 * @return int Thread ID
	 */
	public function get_thread( $comment ) {
		if ( is_int( $comment ) ) {
			$comment = get_comment( $comment );
		}
		return $comment->comment_post_ID;
	}

	/**
	 * Get Image from Thread
	 *
	 * @param $comment
	 *
	 * @return mixed
	 */
	public function get_image( $comment ) {
		return get_post_meta( $comment, 'project_comment_id', true );
	}

	public function get_project_from_thread( $id ) {
		return (int) PH()->image->get_project( $this->get_image( $id ) );
	}

	/**
	 * Get Project From Comment ID
	 *
	 * @param $id int ID of comment or comment thread post
	 *
	 * @return int ID of project
	 */
	public function get_project_from_comment( $id ) {
		return (int) PH()->image->get_project( $this->get_image( $this->get_thread( $id ) ) );
	}
}