<?php
/**
 * Email Functions
 *
 * Functions that send specific emails
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Turns email link into formated button
 *
 * @param       $link
 * @param       $text
 * @param array $attrs
 *
 * @return mixed|void
 */
function ph_email_link( $link, $text, $attrs = array() ) {
	$defaults = array(
		'style' => 'text-align:inherit; margin-top: 35px; margin-bottom: 0;'
	);

	$attrs = wp_parse_args( $attrs, $defaults );

	// force block
	if ( $attrs['style'] ) {
		$attrs['style'] = $attrs['style'] . 'display:block;';
	}

	$attrs_string = '';

	foreach ( $attrs as $field => $attr ) {
		$attr         = esc_html( $attr );
		$attrs_string = "$field=\"$attr\" ";
	}

	return apply_filters( 'ph_email_link', '<span ' . $attrs_string . '>
		<a bgcolor="' . get_option( 'ph_highlight_color', '#1b8ff2' ) . '" height="40" href="' . esc_url( $link ) . '" style="display:inline-block;margin:0 auto;background-color:' . get_option( 'ph_highlight_color', '#1b8ff2' ) . ';color:#fff;min-width:200px;padding:0 25px;text-align:center;text-decoration:none;height:40px;line-height:40px;width:auto;border-radius:3px;font-size:14px; font-family:Helvetica Neue,Helvetica,Arial,sans-serif;" width="auto" target="_blank">
		' . esc_html( $text ) . '</a></span>', $link, $text, $attrs );
}

/**
 * Gets all email addresses for users on the project
 *
 * @since 1.0.0
 *
 * @param int   $project_id ID of the project
 * @param mixed $user       If a user array is set, check against list so they don't get their own notification
 *
 * @return array List of emails
 */
function ph_get_project_emails( $project_id, $user = false ) {
	$emails = (array) apply_filters( 'ph_project_emails', get_post_meta( (int) $project_id, 'ph_project_emails_enable', true ) );

	// normalize
	if ( is_object( $user ) ) {
		if ( isset( $user->user_email ) ) {
			$user = $user->user_email;
		}
	}

	// normalize
	if ( isset( $user['user_email'] ) ) {
		$user = $user['user_email'];
	}

	// if user doing the action is in the email chain, unset before sending
	// so they don't get their own notification
	if ( ( $key = array_search( $user, $emails ) ) !== false ) {
		unset( $emails[ $key ] );
	}
	$emails = array_values( array_unique( $emails ) );

	foreach ( $emails as $key => $email ) {
		$user = get_user_by( 'email', $email );

		if ( $user && isset( $user->ph_project_email_notifications_disable_all ) ) {
			$disabled = (array) $user->ph_project_email_notifications_disable_all;

			if ( in_array( $project_id, $disabled ) ) {
				unset( $emails[ $key ] );
			}
		}
	}

	// get admin notice option
	$admin_notice = get_option( 'ph_admin_emails' );

	// email admin if applicable
	if ( apply_filters( 'ph_admin_notification_emails', $admin_notice ) == 'on' ) {
		$emails[] = get_option( 'admin_email' );
	}

	return array_filter( array_unique( $emails ) );
}

/**
 * Add an email to a project programmatically
 *
 * @param integer $project_id        ID for the project
 * @param string  $email             Email address to add
 * @param boolean $send_notification Whether to send an email notification
 */
function ph_add_project_member( $project_id, $email, $send_notification = true ) {
	// get subscribed
	$emails = (array) get_post_meta( (int) $project_id, 'ph_project_emails_enable', true );
	array_push( $emails, sanitize_email( $email ) );

	// remove duplicates
	$emails = array_unique($emails);

	// auto email user
	if ( $send_notification ) {
		do_action( 'ph_new_email_admin_subscribe', array( $email ), wp_get_current_user(), $project_id );
	}

	// update meta
	update_post_meta( (int) $project_id, 'ph_project_emails_enable', $emails );
	update_post_meta( (int) $project_id, 'ph_project_emails', $emails );
}

/**
 * Email new user their login information
 *
 * @since  2.7.0
 *
 * @param WP_User         $user    User who sent the image
 * @param WP_REST_Request $request Project Model
 */
function ph_email_new_user_registration( $user, $request ) {
	// to emails
	$to_emails = (array) apply_filters( 'ph_email_new_user_registration_to', array( $user->user_email ) );

	// password reset key
	$key        = get_password_reset_key( $user );
	$reset_link = network_site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode( $user->user_login ), 'login' );

	// send each email individually
	foreach ( $to_emails as $email ) {
		$email = ( new PH_Mail )
			->template( ph_locate_template( 'email/registration-email.php' ), [
				'link'         => ph_email_link( $reset_link, __( 'Set Your Password', 'project-huddle' ) ),
				'username'     => esc_html( $user->user_login ),
				'project_name' => sanitize_text_field( get_the_title( (int) $request['project_id'] ) ),
				'project_link' => esc_url( get_permalink( (int) $request['project_id'] ) ),
			] )
			->to( $email )
			->subject( apply_filters( 'ph_email_new_user_registration_subject', 'Please set your password.' ) )
			->send();
	}
}

add_action( 'ph_rest_insert_user', 'ph_email_new_user_registration', 10, 2 );

/**
 * Email image comment notification to everyone on the project
 *
 * @since    1.0.0
 *
 * @param array $user       User who sent the image
 * @param array $model      Image model
 * @param int   $project_id Project Id
 */
function ph_email_image_comment( $user, $model, $image_id, $project_id ) {
	// url
	$hash = esc_attr( get_post( $image_id )->post_name );
	$url  = esc_url( get_permalink( $project_id ) . '#' . $hash . '/comment/' . (int) $model['comment_post_ID'] );

	// to emails
	$to_emails = (array) apply_filters( 'ph_mockup_new_comment_email_to', ph_get_project_emails( $project_id, $user ), get_post( $model['id'] ) );

	// send each email individually
	foreach ( $to_emails as $email ) {
		$email = ( new PH_Mail )
			->template( ph_locate_template( 'email/mockup-new-comment-email.php' ), [
				'commenter'    => esc_html( $user['display_name'] ),
				'avatar'       => get_avatar( get_current_user_id(), 42, '', '', array(
					'force_display' => true,
					'extra_attr'    => 'style="border-radius: 42px"'
				) ),
				'project_name' => sanitize_text_field( get_the_title( (int) $project_id ) ),
				'image_name'   => sanitize_text_field( get_the_title( (int) $image_id ) ),
				'content'      => wpautop( $model['comment_content'] ),
				'link'         => ph_email_link( $url, __( 'View Comment', 'project-huddle' ) )
			] )
			->to( $email )
			->subject( get_option( 'ph_mockup_new_comment_email_subject', '{{commenter}} made a new comment on {{project_name}}.' ) )
			->send();
	}
}

/**
 * Email share comment message with link
 *
 * @since    1.0.0
 *
 * @param array  $address Email addresses
 * @param string $subject Subject link
 * @param string $message Message body
 * @param string $link    Link to share
 * @param object $user    User who shared the image
 */
function ph_email_share_comment( $address, $subject, $message, $link, $user, $data ) {
	// send the email
	$email = ( new PH_Mail )
		->template( ph_locate_template( 'email/mockup-share-thread-email.php' ), [
			'commenter'    => esc_html( $user['display_name'] ),
			'avatar'       => get_avatar( $user['user_email'], 42, '', '', array(
				'force_display' => true,
				'extra_attr'    => 'style="border-radius: 42px;"'
			) ),
			'email'        => sanitize_email( $user['user_email'] ),
			'project_name' => sanitize_text_field( get_the_title( $data['project_id'] ) ),
			'image_name'   => sanitize_text_field( get_the_title( $data['image_id'] ) ),
			'content'      => wpautop( $message ),
			'link'         => ph_email_link( $link, __( 'View Project', 'project-huddle' ) )
		] )
		->to( apply_filters( 'ph_mockup_share_thread_email_to', $address, $data ) )
		->subject( get_option( 'ph_mockup_share_thread_email_subject', $subject, $data ) )
		->send();
}

/**
 * Email share comment message with link
 *
 * @since    1.0.0
 *
 * @param array  $address Email addresses
 * @param string $subject Subject link
 * @param string $message Message body
 * @param string $link    Link to share
 * @param object $user    User who shared the image
 */
function ph_email_share_project( $address, $subject, $message, $link, $user, $data ) {
	// send the email
	$email = ( new PH_Mail )
		->template( ph_locate_template( 'email/mockup-share-project-email.php' ), [
			'commenter'    => esc_html( $user['display_name'] ),
			'avatar'       => get_avatar( $user['user_email'], 42, '', '', array(
				'force_display' => true,
				'extra_attr'    => 'style="border-radius: 42px;"'
			) ),
			'email'        => sanitize_email( $user['user_email'] ),
			'project_name' => sanitize_text_field( get_the_title( $data['project_id'] ) ),
			'image_name'   => sanitize_text_field( get_the_title( $data['image_id'] ) ),
			'content'      => wpautop( $message ),
			'link'         => ph_email_link( $link, __( 'View Project', 'project-huddle' ) )
		] )
		->to( apply_filters( 'ph_mockup_share_thread_email_to', $address, $data ) )
		->subject( get_option( 'ph_mockup_share_thread_email_subject', $subject, $data ) )
		->send();
}

/**
 * Email user they've been manually added to a project
 *
 * @since    1.3.1
 *
 * @param $new_emails array New project emails
 * @param $user       object Current user object
 * @param $post_id    int ID of project
 */
function ph_subscribe_user_admin_email( $new_emails = array(), $user = null, $post_id = 0 ) {
	if ( get_post_type( $post_id) != 'ph-project' ) {
		return;
	}

	$data = apply_filters( 'ph_subscribe_user_admin_email_data', array(
		'user'    => $user,
		'post_id' => $post_id
	) );

	// make sure we have emails
	if ( empty( $new_emails ) || ! $post_id || ! $user ) {
		return;
	}

	// project title and link
	$project_title = get_the_title( $post_id );
	$project_link  = get_the_permalink( $post_id );

	// email each new person
	foreach ( $new_emails as $email ) {
		// don't get your own email
		if ( $email == $user->user_email ) {
			continue;
		}

		// send the email
		$email = ( new PH_Mail )
			->template( ph_locate_template( 'email/mockup-subscribe-project-email.php' ), [
				'commenter'    => esc_html( $user->display_name ),
				'avatar'       => get_avatar( $user->user_email, 42, '', '', array(
					'force_display' => true,
					'extra_attr'    => 'style="border-radius: 42px;"'
				) ),
				'email'        => sanitize_email( $user->user_email ),
				'project_name' => sanitize_text_field( $project_title ),
				'link'         => ph_email_link( esc_url( $project_link ), __( 'View Project', 'project-huddle' ) )
			] )
			->to( apply_filters( 'ph_mockup_subscribe_project_email_to', $email ) )
			->subject( get_option( 'ph_mockup_subscribe_project_email_subject', 'You have been invited to collaborate on {{project_name}}.' ) )
			->send();
	}
}

add_action( 'ph_new_email_admin_subscribe', 'ph_subscribe_user_admin_email', 10, 3 );