<?php
/**
 * Comment Functions
 *
 * Functions related to comments
 *
 * @package     Project Huddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Custom function for Human Time Diff
 *
 * Allows translations to be possible
 *
 * @param        $from
 * @param string $to
 *
 * @return string|void
 */
function ph_human_time_diff( $from, $to = '' ) {

	// If to is empty, get current time
	if ( empty( $to ) ) {
		$to = time();
	}

	$since = '';

	// get difference
	$diff = (int) abs( $to -  $from );

	// check hours, minutes days
	if ( $diff <= HOUR_IN_SECONDS ) {
		$mins = round( $diff / MINUTE_IN_SECONDS );

		// if it's less than one minute
		if ( $mins <= 1 ) {
			$since = __('Now', 'project-huddle');
		} else {
			/* translators: min=minute */
			$since = sprintf( _n( '%sm', '%sm', $mins ), $mins );
		}
	} elseif ( ( $diff <= DAY_IN_SECONDS ) && ( $diff > HOUR_IN_SECONDS ) ) {
		$hours = round( $diff / HOUR_IN_SECONDS );
		if ( $hours <= 1 ) {
			$hours = 1;
		}
		$since = sprintf( _n( '%sh', '%sh', $hours ), $hours );
	} elseif ( $diff >= DAY_IN_SECONDS ) {
		$days = round( $diff / DAY_IN_SECONDS );
		if ( $days <= 1 ) {
			$days = 1;
		}
		$since = sprintf( _n( '%sd', '%sd', $days ), $days );
	}

	return $since;
}

/**
 * Hides image comments on dashboard and other areas
 *
 * @param $query array Original comment query
 *
 * @return array Query
 */
function hide_image_comments( $query ) {
	global $wp_version, $typenow, $pagenow, $ph_ajax_call;

	// always all comments on project/website pages
	if ( is_singular( 'ph-project' ) || $typenow == 'ph-project' || is_singular( 'ph-website' ) || $typenow == 'ph-website' ) {
		return $query;
	}

	// always show for ajax calls by projecthuddle plugin image
	if ( defined( 'DOING_AJAX' ) && DOING_AJAX ){
		if ( check_ajax_referer( 'project-image-nonce', 'nonce', false ) ) {
			return $query;
		}
	}

	// remove from query
	if ( version_compare( floatval( $wp_version ), '4.1', '>=' ) ) {
		$types = isset( $query->query_vars['type__not_in'] ) ? $query->query_vars['type__not_in'] : array();
		if ( ! is_array( $types ) ) {
			$types = array( $types );
		}

		// only show on our pages
		if ( ! isset( $query->query_vars['ph_force_show'] ) || ! $query->query_vars['ph_force_show'] ) {
			$types[] = 'ph_image_comment';
			$types[] = 'ph_website_comment';
		}

		$query->query_vars['type__not_in'] = $types;
	}
	return $query;
}
add_action( 'parse_comment_query', 'hide_image_comments', 10 );

/**
 * Remove reply and edit links from our custom comment
 *
 * @param array  $actions Comment action link array
 * @param object $comment Comment object
 *
 * @return mixed
 */
function ph_change_comment_links( $actions, $comment ) {
	if ( $comment->comment_type == 'ph_image_comment' || $comment->comment_type == 'ph_website_comment' ) {
		unset( $actions['reply'] );
		unset( $actions['edit'] );
	}

	return $actions;
}

add_filter( 'comment_row_actions', 'ph_change_comment_links', 10, 2 );

// add additional html tags for mockup comments
function ph_set_allowed_tags( $data ) {
	global $allowedtags;

	$allowedtags['br'] = array();
	$allowedtags['p']  = array();
	$allowedtags['ol'] = array();
	$allowedtags['ul'] = array();
	$allowedtags['li'] = array();

	return $data;
}
add_filter( 'ph_mockup_new_thread', 'ph_set_allowed_tags', 1 );
add_filter( 'ph_mockup_edit_thread', 'ph_set_allowed_tags', 1 );
add_filter( 'ph_mockup_delete_thread', 'ph_set_allowed_tags', 1 );
add_filter( 'ph_mockup_new_comment', 'ph_set_allowed_tags', 1 );
add_filter( 'ph_mockup_edit_comment', 'ph_set_allowed_tags', 1 );
add_filter( 'ph_mockup_delete_comment', 'ph_set_allowed_tags', 1 );