<?php
/**
 * Permission functions for ProjectHuddle
 *
 * @since 2.3.1
 */

/**
 * Run a permissions check on our custom post types
 * to make sure they have access
 */
function ph_user_has_member_access() {
	global $post;

	if ( is_admin() ) {
		return;
	}

	// check to make sure unsilo isn't checked
	if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
		return;
	}

	$get_post = isset( $_GET['post'] ) ? $_GET['post'] : false;

	// if not our post type
	if ( ! $post || ! in_array( get_post_type( $get_post ), ph_get_post_types() ) ) {
		return;
	}

	// allow on front end if permssions allow
	if ( get_post_meta( $post->ID, 'ph_project_access', true ) != 'login' ) {
		return;
	}

	// if user isn't logged in, do default project permission
	if ( ! is_user_logged_in() ) {
		return;
	}

	// check if member or should have access otherwise
	if ( ! ph_user_is_member( $post ) ) :
		wp_die( __( 'You don\'t have permission to access this project. Please contact the site owner to gain permissions.', 'project-huddle' ), 'ph-project-permissions-check', array( 'back_link' => true ) );
	endif;
}

add_action( 'wp', 'ph_user_has_member_access' );

function ph_user_has_member_access_admin() {
	global $pagenow;

	// check to make sure unsilo isn't checked
	if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
		return;
	}

	// get post object
	$post = isset( $_GET['post'] ) ? get_post( $_GET['post'] ) : false;

	// we're not on the edit post page
	if ( $pagenow != 'post.php' || ! isset( $post ) ) {
		return;
	}

	// we're not on one of our post types
	if ( ! in_array( get_post_type( $post ), ph_get_post_types() ) ) {
		return;
	}

	// check if member or should have access otherwise
	if ( ! ph_user_is_member( $post ) ) :
		wp_die( __( 'You don\'t have permission to access this project. Please contact the site owner to gain permissions.', 'project-huddle' ), 'ph-project-permissions-check', array( 'back_link' => true ) );
	endif;
}

add_action( 'admin_init', 'ph_user_has_member_access_admin' );

/**
 * Restricts All Mockups page to show only subscribed projects
 *
 * @param $query
 *
 * @return mixed
 */
function ph_restrict_for_current_author( $query ) {
	// check option
	if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
		return $query;
	}

	global $pagenow, $typenow;

	// need to be on all posts page and admin
	if ( 'edit.php' != $pagenow || ! $query->is_admin ) {
		return $query;
	}

	// check against post types
	if ( ! in_array( $typenow, ph_get_post_types() ) ) {
		return $query;
	}

	// can view unsubscribed projects
	foreach ( ph_get_post_types() as $type ) {
		if ( $type == $typenow ) {
			if ( current_user_can( 'view_unsubscribed_' . $type . 's' ) ) {
				return $query;
			}
		}
	}

	// if it's a project collaborator and the user is not the post author
	global $user_ID;
	// get user
	$user = get_userdata( $user_ID );

	$query->set( 'meta_query', array(
		array(
			'key'     => 'ph_project_emails_enable',
			'value'   => $user->user_email,
			'compare' => 'LIKE',
		)
	) );

	return $query;
}

add_filter( 'pre_get_posts', 'ph_restrict_for_current_author' );

if ( ! function_exists( 'ph_is_user_subscribed' ) ) :
	/**
	 * Is the user subscribed?
	 * @var $user int|object User object or integer
	 *
	 * @return boolean If the user is subscribed to the project
	 */
	function ph_is_user_subscribed( $user = 0, $post_object = 0 ) {
		// normalize user
		if ( $user ) {
			if ( is_int( $user ) ) {
				$user = get_user_by( 'ID', $user );
			} else if ( ! is_object( $user ) ) {
				return false;
			}
		} else {
			$user = wp_get_current_user();
		}

		global $post;
		$post = $post_object ? $post_object : $post;

		$subscribed = (array) get_post_meta( $post->ID, 'ph_project_emails_enable', true );

		return in_array( $user->user_email, $subscribed );
	}
endif;

if ( ! function_exists( 'ph_user_is_member' ) ) :
	/**
	 * Checks if a user is a member of a specific project
	 *
	 * @param $post int|object Post ID
	 *
	 * @return bool
	 */
	function ph_user_is_member( $post ) {
		// if unsilo is on
		if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
			return true;
		}

		// normalize post object
		if ( is_int( $post ) ) {
			$post = get_post( $post );
		}

		$user = wp_get_current_user();

		// true if subscribed, no matter the role
		if ( ph_is_user_subscribed( $user, $post ) ) {
			return true;
		}

		// otherwise true if current user can view unsubscribed projects or they are the author
		return current_user_can( 'edit_ph-project', $post->ID ) || $post->post_author == get_current_user_id();
	}
endif;

if ( ! function_exists( 'ph_visitor_can_access' ) ) :
	/**
	 * Can the visitor access the website project
	 *
	 * @param int    $post_id      Project ID
	 * @param string $access_token Unique private access token for the project
	 *
	 * @return bool|WP_Error True if visitor should have access
	 */
	function ph_visitor_can_access( $access_token = '', $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}

		// switch project access
		switch ( ph_get_project_access( $post_id ) ) {
			case 'public':
				return true;
				break;
			case 'link':
				// check access token
				if ( $access_token === get_post_meta( $post_id, 'access_token', true ) ) {
					return true;
				} else {
					if ( is_user_logged_in() && ph_user_is_member( $post_id ) ) {
						return true;
					} else {
						return new WP_Error( 'token_incorrect_missing', __( 'The access link provided is incorrect. Please check to make sure you copied it correctly!', 'project-huddle' ), array( 'status' => 403 ) );
					}
				}
				break;
			default:
				if ( is_user_logged_in() && ph_user_is_member( $post_id ) ) {
					return true;
				} elseif ( is_user_logged_in() && ! ph_user_is_member( $post_id ) ) {
					return new WP_Error( 'not_member', __( 'You must be a member of this project to access it.', 'project-huddle' ), array( 'status' => 403 ) );
				} else {
					ph_get_template_part( 'content', 'login' );
				}
				break;
		}
	}
endif;

if ( ! function_exists( 'ph_user_validate_access' ) ) :
	/**
	 * This function processes user access
	 * Adds the user to the project if they have access
	 *
	 * @param int $post_id Project ID
	 *
	 * @return bool|WP_Error
	 */
	function ph_user_validate_access( $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}

		$access_token = isset( $_GET['access_token'] ) ? $_GET['access_token'] : '';

		// store access token in cookie
		if ( isset( $access_token ) ) {
			PH()->session->set( 'website_access', array(
				(int) $post_id => $access_token
			) );
		}

		// does the visitor have access
		$has_access = ph_visitor_can_access( $access_token, (int) $post_id );

		// add the project member if they have access
		if ( ! is_wp_error( $has_access ) && $has_access && $current_user = wp_get_current_user() ) {
			ph_add_project_member( (int) $post_id, $current_user->user_email );
		}

		return $has_access;
	}
endif;

if ( ! function_exists( 'ph_is_token_valid' ) ) :
	/**
	 * Checks if an access token is valid
	 *
	 * @param string  $token   Access Token
	 * @param integer $post_id Project ID
	 *
	 * @return boolean
	 */
	function ph_is_token_valid( $token = '', $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}

		return $token === get_post_meta( $post_id, 'access_token', true );
	}
endif;

if ( ! function_exists( 'ph_get_access_token' ) ) :
	/**
	 * Get the access token for a website project
	 *
	 * @param $post mixed Post
	 *
	 * @return string|WP_Error
	 */
	function ph_get_access_token( $post ) {
		// get post id
		if ( is_a( $post, 'WP_Post' ) ) {
			$post_id = $post->ID;
		} else if ( is_array( $post ) && ! empty( $post['id'] ) ) {
			$post_id = $post['id'];
		} else if ( is_int( $post ) ) {
			$post_id = $post;
		}

		// get session
		$access = PH()->session->get( 'website_access' );
		$token  = ! empty( $access[ $post_id ] ) ? $access[ $post_id ] : '';

		// does the visitor have access
		$has_access = ph_visitor_can_access( $token, (int) $post_id );

		// return the access token if they have access
		if ( ! is_wp_error( $has_access ) && $has_access ) {
			return get_post_meta( $post_id, 'access_token', true ) ? get_post_meta( $post_id, 'access_token', true ) : true;
		} else {
			return $has_access;
		}
	}
endif;

if ( ! function_exists( 'ph_get_project_access' ) ) :
    /**
     * ph_get_project_access
     */
    function ph_get_project_access( $post_id = 0) {
	    if ( ! $post_id ) {
		    global $post;
		    $post_id = is_object( $post ) ? $post->ID : false;
	    }

	    if ( ! $post_id ) {
		    return false;
	    }

	    return get_post_meta( $post_id, 'project_access', true );
    }
endif;

if ( ! function_exists( 'ph_should_script_load' ) ) :
    /**
     * ph_should_script_load
     */
    function ph_should_script_load() {
	    global $post, $wp;

	    if ( ! $post || !( $post instanceof WP_Post ) ){
		    return new WP_Error( 'invalid_post', __( 'This post doens\'t exist.', 'project-huddle' ), array( 'status' => 403 ) );
	    }

		// get comment and url
	    $url = get_post_meta( $post->ID, 'ph_website_url', true );

	    // check api matches something
	    if ( ! ph_public_api_matches( $wp->query_vars['ph_apikey'] ) ) {
		    return new WP_Error( 'wrong_api_key', __( 'Wrong API Key. Please check the code you pasted on the site to make sure it\'s correct.', 'project-huddle' ), array( 'status' => 403 ) );
	    }

	    // double check website domain
	    $referrer = isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : false;
	    if ( !$referrer ) {
		    return new WP_Error( 'wrong_website_domain', __( 'Website url is wrong in settings. Please double check the url to make sure it exactly matches the site!', 'project-huddle' ), array( 'status' => 403 ) );
	    }
	    if ( ! ph_domain_matches( $referrer, array( $url ) ) ) {
		    return new WP_Error( 'wrong_website_domain', __( 'Website url is wrong in settings. Please double check the url to make sure it exactly matches the site!', 'project-huddle' ) . ' Live site is: ' . esc_url( $_SERVER['HTTP_REFERER'] ) . '. You\'ve set: ' . esc_url( $url ) . '.', array( 'status' => 403 ) );
	    }

	    // installed
	    update_post_meta( get_the_ID(), 'ph_installed', true );
	    delete_post_meta( get_the_ID(), 'ph_installation_error' );

	    // true
	    return true;
    }
endif;

/**
 * Add member to project after they are created
 *
 * @param WP_User         $user    Inserted user object.
 * @param WP_REST_Request $request Request object.
 */
function ph_add_member_after_creating( $user, $request ) {
	if ( ! isset( $request['project_id'] ) || ! isset( $user->user_email ) ) {
		return;
	}

	// add member to project
	ph_add_project_member( (int) $request['project_id'], $user->user_email, false );
}

add_action( 'ph_rest_insert_user', 'ph_add_member_after_creating', 10, 2 );