<?php
/**
 * Post Type Functions
 *
 * @package     ProjectHuddle
 * @subpackage  Functions
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Registers and sets up the Downloads custom post type
 *
 * @since 1.0
 * @return void
 */
function ph_setup_post_types() {

	do_action( 'ph_before_register_post_type' );

	/** Project Post Type */
	$project_labels = array(
		'name' 				=> _x( 'Mockups', 'post type general name', 'project-huddle' ),
		'singular_name' 	=> _x( 'Mockup', 'post type singular name', 'project-huddle' ),
		'add_new' 			=> __( 'New Mockup', 'project-huddle' ),
		'add_new_item' 		=> __( 'Add New Mockup', 'project-huddle' ),
		'edit_item' 		=> __( 'Edit Mockup Project', 'project-huddle' ),
		'new_item' 			=> __( 'New Mockup Prject', 'project-huddle' ),
		'all_items' 		=> __( 'Mockups', 'project-huddle' ),
		'view_item' 		=> __( 'View Mockup Project', 'project-huddle' ),
		'search_items' 		=> __( 'Search Mockups', 'project-huddle' ),
		'not_found' 		=> __( 'No Mockups found', 'project-huddle' ),
		'not_found_in_trash'=> __( 'No Mockups found in Trash', 'project-huddle' ),
		'parent_item_colon' => '',
		'menu_name' 		=> __( 'ProjectHuddle', 'project-huddle' )
	);

	$project_args = array(
		'labels'              => apply_filters( 'ph_project_labels', $project_labels ),
		'public'              => true,
		'menu_icon'           => 'dashicons-testimonial',
		'publicly_queryable'  => true,
		'exclude_from_search' => true,
		'show_ui'             => true,
		'show_in_menu'        => true,
		'query_var'           => true,
		'rewrite'             => array(
			'slug'       => 'mockup',
			'with_front' => false // permalink structure shouldn't prepended with the front base
		),
		'capability_type'     => array( 'ph-project', 'ph-projects' ),
		'map_meta_cap'        => true,
		'hierarchical'        => false,
		'supports'            => apply_filters( 'ph_project_supports', array( 'title', 'author' ) ),
	);

	if ( defined( 'PROJECT_HUDDLE_DEBUG' ) && PROJECT_HUDDLE_DEBUG ) { $project_args['supports'][] = 'custom-fields'; }

	register_post_type( 'ph-project', apply_filters( 'ph_register_post_type_project', $project_args ) );

	/**
	 * Register project image
	 *
	 * Allows us to use a post relationship to store comment posts, thumbnail, versions, etc.
	 * @since 1.0
	 */
	register_post_type( 'project_image',
		apply_filters( 'ph_register_post_type_project_image',
			array(
				'label'               => __( 'Project Images', 'project-huddle' ),
				'public'              => true,
				'exclude_from_search' => true,
				'show_in_menu'        => ( defined( 'PROJECT_HUDDLE_DEBUG' ) && PROJECT_HUDDLE_DEBUG ) ? true : false,
				'hierarchical'        => true, // add hierarchy for versions
				'supports'            => array( 'title', 'excerpt', 'thumbnail', 'custom-fields', 'page-attributes' )
			)
		)
	);

	/**
	 * Register comment location post type
	 *
	 * Allows us to use a post relationship to store comment location meta data and have unique ids.
	 * @since 1.0
	 */
	register_post_type( 'ph_comment_location',
		apply_filters( 'ph_register_post_type_comment_location',
			array(
				'label'               => __( 'Comment Locations', 'project-huddle' ),
				'public'              => true,
				'exclude_from_search' => true,
				'show_in_menu'        => ( defined( 'PROJECT_HUDDLE_DEBUG' ) && PROJECT_HUDDLE_DEBUG ) ? true : false,
				'hierarchical'        => false, // non-hierarchical
				'taxonomies'          => array( 'ph_status' ),
				'supports'            => array( 'comments', 'custom-fields' ),
				'capability_type'     => array( 'ph_comment_location', 'ph_comment_locations' ),
				'map_meta_cap'        => true,
			)
		)
	);

	do_action( 'ph_after_register_post_type' );
}
add_action( 'init', 'ph_setup_post_types', 999 );

/**
 * Setup plugin taxonomies
 */
function ph_setup_taxonomies() {
	/**
	 * Creates the "Status" taxonomy.
	 * @since 1.0
	 */
	$labels = array(
		'name'              => _x( 'Available Statuses', 'taxonomy general name', 'framework' ),
		'singular_name'     => _x( 'Status', 'taxonomy singular name', 'framework' ),
		'search_items'      => __( 'Search Available Statuses', 'framework' ),
		'all_items'         => __( 'All Statuses', 'framework' ),
		'parent_item'       => __( 'Parent Status', 'framework' ),
		'parent_item_colon' => __( 'Parent Status:', 'framework' ),
		'edit_item'         => __( 'Edit Status', 'framework' ),
		'update_item'       => __( 'Update Status', 'framework' ),
		'add_new_item'      => __( 'Add New Status', 'framework' ),
		'new_item_name'     => __( 'New Status Name', 'framework' ),
		'menu_name'         => __( 'Statuses', 'framework' ),
	);

	register_taxonomy( 'ph_status',
		array( 'phw_comment_loc', 'ph_comment_location', 'ph-website' ),
		array(
			'hierarchical' => false,
			'labels'       => $labels,
			'show_ui'      => true,
			'query_var'    => true,
			'rewrite'      => array(
				'slug'       => 'status',
				'with_front' => false // permalink structure shouldn't prepended with the front base
			), // This is the url slug
		)
	);
}
add_action( 'init', 'ph_setup_taxonomies', 9999999 );

/**
 * Turn off comment moderation for our post type
 *
 * @param $value boolean Comment moderation on/off
 *
 * @return int Comment moderation value
 * @since 1.0
 */
function ph_disable_comment_moderation( $value ) {
	if ( get_post_type() == 'ph_comment_location' || get_post_type() == 'phw_comment_loc' ) {
		return 0;
	}

	return $value;
}
add_filter( 'option_comment_moderation', 'ph_disable_comment_moderation' );

/**
 * Remove comments meta box from ph_comment_location post
 */
function ph_remove_meta_boxes() {
	remove_meta_box( 'commentsdiv', 'ph_comment_location', 'normal' );
	remove_meta_box( 'commentstatusdiv', 'ph_comment_location', 'normal' );
}
add_action( 'admin_menu', 'ph_remove_meta_boxes' );

if ( ! function_exists( 'ph_get_post_types' ) ) :
    /**
     * Get post types for plugin
     */
    function ph_get_post_types() {
	    return apply_filters( 'ph_post_types', array( 'ph-project' ) );
    }
endif;

if ( ! function_exists( 'ph_get_child_post_types' ) ) :
	/**
	 * Get post types for plugin
	 */
	function ph_get_child_post_types() {
		return apply_filters( 'ph_child_post_types', array( 'ph_comment_location', 'project_image' ) );
	}
endif;