<?php
/**
 * Shortcodes
 *
 * @package     Project-Huddle
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Builds the Project Gallery shortcode output.
 *
 * This is based of the WordPress gallery shortcode functionality to match styling to the theme.
 *
 * @since 1.0.0
 *
 * @param array $attr {
 *     Attributes of the gallery shortcode.
 *
 *     @type int    $id         Project ID.
 *     @type bool   $titles     Show/Hide the project image titles
 *     @type string $itemtag    HTML tag to use for each image in the gallery.
 *                              Default 'dl', or 'figure' when the theme registers HTML5 gallery support.
 *     @type string $icontag    HTML tag to use for each image's icon.
 *                              Default 'dt', or 'div' when the theme registers HTML5 gallery support.
 *     @type string $captiontag HTML tag to use for each image's caption.
 *                              Default 'dd', or 'figcaption' when the theme registers HTML5 gallery support.
 *     @type int    $columns    Number of columns of images to display. Default 3.
 *     @type string $size       Size of the images to display. Default 'thumbnail'.
 * }
 * @return string HTML content to display gallery.
 */
function project_images_shortcode( $attr ) {

	// add shortcodes and scripts
	wp_enqueue_style( 'project-huddle-shortcodes' );
	wp_enqueue_script( 'ph-shortcodes' );

	// keep track of instances on page
	static $instance = 0;
	$instance ++;

	/**
	 * Filter the default project gallery shortcode output.
	 *
	 * If the filtered output isn't empty, it will be used instead of generating
	 * the default gallery template.
	 *
	 * @since 1.0.0
	 *
	 * @see project_images_shortcode()
	 *
	 * @param string $output   The gallery output. Default empty.
	 * @param array  $attr     Attributes of the gallery shortcode.
	 * @param int    $instance Unique numeric ID of this gallery shortcode instance.
	 */
	$output = apply_filters( 'ph_project_thumbnails_shortcode', '', $attr, $instance );
	if ( $output != '' ) {
		return $output;
	}

	// find html5 support
	$html5 = current_theme_supports( 'html5', 'gallery' );

	// get attributes
	$atts  = shortcode_atts( array(
		'id'               => false,
		'titles'           => false,
		'itemtag'          => $html5 ? 'figure' : 'dl',
		'icontag'          => $html5 ? 'div' : 'dt',
		'captiontag'       => $html5 ? 'figcaption' : 'dd',
		'columns'          => 3,
		'target'           => '_blank',
		'multisite'        => false,
		'size'             => 'thumbnail',
	), $attr, 'project_huddle' );

	// sanitize project id
	$id = intval( $atts['id'] );

	// return if no project id
	if ( ! $atts['id'] ) {
		return false;
	}

	if ( $atts['multisite'] && is_multisite() && defined( 'BLOG_ID_CURRENT_SITE' ) ) {
		$blog_id = get_current_blog_id();
		switch_to_blog( BLOG_ID_CURRENT_SITE );
	}

	// if project is password protected
	if ( post_password_required( $id ) ) {
		echo get_the_password_form();
		return false;
	}

	// if project is private or unpublished && user isn't logged in
	if ( get_post_status($id) != 'publish' && ! is_user_logged_in() ) {
		echo wp_login_form();
		return false;
	}

	// if tags are manually entered, validate and escape
	$itemtag    = tag_escape( $atts['itemtag'] );
	$captiontag = tag_escape( $atts['captiontag'] );
	$icontag    = tag_escape( $atts['icontag'] );
	$valid_tags = wp_kses_allowed_html( 'post' );
	if ( ! isset( $valid_tags[ $itemtag ] ) ) {
		$itemtag = 'dl';
	}
	if ( ! isset( $valid_tags[ $captiontag ] ) ) {
		$captiontag = 'dd';
	}
	if ( ! isset( $valid_tags[ $icontag ] ) ) {
		$icontag = 'dt';
	}

	// get columns
	$columns = intval( $atts['columns'] );
	$itemwidth = $columns > 0 ? floor(100/$columns) : 100;
	$float = is_rtl() ? 'right' : 'left';

	$selector = "ph-project-gallery-{$instance}";

	$gallery_style = '';

	/**
	 * Filter whether to print default gallery styles.
	 *
	 * @since 1.1.1
	 *
	 * @param bool $print Whether to print default gallery styles.
	 *                    Defaults to false if the theme supports HTML5 galleries.
	 *                    Otherwise, defaults to true.
	 */
	if ( apply_filters( 'ph_use_default_project_gallery_style', ! $html5 ) ) {
		$gallery_style = "
            <style type='text/css'>
                    #{$selector} {
                            margin: auto;
                    }
                    #{$selector} .gallery-item {
                            float: {$float};
                            margin-top: 10px;
                            text-align: center;
                            width: {$itemwidth}%;
                    }
                    #{$selector} img {
                            border: 2px solid #cfcfcf;
                    }
            </style>\n\t\t";
	}

	$size_class  = sanitize_html_class( $atts['size'] );
	$gallery_div = "<div id='$selector' class='ph-project-gallery gallery galleryid-{$id} gallery-columns-{$columns} gallery-size-{$size_class}'>";

	/**
	 * Filter the default gallery shortcode CSS styles.
	 *
	 * @since 1.0.0
	 *
	 * @param string $gallery_style Default CSS styles and opening HTML div container
	 *                              for the gallery shortcode output.
	 */
	$output = apply_filters( 'ph_project_gallery_style', $gallery_style . $gallery_div );

	$url = get_permalink( (int) $id );

	// get images by project id
	$project_images = PH()->project->images( (int) $id );

	$i = 0;

	foreach ( $project_images as $image ) {

		// get image
		$thumb      = wp_get_attachment_image( $image['imageID'], $atts['size'], false, $attr );
		$image_meta = wp_get_attachment_metadata( $image['imageID'] );

		$orientation = '';
		if ( isset( $image_meta['height'], $image_meta['width'] ) ) {
			$orientation = ( $image_meta['height'] > $image_meta['width'] ) ? 'portrait' : 'landscape';
		}
		$output .= "<{$itemtag} class='gallery-item'>";

		$output .= "
	        <{$icontag} class='gallery-icon {$orientation}'>
	            <a target='" . $atts['target'] . "' href='" . $url ."#" . $image['hash'] . "' class='ph-shortcode-image-link' >
		            $thumb
		        </a>
	        </{$icontag}>";

		if ( $captiontag ) {
			if ( apply_filters( 'shortcode_show_title_project_huddle', $atts['titles'] && trim( $image['title'] ), $atts ) ) {
				$output .= "<{$captiontag} class='wp-caption-text gallery-caption' id='$selector-$id'>";
				if ( $atts['titles'] ) {
					$output .= wptexturize( $image['title'] );
				}
				$output .= apply_filters( 'shortcode_title_project_huddle', $atts, $image );
				$output .= "</{$captiontag}>";
			}
		}

		$output .= "</{$itemtag}>";
		if ( ! $html5 && $columns > 0 && ++ $i % $columns == 0 ) {
			$output .= '<br style="clear: both" />';
		}
	}

	if ( ! $html5 && $columns > 0 && $i % $columns !== 0 ) {
		$output .= "
	    		<br style='clear: both' />";
	}

	$output .= "
		</div>\n";

	if ( is_multisite() && isset( $blog_id ) ) {
		switch_to_blog( $blog_id );
	}

	return $output;
}
add_shortcode( 'project_huddle', 'project_images_shortcode' );

/**
 * User projects shortcode
 *
 * @param $attr
 *
 * @return bool|string
 */
function ph_user_projects( $attr ) {
	// add shortcodes and scripts
	wp_enqueue_style( 'project-huddle-shortcodes' );
	wp_enqueue_script( 'ph-shortcodes' );

	// get attributes
	$html5 = current_theme_supports( 'html5', 'gallery' );
	$atts = shortcode_atts( array(
		'format'        => 'list', // list or gallery
		'titles'        => false,
		'project_title' => true,
		'mockups'       => true,
		'itemtag'       => $html5 ? 'figure' : 'dl',
		'icontag'       => $html5 ? 'div' : 'dt',
		'captiontag'    => $html5 ? 'figcaption' : 'dd',
		'columns'       => 3,
		'multisite'     => false,
		'limit'         => - 1,
		'target'        => '_blank',
		'size'          => 'thumbnail',
	), $attr, 'ph_subscribed_projects' );

	$atts['mockups'] = filter_var( $atts['mockups'], FILTER_VALIDATE_BOOLEAN );
	$atts['multisite'] = filter_var( $atts['multisite'], FILTER_VALIDATE_BOOLEAN );

	// get current user
	$current_user = wp_get_current_user();

	// output start
	$output = '';

	// must be logged in or don't do anything
	if ( 0 == $current_user->ID ) {
		return false;
	}

	if ( $atts['multisite'] && is_multisite() && defined( 'BLOG_ID_CURRENT_SITE' ) ) {
		$blog_id = get_current_blog_id();
		switch_to_blog( BLOG_ID_CURRENT_SITE );
	}

	// use paged, unless we're on the homepage
	$mockup_page = isset( $_GET['mockup_page'] ) ? (int) $_GET['mockup_page'] : 1;

	// query by enabled emails meta
	$args = apply_filters( 'ph_subscribed_projects_args', array(
		'post_type'      => 'ph-project',
		'posts_per_page' => $atts['limit'],
		'paged'          => $mockup_page,
		'meta_query'     => array(
			array(
				'key'     => 'ph_project_emails_enable',
				'value'   => $current_user->user_email,
				'compare' => 'LIKE',
			)
		)
	), $current_user );

	// The Query
	$mockup_query = new WP_Query( $args );

	// The Loop
	if ( $mockup_query->have_posts() &&  $atts['mockups'] ) {

		// gallery or list format
		if ( $atts['format'] == 'gallery' ) {
			while ( $mockup_query->have_posts() ) {
				$mockup_query->the_post();

				// find if all are approved
				$all_approved = PH_Approvals()->project->is_approved( get_the_ID() );

				if ( $atts['project_title'] ) {

					// filter title tag
					$title_tag = apply_filters( 'ph_subscribed_projects_shortcode_title_tag', 'h3', get_the_ID() );

					// title tag
					$output .= "<$title_tag>";

					// title
					$output .= apply_filters('ph_user_projects_shortcode_title', esc_html( ph_get_the_title( get_the_ID() ) ) );

					// approvals/unapprovals
					if ( $atts['show_approvals'] ) {
						if ( $all_approved ) {
							$output .= ' <small class="ph-approved" style="color:' . get_option( 'ph_approve_color', '#7bae54' ) . '; ">' . __( 'Approved', 'project-huddle' ) . '</small>';
						}
					}
					if ( $atts['show_unapprovals'] ) {
						if ( ! $all_approved ) {
							$output .= ' <small class="ph-unapproved" style="color:' . get_option( 'ph_unapprove_color', '#6F7C8A' ) . '; ">' . __( 'Unapproved', 'project-huddle' ) . '</small>';
						}
					}

					$output .= "</$title_tag>";
				}

				// start shortcode with id
				$shortcode = '[project_huddle id=' . get_the_ID() . ' ';

				// add attributes
				foreach ( $atts as $att => $value ) {
					if ( isset( $value ) ) {
						$shortcode .= $att . '=' . $value . ' ';
					}
				}

				// end shortcode
				$shortcode .= ']';

				// output shortcode
				$output .= do_shortcode( esc_html( $shortcode ) );
			}
		} else {
			$output = '<ul class="ph-subscribed-projects-list">';
			while ( $mockup_query->have_posts() ) {
				$mockup_query->the_post();

				$output .= '<li><a target="' . $atts['target'] . '" href="' . esc_url( get_the_permalink() ) . '">';
				$output .= apply_filters( 'ph_subscribed_project_list_title', esc_html( ph_get_the_title( get_the_ID() ) ), get_the_ID(), $atts );
				$output .= '</a></li>';
			}
			$output .= '</ul>';
		}

		// only display if the max number of pages is greater than 1
		if ( $mockup_query->max_num_pages > 1 ) :
			$big = 999999999; // need an unlikely integer

			// default permalinks
			$base = str_replace( 'paged=' . $big, 'mockup_page=%#%', esc_url( get_pagenum_link( $big ) ) );
			// pretty permalinks
			$base = str_replace( 'page/' . $big, '?mockup_page=%#%', $base );
			$base = preg_replace( '/\?\bmockup_page=(.*)\b/', '?mockup_page=%#%', $base );

			$output .= paginate_links( array(
				'base'     => $base,
				'current'  => max( 1, $mockup_page ),
				'total'    => $mockup_query->max_num_pages,
				'add_args' => false
			) );
		endif;

	} elseif ( $atts['mockups']) {
		$output .= apply_filters( 'ph_no_projects_shortcode_text', __( 'You have not been added to any projects.', 'project-huddle' ) );
	}

	// after hook
	do_action( 'ph_subscribed_projects_shortcode_after', $args );

	if ( is_multisite() && isset( $blog_id ) ) {
		switch_to_blog( $blog_id );
	}

	// Restore original Post Data
	wp_reset_postdata();

	return apply_filters( 'ph_subscribed_projects_output', $output, $atts );
}

add_shortcode( 'ph_subscribed_projects', 'ph_user_projects' );